@@ -15,15 +15,13 @@
 
 class IUWTDeconvolution : public DeconvolutionAlgorithm {
  public:
-  IUWTDeconvolution(class FFTWManager& fftwManager)
-      : _fftwManager(fftwManager), _useSNRTest(false) {}
+  IUWTDeconvolution() : _useSNRTest(false) {}
 
-  virtual float ExecuteMajorIteration(
-      ImageSet& dataImage, ImageSet& modelImage,
-      const aocommon::UVector<const float*>& psfImages, size_t width,
-      size_t height, bool& reachedMajorThreshold) final override {
+  float ExecuteMajorIteration(ImageSet& dataImage, ImageSet& modelImage,
+                              const std::vector<aocommon::Image>& psfImages,
+                              bool& reachedMajorThreshold) final override {
     IUWTDeconvolutionAlgorithm algorithm(
-        _fftwManager, width, height, _gain, _mGain, _cleanBorderRatio,
+        dataImage.Width(), dataImage.Height(), _gain, _mGain, _cleanBorderRatio,
         _allowNegativeComponents, _cleanMask, _threshold, _useSNRTest);
     float val = algorithm.PerformMajorIteration(
         _iterationNumber, MaxNIter(), modelImage, dataImage, psfImages,
@@ -33,13 +31,12 @@ class IUWTDeconvolution : public DeconvolutionAlgorithm {
   }
 
   std::unique_ptr<DeconvolutionAlgorithm> Clone() const final override {
-    return std::unique_ptr<IUWTDeconvolution>(new IUWTDeconvolution(*this));
+    return std::make_unique<IUWTDeconvolution>(*this);
   }
 
   void SetUseSNRTest(bool useSNRTest) { _useSNRTest = useSNRTest; }
 
  private:
-  class FFTWManager& _fftwManager;
   bool _useSNRTest;
 };
 
