@@ -5,14 +5,16 @@
 #include "../main/commandline.h"
 #include "../main/wsclean.h"
 
-#include "../io/logger.h"
-#include "../system/check_openblas_multithreading.h"
+#include <aocommon/checkblas.h>
+#include <aocommon/logger.h>
 
 #include <exception>
 #include <iostream>
 
 #include <mpi.h>
 
+using aocommon::Logger;
+
 int main(int argc, char* argv[]) {
   int provided;
   MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &provided);
