@@ -1,19 +1,20 @@
-#ifndef FITSWRITER_H
-#define FITSWRITER_H
-
-#include <fitsio.h>
-
-#include <array>
-#include <string>
-#include <vector>
-#include <map>
-#include <cmath>
+#ifndef AOCOMMON_FITS_FITSWRITER_H_
+#define AOCOMMON_FITS_FITSWRITER_H_
 
 #include "fitsbase.h"
 #include "fitsreader.h"
 #include "../polarization.h"
 #include "../uvector.h"
 
+#include <array>
+#include <cmath>
+#include <limits>
+#include <map>
+#include <string>
+#include <vector>
+
+#include <fitsio.h>
+
 namespace aocommon {
 
 /// Requires the fitsio library.
@@ -114,7 +115,7 @@ class FitsWriter : public FitsBase {
   }
 
   void StartMulti(const std::string& filename) {
-    if (_multiFPtr != 0)
+    if (_multiFPtr != nullptr)
       throw std::runtime_error(
           "StartMulti() called twice without calling FinishMulti()");
     _multiFilename = filename;
@@ -124,7 +125,7 @@ class FitsWriter : public FitsBase {
 
   template <typename NumType>
   void AddToMulti(const NumType* image) {
-    if (_multiFPtr == 0)
+    if (_multiFPtr == nullptr)
       throw std::runtime_error("AddToMulti() called before StartMulti()");
     writeImage(_multiFPtr, _multiFilename, image, _currentPixel.data());
     size_t index = 2;
@@ -141,7 +142,7 @@ class FitsWriter : public FitsBase {
     int status = 0;
     fits_close_file(_multiFPtr, &status);
     checkStatus(status, _multiFilename);
-    _multiFPtr = 0;
+    _multiFPtr = nullptr;
   }
 
   void SetBeamInfo(double widthRad) { SetBeamInfo(widthRad, widthRad, 0.0); }
@@ -603,6 +604,7 @@ class FitsWriter : public FitsBase {
               break;
             case aocommon::Polarization::FullStokes:
             case aocommon::Polarization::Instrumental:
+            case aocommon::Polarization::DiagonalInstrumental:
               throw std::runtime_error(
                   "Incorrect polarization given to fits writer");
           }
