@@ -7,24 +7,13 @@
 
 #include <aocommon/io/serialistream.h>
 #include <aocommon/io/serialostream.h>
+#include <aocommon/imagecoordinates.h>
 
 using schaapcommon::facets::BoundingBox;
 using schaapcommon::facets::Coord;
 using schaapcommon::facets::Facet;
 using schaapcommon::facets::Pixel;
 
-namespace schaapcommon {
-namespace facets {
-
-// Implements the definition from tfacet.h.
-std::ostream& operator<<(std::ostream& stream, const Pixel& pixel) {
-  stream << "Pixel(" << pixel.x << ", " << pixel.y << ")";
-  return stream;
-}
-
-}  // namespace facets
-}  // namespace schaapcommon
-
 namespace {
 const double kScale = 0.001;
 const size_t kImageSize = 100;
@@ -32,7 +21,8 @@ const size_t kImageSize = 100;
 /**
  * Creates a facet with a diamond shape.
  */
-Facet CreateDiamondFacet(int offset_x = 0, int offset_y = 0) {
+Facet CreateDiamondFacet(int offset_x = 0, int offset_y = 0,
+                         bool set_custom_direction = false) {
   Facet facet;
 
   const std::vector<Coord> coords{
@@ -45,6 +35,10 @@ Facet CreateDiamondFacet(int offset_x = 0, int offset_y = 0) {
   const double dec = -offset_y * kScale;
   const double l = 0.0;
   const double m = 0.0;
+  if (set_custom_direction) {
+    facet.SetRA(-0.02);
+    facet.SetDec(0.01);
+  }
   facet.CalculatePixels(ra, dec, kScale, kScale, kImageSize, kImageSize, l, m);
 
   return facet;
@@ -243,10 +237,12 @@ BOOST_AUTO_TEST_CASE(polygon_two_intersections) {
 BOOST_AUTO_TEST_CASE(constructor) {
   Facet facet;
 
-  BOOST_CHECK_EQUAL(facet.RA(), 0.0);
-  BOOST_CHECK_EQUAL(facet.Dec(), 0.0);
+  // BOOST_CHECK_EQUAL(facet.RA(), );
+  BOOST_CHECK(std::isnan(facet.RA()));
+  // BOOST_CHECK_EQUAL(facet.Dec(), 0.0);
+  BOOST_CHECK(std::isnan(facet.Dec()));
   BOOST_CHECK(facet.GetPixels().empty());
-  BOOST_CHECK(facet.Direction().empty());
+  BOOST_CHECK(facet.DirectionLabel().empty());
 
   CheckBoundingBoxes(facet, Pixel(0, 0), Pixel(0, 0), Pixel(0, 0), Pixel(0, 0));
   BOOST_CHECK_THROW(facet.CalculatePixels(0.0, 0.0, kScale, kScale, kImageSize,
@@ -309,11 +305,19 @@ BOOST_AUTO_TEST_CASE(no_clipping) {
   BOOST_CHECK_EQUAL(pixels[1], Pixel(50, 60));
   BOOST_CHECK_EQUAL(pixels[2], Pixel(70, 50));
   BOOST_CHECK_EQUAL(pixels[3], Pixel(50, 40));
+
+  // Check facet centroid
+  const Pixel centroid = facet.Centroid();
+  BOOST_CHECK_EQUAL(centroid, Pixel(50, 50));
+  BOOST_CHECK_EQUAL(facet.RA(), 0.0);
+  BOOST_CHECK_EQUAL(facet.Dec(), 0.0);
 }
 
 // Create a diamond in the top right corner of the image.
 BOOST_AUTO_TEST_CASE(clip_top_right) {
-  Facet facet = CreateDiamondFacet(50, 50);
+  const int offset_x = 50;
+  const int offset_y = offset_x;
+  Facet facet = CreateDiamondFacet(offset_x, offset_y);
 
   const std::vector<Pixel>& pixels = facet.GetPixels();
   // Facet clipped to triangle
@@ -321,11 +325,31 @@ BOOST_AUTO_TEST_CASE(clip_top_right) {
   BOOST_CHECK_EQUAL(pixels[0], Pixel(100, 90));
   BOOST_CHECK_EQUAL(pixels[1], Pixel(80, 100));
   BOOST_CHECK_EQUAL(pixels[2], Pixel(100, 100));
+  Pixel centroid_ref((pixels[0].x + pixels[1].x + pixels[2].x) / 3,
+                     (pixels[0].y + pixels[1].y + pixels[2].y) / 3);
+  BOOST_CHECK_EQUAL(facet.Centroid(), centroid_ref);
+
+  // Manually convert centroid to ra,dec coords, and check
+  const double phase_centre_ra = offset_x * kScale;
+  const double phase_centre_dec = -offset_y * kScale;
+  double l;
+  double m;
+  double ra;
+  double dec;
+  aocommon::ImageCoordinates::XYToLM(facet.Centroid().x, facet.Centroid().y,
+                                     kScale, kScale, kImageSize, kImageSize, l,
+                                     m);
+
+  aocommon::ImageCoordinates::LMToRaDec(l, m, phase_centre_ra, phase_centre_dec,
+                                        ra, dec);
+  BOOST_CHECK_CLOSE(facet.RA(), ra, 1e-6);
+  BOOST_CHECK_CLOSE(facet.Dec(), dec, 1e-6);
 }
 
 // Create a diamond in the bottom left corner of the image.
 BOOST_AUTO_TEST_CASE(clip_bottom_left) {
-  Facet facet = CreateDiamondFacet(-50, -50);
+  const bool set_custom_direction = true;
+  Facet facet = CreateDiamondFacet(-50, -50, set_custom_direction);
 
   const std::vector<Pixel>& pixels = facet.GetPixels();
   // Facet clipped to triangle
@@ -333,6 +357,12 @@ BOOST_AUTO_TEST_CASE(clip_bottom_left) {
   BOOST_CHECK_EQUAL(pixels[0], Pixel(0, 10));
   BOOST_CHECK_EQUAL(pixels[1], Pixel(20, 0));
   BOOST_CHECK_EQUAL(pixels[2], Pixel(0, 0));
+  Pixel centroid_ref((pixels[0].x + pixels[1].x + pixels[2].x) / 3,
+                     (pixels[0].y + pixels[1].y + pixels[2].y) / 3);
+  BOOST_CHECK_EQUAL(facet.Centroid(), centroid_ref);
+  // Custom (ra, dec) direction reproduced?
+  BOOST_CHECK_EQUAL(facet.RA(), -0.02);
+  BOOST_CHECK_EQUAL(facet.Dec(), 0.01);
 }
 
 BOOST_AUTO_TEST_CASE(point_in_polygon) {
@@ -474,6 +504,7 @@ BOOST_AUTO_TEST_CASE(horizontal_intersections_concave_facet) {
 
   // Empty at top
   CheckIntersections(facet, 60, {});
+  BOOST_CHECK_EQUAL(facet.Centroid(), Pixel(50, 46));
 #endif
 }
 
@@ -676,9 +707,11 @@ BOOST_AUTO_TEST_CASE(serialization) {
   // Create a facet with different bounding boxes. See facet_bounding_boxes
   // test.
   Facet input = CreateRectangularFacet(1.5, 25, true);
+  BOOST_CHECK_EQUAL(input.RA(), 0.0);
+  BOOST_CHECK_EQUAL(input.Dec(), 0.0);
   input.SetRA(kRa);
   input.SetDec(kDec);
-  input.SetDirection(kDirection);
+  input.SetDirectionLabel(kDirection);
   aocommon::SerialOStream ostr;
   input.Serialize(ostr);
 
@@ -695,7 +728,8 @@ BOOST_AUTO_TEST_CASE(serialization) {
   }
   BOOST_CHECK_EQUAL(output.RA(), kRa);
   BOOST_CHECK_EQUAL(output.Dec(), kDec);
-  BOOST_CHECK_EQUAL(output.Direction(), kDirection);
+  BOOST_CHECK_EQUAL(input.Centroid(), output.Centroid());
+  BOOST_CHECK_EQUAL(output.DirectionLabel(), kDirection);
   BOOST_CHECK_EQUAL(input.GetTrimmedBoundingBox().Min(),
                     output.GetTrimmedBoundingBox().Min());
   BOOST_CHECK_EQUAL(input.GetTrimmedBoundingBox().Max(),
