@@ -3,12 +3,13 @@
 
 #include "gridmode.h"
 
+#include <aocommon/banddata.h>
+#include <aocommon/image.h>
 #include <aocommon/polarization.h>
 #include <aocommon/imagecoordinates.h>
 
 #include "../structures/observationinfo.h"
 #include "../structures/msselection.h"
-#include "../structures/image.h"
 #include "../structures/weightmode.h"
 
 #include "visibilityweightingmode.h"
@@ -18,9 +19,9 @@
 #include "../scheduling/metadatacache.h"
 #include "../scheduling/griddingtaskmanager.h"
 
-#include "../structures/multibanddata.h"
-
 #ifdef HAVE_EVERYBEAM
+#include <EveryBeam/beammode.h>
+#include <EveryBeam/beamnormalisationmode.h>
 #include <EveryBeam/pointresponse/pointresponse.h>
 #endif
 
@@ -53,6 +54,7 @@ class MSGridderBase {
 
   size_t ImageWidth() const { return _imageWidth; }
   size_t ImageHeight() const { return _imageHeight; }
+  double ImagePadding() const { return _imagePadding; }
   double PixelSizeX() const { return _pixelSizeX; }
   double PixelSizeY() const { return _pixelSizeY; }
   size_t ActualWGridSize() const { return _actualWGridSize; }
@@ -80,7 +82,7 @@ class MSGridderBase {
   bool SmallInversion() const { return _smallInversion; }
   aocommon::PolarizationEnum Polarization() const { return _polarization; }
   WeightMode Weighting() const { return _weighting; }
-  const class ImageWeights* GetImageWeights() const {
+  const ImageWeights* GetImageWeights() const {
     return _precalculatedWeightInfo;
   }
   bool IsComplex() const { return _isComplex; }
@@ -106,6 +108,7 @@ class MSGridderBase {
     _actualWGridSize = actualWGridSize;
   }
   void SetDoImagePSF(bool doImagePSF) { _doImagePSF = doImagePSF; }
+  void SetImagePadding(double imagePadding) { _imagePadding = imagePadding; }
   void SetPolarization(aocommon::PolarizationEnum polarization) {
     _polarization = polarization;
   }
@@ -137,28 +140,34 @@ class MSGridderBase {
 
   virtual void Invert() = 0;
 
-  virtual void Predict(std::vector<Image>&& images) = 0;
+  virtual void Predict(std::vector<aocommon::Image>&& images) = 0;
 
-  virtual std::vector<Image> ResultImages() = 0;
+  virtual std::vector<aocommon::Image> ResultImages() = 0;
 
   void SetPhaseCentreRA(const double phaseCentreRA) {
     _phaseCentreRA = phaseCentreRA;
-    computeFacetCentre();
   }
   void SetPhaseCentreDec(const double phaseCentreDec) {
     _phaseCentreDec = phaseCentreDec;
-    computeFacetCentre();
   }
   double PhaseCentreRA() const { return _phaseCentreRA; }
   double PhaseCentreDec() const { return _phaseCentreDec; }
   void SetPhaseCentreDL(const double phaseCentreDL) {
     _phaseCentreDL = phaseCentreDL;
-    computeFacetCentre();
   }
   void SetPhaseCentreDM(const double phaseCentreDM) {
     _phaseCentreDM = phaseCentreDM;
-    computeFacetCentre();
   }
+
+  void SetFacetDirectionRA(double facetDirectionRA) {
+    _facetDirectionRA = facetDirectionRA;
+  }
+  void SetFacetDirectionDec(double facetDirectionDec) {
+    _facetDirectionDec = facetDirectionDec;
+  }
+
+  double FacetDirectionRA() const { return _facetDirectionRA; }
+  double FacetDirectionDec() const { return _facetDirectionDec; }
   double PhaseCentreDL() const { return _phaseCentreDL; }
   double PhaseCentreDM() const { return _phaseCentreDM; }
 
@@ -169,9 +178,6 @@ class MSGridderBase {
    */
   virtual void FreeImagingData() {}
 
-  virtual size_t ActualInversionWidth() const { return _imageWidth; }
-  virtual size_t ActualInversionHeight() const { return _imageHeight; }
-
   GridMode GetGridMode() const { return _gridMode; }
   void SetGridMode(GridMode gridMode) { _gridMode = gridMode; }
 
@@ -223,15 +229,24 @@ class MSGridderBase {
   }
 
  protected:
+  size_t ActualInversionWidth() const { return _actualInversionWidth; }
+  size_t ActualInversionHeight() const { return _actualInversionHeight; }
+  double ActualPixelSizeX() const { return _actualPixelSizeX; }
+  double ActualPixelSizeY() const { return _actualPixelSizeY; }
+
   int64_t getAvailableMemory(double memFraction, double absMemLimit);
 
   struct MSData {
    public:
     MSData();
-    ~MSData();
-    class MSProvider* msProvider;
+    MSData(const MSData& source) = delete;
+    ~MSData() = default;
+    MSData& operator=(const MSData& source) = delete;
+
+    MSProvider* msProvider;
     size_t msIndex;
-    MultiBandData bandData;
+    size_t dataDescId;
+    aocommon::BandData bandData;
     size_t startChannel, endChannel;
     size_t matchingRows, totalRowsProcessed;
     double minW, maxW, maxWWithFlags, maxBaselineUVW, maxBaselineInM;
@@ -239,19 +254,14 @@ class MSGridderBase {
     double integrationTime;
     std::vector<std::string> antennaNames;
 
-    MultiBandData SelectedBand() const {
-      return MultiBandData(bandData, startChannel, endChannel);
+    aocommon::BandData SelectedBand() const {
+      return aocommon::BandData(bandData, startChannel, endChannel);
     }
-
-   private:
-    MSData(const MSData& source);
-
-    void operator=(const MSData& source);
   };
 
   struct InversionRow {
     double uvw[3];
-    size_t dataDescId, rowId;
+    size_t rowId;
     std::complex<float>* data;
   };
 
@@ -279,7 +289,8 @@ class MSGridderBase {
    * values, they still need to provide an already allocated buffer. This is to
    * avoid having to allocate memory within this method.
    * @tparam PolarizationCount Normally set to one when imaging a single
-   * polarization, but set to 4 for IDG as it images all polarizations at once.
+   * polarization, but set to 2 or 4 for IDG as it images multiple polarizations
+   * at once.
    * @tparam DDGainMatrix Selects which entry or entries in the gain matrix
    * (provided by EveryBeam and/or an h5 solution) file to use for correcting
    * the visibilities. Can be kXX for the XX-entry, kYY for the YY-entry, kTrace
@@ -299,7 +310,8 @@ class MSGridderBase {
   template <size_t PolarizationCount, DDGainMatrix GainEntry>
   void readAndWeightVisibilities(MSReader& msReader,
                                  const std::vector<std::string>& antennaNames,
-                                 InversionRow& rowData, const BandData& curBand,
+                                 InversionRow& rowData,
+                                 const aocommon::BandData& curBand,
                                  float* weightBuffer,
                                  std::complex<float>* modelBuffer,
                                  const bool* isSelected);
@@ -307,18 +319,17 @@ class MSGridderBase {
   /**
    * @brief Write (modelled) visibilities to MS, provides an interface to
    * MSProvider::WriteModel(). Method can be templated on the number of
-   * polarizations (1 or 4), and the DDGainMatrix which can be used to
+   * polarizations (1, 2 or 4), and the DDGainMatrix which can be used to
    * select an entry or entries from the gain matrix that should be used for the
    * correction (XX-pol: kXX, YY-pol: kYY, Trace: kTrace, Full Jones: kFull)
    */
   template <size_t PolarizationCount, DDGainMatrix GainEntry>
   void writeVisibilities(MSProvider& msProvider,
                          const std::vector<std::string>& antennaNames,
-                         const BandData& curBand, std::complex<float>* buffer);
+                         const aocommon::BandData& curBand,
+                         std::complex<float>* buffer);
 
   double _maxW, _minW;
-  size_t _actualInversionWidth, _actualInversionHeight;
-  double _actualPixelSizeX, _actualPixelSizeY;
 
   virtual size_t getSuggestedWGridSize() const = 0;
 
@@ -333,26 +344,47 @@ class MSGridderBase {
 
   void initializeMSDataVector(std::vector<MSData>& msDataVector);
 
-  std::unique_ptr<struct MetaDataCache> _metaDataCache;
+  std::unique_ptr<MetaDataCache> _metaDataCache;
 
   template <size_t PolarizationCount>
-  static void rotateVisibilities(const BandData& bandData, double shiftFactor,
+  static void rotateVisibilities(const aocommon::BandData& bandData,
+                                 double shiftFactor,
                                  std::complex<float>* dataIter);
 
   const Settings& _settings;
 
  private:
+  size_t _actualInversionWidth;
+  size_t _actualInversionHeight;
+  double _actualPixelSizeX;
+  double _actualPixelSizeY;
+
   static std::vector<std::string> getAntennaNames(
       const casacore::MSAntenna& msAntenna);
-  void computeFacetCentre() {
-    aocommon::ImageCoordinates::LMToRaDec(_phaseCentreDL, _phaseCentreDM,
-                                          _phaseCentreRA, _phaseCentreDec,
-                                          _facetCentreRA, _facetCentreDec);
-  }
+
+#ifdef HAVE_EVERYBEAM
+  /**
+   * @brief Compute and cache the beam response if no cached response
+   * present for the provided time.
+   */
+  void CacheBeamResponse(double time, size_t fieldId,
+                         const aocommon::BandData& curBand);
+#endif
+
+  /**
+   * @brief Cache the solutions from a h5 solution file and update the
+   * associated time.
+   */
+  void CacheParmResponse(double time,
+                         const std::vector<std::string>& antennaNames,
+                         const aocommon::BandData& curBand);
+
+  void SetH5Parms();
 
   void resetMetaData() { _hasFrequencies = false; }
 
-  void calculateMSLimits(const MultiBandData& selectedBand, double startTime) {
+  void calculateMSLimits(const aocommon::BandData& selectedBand,
+                         double startTime) {
     if (_hasFrequencies) {
       _freqLow = std::min(_freqLow, selectedBand.LowestFrequency());
       _freqHigh = std::max(_freqHigh, selectedBand.HighestFrequency());
@@ -383,8 +415,33 @@ class MSGridderBase {
   void initializePointResponse(const MSGridderBase::MSData& msData);
   void initializePredictReader(MSProvider& msProvider);
 
+  template <size_t PolarizationCount, DDGainMatrix GainEntry>
+  void ApplyConjugatedH5Parm(MSReader& msReader,
+                             const std::vector<std::string>& antennaNames,
+                             InversionRow& rowData,
+                             const aocommon::BandData& curBand,
+                             const float* weightBuffer);
+
+#ifdef HAVE_EVERYBEAM
+  template <size_t PolarizationCount, DDGainMatrix GainEntry>
+  void ApplyConjugatedFacetBeam(MSReader& msReader, InversionRow& rowData,
+                                const aocommon::BandData& curBand,
+                                const float* weightBuffer);
+
+  /**
+   * @brief Applies both the conjugated facet beam and the conjugated h5 parm
+   * solutions to the visibilities and computes the weight corresponding to the
+   * combined effect.
+   */
+  template <size_t PolarizationCount, DDGainMatrix GainEntry>
+  void ApplyConjugatedFacetDdEffects(
+      MSReader& msReader, const std::vector<std::string>& antennaNames,
+      InversionRow& rowData, const aocommon::BandData& curBand,
+      const float* weightBuffer);
+#endif  // HAVE_EVERYBEAM
+
   double _phaseCentreRA, _phaseCentreDec, _phaseCentreDL, _phaseCentreDM;
-  double _facetCentreRA, _facetCentreDec;
+  double _facetDirectionRA, _facetDirectionDec;
   size_t _facetIndex;
   /// @p _facetGroupIndex and @p _msIndex in conjunction with the @p
   /// MeasurementSetCount() determine the index in the _writerGroupLocks vector,
@@ -394,6 +451,7 @@ class MSGridderBase {
   size_t _msIndex;
   /// @see SetAdditivePredict()
   bool _additivePredict;
+  double _imagePadding;
   size_t _imageWidth, _imageHeight;
   size_t _trimWidth, _trimHeight;
   double _pixelSizeX, _pixelSizeY;
@@ -423,7 +481,7 @@ class MSGridderBase {
   double _maxGriddedWeight;
   double _visibilityWeightSum;
 
-  aocommon::UVector<float> _scratchWeights;
+  aocommon::UVector<float> _scratchImageWeights;
 
   std::unique_ptr<MSReader> _predictReader;
   WriterLockManager* _writerLockManager;
@@ -433,13 +491,17 @@ class MSGridderBase {
   std::unique_ptr<everybeam::telescope::Telescope> _telescope;
   std::unique_ptr<everybeam::pointresponse::PointResponse> _pointResponse;
   aocommon::UVector<std::complex<float>> _cachedBeamResponse;
+  const everybeam::BeamMode _beamMode;
+  const everybeam::BeamNormalisationMode _beamNormalisationMode;
 #endif
-  aocommon::UVector<std::complex<float>> _cachedParmResponse;
-  std::unique_ptr<schaapcommon::h5parm::H5Parm> _h5parm;
-  std::pair<schaapcommon::h5parm::SolTab*, schaapcommon::h5parm::SolTab*>
+  std::vector<std::vector<std::complex<float>>> _cachedParmResponse;
+  std::vector<std::unique_ptr<schaapcommon::h5parm::H5Parm>> _h5parms;
+  std::vector<
+      std::pair<schaapcommon::h5parm::SolTab*, schaapcommon::h5parm::SolTab*>>
       _h5SolTabs;
-  schaapcommon::h5parm::JonesParameters::CorrectType _correctType;
-  std::pair<size_t, size_t> _h5TimeIndex;
+  std::vector<schaapcommon::h5parm::JonesParameters::CorrectType> _correctType;
+  std::vector<std::vector<double>> _cachedMSTimes;
+  std::vector<size_t> _timeOffset;
 };
 
 #endif
