@@ -8,19 +8,20 @@
 #include <idg-api.h>
 
 #include <aocommon/fits/fitsreader.h>
+#include <aocommon/logger.h>
 
 #include "../msproviders/msprovider.h"
 #include "../msproviders/timestepbuffer.h"
 
 #include "../io/findmwacoefffile.h"
 #include "../io/imagefilename.h"
-#include "../io/logger.h"
 #include "../io/parsetreader.h"
 
 #include "../structures/imagingtable.h"
 
 #include "../main/settings.h"
 
+#include "averagebeam.h"
 #include "idgconfiguration.h"
 
 #ifdef HAVE_EVERYBEAM
@@ -36,6 +37,13 @@ using everybeam::aterms::ATermConfig;
 using everybeam::coords::CoordinateSystem;
 #endif  // HAVE_EVERYBEAM
 
+using aocommon::Image;
+using aocommon::Logger;
+
+namespace {
+constexpr const size_t kGridderIndex = 0;
+}
+
 IdgMsGridder::IdgMsGridder(const Settings& settings)
     : MSGridderBase(settings),
       _averageBeam(nullptr),
@@ -67,9 +75,12 @@ void IdgMsGridder::Invert() {
 
   _options["padded_size"] = untrimmedWidth;
 
-  if (!_metaDataCache->averageBeam)
-    _metaDataCache->averageBeam.reset(new AverageBeam());
-  _averageBeam = static_cast<AverageBeam*>(_metaDataCache->averageBeam.get());
+  const bool stokes_I_only =
+      (Polarization() == aocommon::Polarization::StokesI);
+  _options["stokes_I_only"] = stokes_I_only;
+  const size_t n_image_polarizations = stokes_I_only ? 1 : 4;
+
+  if (!_averageBeam) _averageBeam.reset(new AverageBeam());
 
   std::vector<MSData> msDataVector;
   initializeMSDataVector(msDataVector);
@@ -83,7 +94,7 @@ void IdgMsGridder::Invert() {
   const double shiftm = PhaseCentreDM();
   const double shiftp =
       std::sqrt(1.0 - shiftl * shiftl - shiftm * shiftm) - 1.0;
-  _bufferset->init(width, _actualPixelSizeX, max_w + 1.0, shiftl, shiftm,
+  _bufferset->init(width, ActualPixelSizeX(), max_w + 1.0, shiftl, shiftm,
                    shiftp, _options);
   Logger::Debug << "IDG subgrid size: " << _bufferset->get_subgridsize()
                 << '\n';
@@ -102,9 +113,11 @@ void IdgMsGridder::Invert() {
       gridMeasurementSet(msData);
     }
     _bufferset->finalize_compute_avg_beam();
-    _averageBeam->SetScalarBeam(_bufferset->get_scalar_beam());
-    _averageBeam->SetMatrixInverseBeam(_bufferset->get_matrix_inverse_beam());
-    _image.assign(4 * width * height, 0.0);
+    _averageBeam->SetScalarBeam(_bufferset->get_scalar_beam(), width, height);
+    _averageBeam->SetMatrixInverseBeam(_bufferset->get_matrix_inverse_beam(),
+                                       _bufferset->get_subgridsize(),
+                                       _bufferset->get_subgridsize());
+    _image.assign(n_image_polarizations * width * height, 0.0);
     _bufferset->get_image(_image.data());
 
     Logger::Debug << "Total weight: " << totalWeight() << '\n';
@@ -130,8 +143,10 @@ void IdgMsGridder::Invert() {
       }
       _bufferset->finalize_compute_avg_beam();
       Logger::Debug << "Finished computing average beam.\n";
-      _averageBeam->SetScalarBeam(_bufferset->get_scalar_beam());
-      _averageBeam->SetMatrixInverseBeam(_bufferset->get_matrix_inverse_beam());
+      _averageBeam->SetScalarBeam(_bufferset->get_scalar_beam(), width, height);
+      _averageBeam->SetMatrixInverseBeam(_bufferset->get_matrix_inverse_beam(),
+                                         _bufferset->get_subgridsize(),
+                                         _bufferset->get_subgridsize());
     }
 
     resetVisibilityCounters();
@@ -139,7 +154,7 @@ void IdgMsGridder::Invert() {
       // Adds the gridding result to _image member
       gridMeasurementSet(msData);
     }
-    _image.assign(4 * width * height, 0.0);
+    _image.assign(n_image_polarizations * width * height, 0.0);
     _bufferset->get_image(_image.data());
   }
 
@@ -163,12 +178,16 @@ void IdgMsGridder::gridMeasurementSet(const MSGridderBase::MSData& msData) {
 
   StartMeasurementSet(msData, false);
 
-  aocommon::UVector<float> weightBuffer(_selectedBands.MaxChannels() * 4);
+  const size_t n_vis_polarizations = msData.msProvider->NPolarizations();
+  constexpr size_t n_idg_polarizations = 4;
+  aocommon::UVector<float> weightBuffer(_selectedBand.ChannelCount() *
+                                        n_idg_polarizations);
   aocommon::UVector<std::complex<float>> modelBuffer(
-      _selectedBands.MaxChannels() * 4);
-  aocommon::UVector<bool> isSelected(_selectedBands.MaxChannels() * 4, true);
+      _selectedBand.ChannelCount() * n_idg_polarizations);
+  aocommon::UVector<bool> isSelected(
+      _selectedBand.ChannelCount() * n_idg_polarizations, true);
   aocommon::UVector<std::complex<float>> dataBuffer(
-      _selectedBands.MaxChannels() * 4);
+      _selectedBand.ChannelCount() * n_idg_polarizations);
 
   _griddingWatch.Start();
 
@@ -193,11 +212,10 @@ void IdgMsGridder::gridMeasurementSet(const MSGridderBase::MSData& msData) {
 #ifdef HAVE_EVERYBEAM
       if (aTermMaker) {
         timestepReader.GetUVWsForTimestep(uvws);
-        if (aTermMaker->Calculate(
-                aTermBuffer.data(), currentTime,
-                _selectedBands[metaData.dataDescId].CentreFrequency(),
-                metaData.fieldId, uvws.data())) {
-          _bufferset->get_gridder(metaData.dataDescId)
+        if (aTermMaker->Calculate(aTermBuffer.data(), currentTime,
+                                  _selectedBand.CentreFrequency(),
+                                  metaData.fieldId, uvws.data())) {
+          _bufferset->get_gridder(kGridderIndex)
               ->set_aterm(timeIndex, aTermBuffer.data());
           Logger::Debug << "Calculated a-terms for timestep " << timeIndex
                         << "\n";
@@ -205,7 +223,6 @@ void IdgMsGridder::gridMeasurementSet(const MSGridderBase::MSData& msData) {
       }
 #endif
     }
-    const BandData& curBand(_selectedBands[metaData.dataDescId]);
     IDGInversionRow rowData;
 
     rowData.data = dataBuffer.data();
@@ -216,15 +233,36 @@ void IdgMsGridder::gridMeasurementSet(const MSGridderBase::MSData& msData) {
     rowData.antenna1 = metaData.antenna1;
     rowData.antenna2 = metaData.antenna2;
     rowData.timeIndex = timeIndex;
-    rowData.dataDescId = metaData.dataDescId;
-    readAndWeightVisibilities<4, DDGainMatrix::kFull>(
-        *msReader, msData.antennaNames, rowData, curBand, weightBuffer.data(),
-        modelBuffer.data(), isSelected.data());
+    if (n_vis_polarizations == 2) {
+      readAndWeightVisibilities<2, DDGainMatrix::kFull>(
+          *msReader, msData.antennaNames, rowData, _selectedBand,
+          weightBuffer.data(), modelBuffer.data(), isSelected.data());
+      // The data is placed in the first half of the buffers: reverse copy it
+      // and expand it to 4 polarizations. TODO at a later time, IDG should
+      // directly accept 2 pols instead of 4.
+      size_t source_index = dataBuffer.size() / 2;
+      for (size_t i = dataBuffer.size(); i != 0; i -= 4) {
+        rowData.data[i - 1] = rowData.data[source_index - 1];
+        rowData.data[i - 2] = 0.0;
+        rowData.data[i - 3] = 0.0;
+        rowData.data[i - 4] = rowData.data[source_index - 2];
+        weightBuffer[i - 1] = weightBuffer[source_index - 1];
+        weightBuffer[i - 2] = weightBuffer[source_index - 1];
+        weightBuffer[i - 3] = weightBuffer[source_index - 2];
+        weightBuffer[i - 4] = weightBuffer[source_index - 2];
+        source_index -= 2;
+      }
+    } else {
+      assert(n_vis_polarizations == 4);
+      readAndWeightVisibilities<4, DDGainMatrix::kFull>(
+          *msReader, msData.antennaNames, rowData, _selectedBand,
+          weightBuffer.data(), modelBuffer.data(), isSelected.data());
+    }
 
     rowData.uvw[1] = -metaData.vInM;  // DEBUG vdtol, flip axis
     rowData.uvw[2] = -metaData.wInM;  //
 
-    _bufferset->get_gridder(rowData.dataDescId)
+    _bufferset->get_gridder(kGridderIndex)
         ->grid_visibilities(timeIndex, metaData.antenna1, metaData.antenna2,
                             rowData.uvw, rowData.data, weightBuffer.data());
   }
@@ -244,24 +282,27 @@ void IdgMsGridder::Predict(std::vector<Image>&& images) {
 
   _options["padded_size"] = untrimmedWidth;
 
-  _image.assign(4 * width * height, 0.0);
-  if (!_metaDataCache->averageBeam) {
-    Logger::Info << "no average_beam in cache, creating an empty one.\n";
-    _metaDataCache->averageBeam.reset(new AverageBeam());
-  }
-  _averageBeam = static_cast<AverageBeam*>(_metaDataCache->averageBeam.get());
+  const bool stokes_I_only =
+      (Polarization() == aocommon::Polarization::StokesI);
+  _options["stokes_I_only"] = stokes_I_only;
+  const size_t n_image_polarizations = stokes_I_only ? 1 : 4;
 
+  _image.assign(n_image_polarizations * width * height, 0.0);
+  if (!_averageBeam) {
+    Logger::Debug << "No average beam in cache, creating an empty one.\n";
+    _averageBeam.reset(new AverageBeam());
+  }
+
+  assert(images.size() == n_image_polarizations);
   if (Polarization() == aocommon::Polarization::FullStokes) {
-    assert(images.size() == 4);
-    for (size_t polIndex = 0; polIndex != 4; ++polIndex) {
-      std::copy_n(images[polIndex].data(), width * height,
+    for (size_t polIndex = 0; polIndex != n_image_polarizations; ++polIndex) {
+      std::copy_n(images[polIndex].Data(), width * height,
                   _image.data() + polIndex * width * height);
     }
   } else {
-    assert(images.size() == 1);
     const size_t stokesIndex =
         aocommon::Polarization::StokesToIndex(Polarization());
-    std::copy_n(images[0].data(), width * height,
+    std::copy_n(images[0].Data(), width * height,
                 _image.data() + stokesIndex * width * height);
   }
 
@@ -285,7 +326,7 @@ void IdgMsGridder::Predict(std::vector<Image>&& images) {
   const double shiftm = PhaseCentreDM();
   const double shiftp =
       std::sqrt(1.0 - shiftl * shiftl - shiftm * shiftm) - 1.0;
-  _bufferset->init(width, _actualPixelSizeX, max_w + 1.0, shiftl, shiftm,
+  _bufferset->init(width, ActualPixelSizeX(), max_w + 1.0, shiftl, shiftm,
                    shiftp, _options);
   _bufferset->set_image(_image.data(), do_scale);
 
@@ -326,8 +367,9 @@ void IdgMsGridder::predictMeasurementSet(const MSGridderBase::MSData& msData) {
   _outputProvider = msData.msProvider;
   StartMeasurementSet(msData, true);
 
-  aocommon::UVector<std::complex<float>> buffer(_selectedBands.MaxChannels() *
-                                                4);
+  constexpr size_t n_idg_polarizations = 4;
+  aocommon::UVector<std::complex<float>> buffer(_selectedBand.ChannelCount() *
+                                                n_idg_polarizations);
   _degriddingWatch.Start();
 
   int timeIndex = -1;
@@ -352,11 +394,10 @@ void IdgMsGridder::predictMeasurementSet(const MSGridderBase::MSData& msData) {
 #ifdef HAVE_EVERYBEAM
       if (aTermMaker) {
         timestepReader.GetUVWsForTimestep(uvws);
-        if (aTermMaker->Calculate(
-                aTermBuffer.data(), currentTime,
-                _selectedBands[metaData.dataDescId].CentreFrequency(),
-                metaData.fieldId, uvws.data())) {
-          _bufferset->get_degridder(metaData.dataDescId)
+        if (aTermMaker->Calculate(aTermBuffer.data(), currentTime,
+                                  _selectedBand.CentreFrequency(),
+                                  metaData.fieldId, uvws.data())) {
+          _bufferset->get_degridder(kGridderIndex)
               ->set_aterm(timeIndex, aTermBuffer.data());
           Logger::Debug << "Calculated new a-terms for timestep " << timeIndex
                         << "\n";
@@ -372,34 +413,45 @@ void IdgMsGridder::predictMeasurementSet(const MSGridderBase::MSData& msData) {
     row.antenna1 = metaData.antenna1;
     row.antenna2 = metaData.antenna2;
     row.timeIndex = timeIndex;
-    row.dataDescId = metaData.dataDescId;
     row.rowId = provRowId;
     predictRow(row, msData.antennaNames);
   }
 
-  for (size_t d = 0; d != _selectedBands.DataDescCount(); ++d)
-    computePredictionBuffer(d, msData.antennaNames);
+  computePredictionBuffer(msData.antennaNames);
 }
 
 void IdgMsGridder::predictRow(IDGPredictionRow& row,
                               const std::vector<std::string>& antennaNames) {
-  while (_bufferset->get_degridder(row.dataDescId)
+  while (_bufferset->get_degridder(kGridderIndex)
              ->request_visibilities(row.rowId, row.timeIndex, row.antenna1,
                                     row.antenna2, row.uvw)) {
-    computePredictionBuffer(row.dataDescId, antennaNames);
+    computePredictionBuffer(antennaNames);
   }
 }
 
 void IdgMsGridder::computePredictionBuffer(
-    size_t dataDescId, const std::vector<std::string>& antennaNames) {
-  auto available_row_ids = _bufferset->get_degridder(dataDescId)->compute();
+    const std::vector<std::string>& antennaNames) {
+  auto available_row_ids = _bufferset->get_degridder(kGridderIndex)->compute();
   Logger::Debug << "Computed " << available_row_ids.size() << " rows.\n";
-  const BandData& curBand(_selectedBands[dataDescId]);
-  for (auto i : available_row_ids) {
-    writeVisibilities<4, DDGainMatrix::kFull>(*_outputProvider, antennaNames,
-                                              curBand, i.second);
+  const size_t n_vis_polarizations = _outputProvider->NPolarizations();
+  for (std::pair<long unsigned, std::complex<float>*>& row :
+       available_row_ids) {
+    if (n_vis_polarizations == 2) {
+      // Remove the XY/YX pols from the data and place the result in the first
+      // half of the array
+      for (size_t i = 0; i != _selectedBand.ChannelCount(); ++i) {
+        row.second[i * 2] = row.second[i * 4];
+        row.second[i * 2 + 1] = row.second[i * 4 + 3];
+      }
+      writeVisibilities<2, DDGainMatrix::kFull>(*_outputProvider, antennaNames,
+                                                _selectedBand, row.second);
+    } else {
+      assert(n_vis_polarizations == 4);
+      writeVisibilities<4, DDGainMatrix::kFull>(*_outputProvider, antennaNames,
+                                                _selectedBand, row.second);
+    }
   }
-  _bufferset->get_degridder(dataDescId)->finished_reading();
+  _bufferset->get_degridder(kGridderIndex)->finished_reading();
   _degriddingWatch.Pause();
 }
 
@@ -412,27 +464,30 @@ std::vector<Image> IdgMsGridder::ResultImages() {
     for (size_t polIndex = 0; polIndex != 4; ++polIndex) {
       images.emplace_back(width, height);
       std::copy_n(_image.data() + polIndex * width * height, width * height,
-                  images[polIndex].data());
+                  images[polIndex].Data());
     }
   } else {
     size_t polIndex = aocommon::Polarization::StokesToIndex(Polarization());
     images.emplace_back(width, height);
     std::copy_n(_image.data() + polIndex * width * height, width * height,
-                images[0].data());
+                images[0].Data());
   }
   return images;
 }
 
+void IdgMsGridder::SetAverageBeam(std::unique_ptr<AverageBeam> average_beam) {
+  _averageBeam = std::move(average_beam);
+}
+
+std::unique_ptr<AverageBeam> IdgMsGridder::ReleaseAverageBeam() {
+  return std::move(_averageBeam);
+}
+
 void IdgMsGridder::SaveBeamImage(const ImagingTableEntry& entry,
                                  ImageFilename& filename,
                                  const Settings& settings, double ra,
                                  double dec, double pdl, double pdm,
-                                 const MetaDataCache& cache) {
-  if (!cache.averageBeam || cache.averageBeam->Empty()) {
-    throw std::runtime_error(
-        "IDG gridder can not save the beam image. Beam has not been computed "
-        "yet.");
-  }
+                                 const AverageBeam& average_beam) {
   aocommon::FitsWriter writer;
   writer.SetImageDimensions(settings.trimmedImageWidth,
                             settings.trimmedImageHeight, ra, dec,
@@ -444,7 +499,7 @@ void IdgMsGridder::SaveBeamImage(const ImagingTableEntry& entry,
   writer.SetFrequency(entry.CentralFrequency(),
                       entry.bandEndFrequency - entry.bandStartFrequency);
   writer.Write(polName.GetBeamPrefix(settings) + ".fits",
-               cache.averageBeam->ScalarBeam()->data());
+               average_beam.ScalarBeam()->data());
 }
 
 void IdgMsGridder::SavePBCorrectedImages(aocommon::FitsWriter& writer,
@@ -456,7 +511,7 @@ void IdgMsGridder::SavePBCorrectedImages(aocommon::FitsWriter& writer,
   aocommon::FitsReader reader(beamName.GetBeamPrefix(settings) + ".fits");
 
   Image beam(reader.ImageWidth(), reader.ImageHeight());
-  reader.Read(beam.data());
+  reader.Read(beam.Data());
 
   Image image;
   for (size_t polIndex = 0; polIndex != 4; ++polIndex) {
@@ -466,8 +521,8 @@ void IdgMsGridder::SavePBCorrectedImages(aocommon::FitsWriter& writer,
     name.SetPolarization(pol);
     aocommon::FitsReader reader(name.GetPrefix(settings) + "-" + filenameKind +
                                 ".fits");
-    if (image.empty()) image = Image(reader.ImageWidth(), reader.ImageHeight());
-    reader.Read(image.data());
+    if (image.Empty()) image = Image(reader.ImageWidth(), reader.ImageHeight());
+    reader.Read(image.Data());
 
     for (size_t i = 0; i != reader.ImageWidth() * reader.ImageHeight(); ++i) {
       if (beam[i] > 1e-6)
@@ -478,7 +533,7 @@ void IdgMsGridder::SavePBCorrectedImages(aocommon::FitsWriter& writer,
 
     writer.SetPolarization(pol);
     writer.Write(name.GetPrefix(settings) + "-" + filenameKind + "-pb.fits",
-                 image.data());
+                 image.Data());
   }
 }
 
@@ -497,18 +552,15 @@ bool IdgMsGridder::prepareForMeasurementSet(
   // Skip this ms if there is no data in it
   if (!max_baseline) return false;
 
-  _selectedBands = msData.SelectedBand();
+  _selectedBand = msData.SelectedBand();
 
   // TODO for now we map the ms antennas directly to the gridder's antenna,
   // including non-selected antennas. Later this can be made more efficient.
-  size_t nStations = msData.msProvider->MS()->antenna().nrow();
+  const size_t nStations = msData.msProvider->MS()->antenna().nrow();
 
   std::vector<std::vector<double>> bands;
-  for (size_t i = 0; i != _selectedBands.BandCount(); ++i) {
-    bands.push_back(std::vector<double>(_selectedBands[i].begin(),
-                                        _selectedBands[i].end()));
-  }
-  size_t nChannels = _selectedBands.MaxChannels();
+  bands.emplace_back(_selectedBand.begin(), _selectedBand.end());
+  const size_t nChannels = _selectedBand.ChannelCount();
 
   uint64_t memSize =
       getAvailableMemory(_settings.memFraction, _settings.absMemLimit);
@@ -522,7 +574,7 @@ bool IdgMsGridder::prepareForMeasurementSet(
 #ifdef HAVE_EVERYBEAM
   aTermMaker = getATermMaker(msData);
   if (aTermMaker) {
-    size_t subgridsize = _bufferset->get_subgridsize();
+    const size_t subgridsize = _bufferset->get_subgridsize();
     aTermBuffer.resize(subgridsize * subgridsize * 4 * nStations);
     // When a-terms are used, they will also take memory. Here we calculate
     // their approx contribution.
@@ -530,7 +582,7 @@ bool IdgMsGridder::prepareForMeasurementSet(
     Logger::Debug << "A-terms change on average every " << avgUpdate
                   << " s, once every " << (avgUpdate / msData.integrationTime)
                   << " timesteps.\n";
-    uint64_t atermMemPerTimestep =
+    const uint64_t atermMemPerTimestep =
         subgridsize * subgridsize * nStations *  // size of grid x nr of grids
         (4 * 8) *  // 4 pol, 8 bytes per complex value
         (msData.integrationTime /
@@ -568,7 +620,6 @@ std::unique_ptr<class ATermBase> IdgMsGridder::getATermMaker(
     const MSGridderBase::MSData& msData) {
   SynchronizedMS ms = msData.msProvider->MS();
   size_t nr_stations = ms->antenna().nrow();
-  aocommon::UVector<std::complex<float>> aTermBuffer;
   if (!_settings.atermConfigFilename.empty() || _settings.gridWithBeam) {
     // IDG uses a flipped coordinate system which is moved by half a pixel:
     everybeam::coords::CoordinateSystem system;
@@ -606,7 +657,7 @@ std::unique_ptr<class ATermBase> IdgMsGridder::getATermMaker(
           new ATermConfig(nr_stations, system, aterm_settings));
       config->SetSaveATerms(_settings.saveATerms, _settings.prefixName);
       config->Read(*ms, parset_aterms, ms.Filename());
-      return std::move(config);
+      return config;
     } else {
       // If MWA MS, get the path to the coefficient file
       if (everybeam::GetTelescopeType(*ms) ==
@@ -621,11 +672,11 @@ std::unique_ptr<class ATermBase> IdgMsGridder::getATermMaker(
 
       std::unique_ptr<ATermBeam> beam = ATermConfig::GetATermBeam(
           *ms, system, aterm_settings, frequencyInterpolation,
-          _settings.useDifferentialLofarBeam, useChannelFrequency,
-          elementResponseModel);
+          _settings.beamNormalisationMode, useChannelFrequency,
+          elementResponseModel, _settings.beamMode);
       beam->SetSaveATerms(_settings.saveATerms, _settings.prefixName);
       beam->SetUpdateInterval(_settings.beamAtermUpdateTime);
-      return std::move(beam);
+      return beam;
     }
   } else {
     return std::unique_ptr<ATermBase>();
