@@ -7,6 +7,7 @@
 
 #include <idg-api.h>
 
+#include <aocommon/image.h>
 #include <aocommon/lane.h>
 #include <aocommon/uvector.h>
 #include <aocommon/fits/fitswriter.h>
@@ -19,7 +20,10 @@
 
 #include "../main/stopwatch.h"
 
-class IdgMsGridder : public MSGridderBase {
+struct ImagingTableEntry;
+class ImageFilename;
+
+class IdgMsGridder final : public MSGridderBase {
  public:
   IdgMsGridder(const class Settings& settings);
 
@@ -27,22 +31,26 @@ class IdgMsGridder : public MSGridderBase {
 
   virtual void Invert() final override;
 
-  virtual void Predict(std::vector<Image>&& images) final override;
+  virtual void Predict(std::vector<aocommon::Image>&& images) final override;
 
-  virtual std::vector<Image> ResultImages() final override;
+  virtual std::vector<aocommon::Image> ResultImages() final override;
 
-  static void SavePBCorrectedImages(class aocommon::FitsWriter& writer,
-                                    const class ImageFilename& filename,
+  static void SavePBCorrectedImages(aocommon::FitsWriter& writer,
+                                    const ImageFilename& filename,
                                     const std::string& filenameKind,
                                     const Settings& settings);
 
-  static void SaveBeamImage(const struct ImagingTableEntry& entry,
-                            class ImageFilename& filename,
-                            const Settings& settings, double ra, double dec,
-                            double pdl, double pdm, const MetaDataCache& cache);
+  static void SaveBeamImage(const ImagingTableEntry& entry,
+                            ImageFilename& filename, const Settings& settings,
+                            double ra, double dec, double pdl, double pdm,
+                            const AverageBeam& average_beam);
+
+  void SetAverageBeam(std::unique_ptr<AverageBeam> average_beam);
+
+  std::unique_ptr<AverageBeam> ReleaseAverageBeam();
 
  private:
-  AverageBeam* _averageBeam;
+  std::unique_ptr<AverageBeam> _averageBeam;
 
   virtual size_t getSuggestedWGridSize() const override final {
     return 1;  // TODO
@@ -76,12 +84,11 @@ class IdgMsGridder : public MSGridderBase {
   };
   struct IDGPredictionRow {
     double uvw[3];
-    size_t dataDescId, antenna1, antenna2, timeIndex, rowId;
+    size_t antenna1, antenna2, timeIndex, rowId;
   };
   void predictRow(IDGPredictionRow& row,
                   const std::vector<std::string>& antennaNames);
-  void computePredictionBuffer(size_t dataDescId,
-                               const std::vector<std::string>& antennaNames);
+  void computePredictionBuffer(const std::vector<std::string>& antennaNames);
 
   std::unique_ptr<idg::api::BufferSet> _bufferset;
   size_t _subgridSize;
@@ -89,7 +96,7 @@ class IdgMsGridder : public MSGridderBase {
   aocommon::UVector<float> _taper_subgrid;
   aocommon::UVector<float> _taper_grid;
   MSProvider* _outputProvider;
-  MultiBandData _selectedBands;
+  aocommon::BandData _selectedBand;
   idg::api::Type _proxyType;
   int _buffersize;
   idg::api::options_type _options;
