@@ -2,29 +2,26 @@
 
 #include "imageanalysis.h"
 
-#include "../system/fftwmanager.h"
-#include "../system/threadpool.h"
-
-#include "../math/fftconvolver.h"
-#include "../math/gaussianfitter.h"
-#include "../math/modelrenderer.h"
-
 #include "../deconvolution/imageset.h"
 
-#include "../structures/image.h"
+#include <aocommon/image.h>
+#include <aocommon/system.h>
+
+#include <schaapcommon/fft/convolution.h>
+#include <schaapcommon/fitters/gaussianfitter.h>
+
+#include <boost/numeric/conversion/bounds.hpp>
 
 #include <algorithm>
 #include <iostream>
 
-#include <boost/numeric/conversion/bounds.hpp>
+using aocommon::Image;
 
 IUWTDeconvolutionAlgorithm::IUWTDeconvolutionAlgorithm(
-    FFTWManager& fftwManager, size_t width, size_t height, float gain,
-    float mGain, float cleanBorder, bool allowNegativeComponents,
-    const bool* mask, float absoluteThreshold, float thresholdSigmaLevel,
-    float tolerance, bool useSNRTest)
-    : _fftwManager(fftwManager),
-      _width(width),
+    size_t width, size_t height, float gain, float mGain, float cleanBorder,
+    bool allowNegativeComponents, const bool* mask, float absoluteThreshold,
+    float thresholdSigmaLevel, float tolerance, bool useSNRTest)
+    : _width(width),
       _height(height),
       _gain(gain),
       _mGain(mGain),
@@ -40,13 +37,13 @@ void IUWTDeconvolutionAlgorithm::measureRMSPerScale(
     const float* image, const float* convolvedImage, float* scratch,
     size_t endScale, std::vector<ScaleResponse>& psfResponse) {
   IUWTDecomposition imageIUWT(endScale, _width, _height);
-  imageIUWT.Decompose(*_threadPool, image, scratch, false);
+  imageIUWT.Decompose(*_staticFor, image, scratch, false);
 
   _psfMaj = 2.0;
   _psfMin = 2.0;
   _psfPA = 0.0;
   double fl = 0.0;
-  GaussianFitter fitter;
+  schaapcommon::fitters::GaussianFitter fitter;
   fitter.Fit2DGaussianCentred(image, _width, _height, 2.0, _psfMaj, _psfMin,
                               _psfPA);
   _psfVolume = (M_PI / 4.0) * _psfMaj * _psfMin / M_LOG2E;
@@ -67,21 +64,21 @@ void IUWTDeconvolutionAlgorithm::measureRMSPerScale(
     v = 1.0;
     x = _width / 2;
     y = _height / 2;
-    fitter.Fit2DGaussianFull(imageIUWT[scale].Coefficients().data(), _width,
+    fitter.Fit2DGaussianFull(imageIUWT[scale].Coefficients().Data(), _width,
                              _height, v, x, y, bMaj, bMin, bPA, &fl);
     psfResponse[scale].bMaj = bMaj;
     psfResponse[scale].bMin = bMin;
     psfResponse[scale].bPA = bPA;
   }
 
-  imageIUWT.Decompose(*_threadPool, imageIUWT[1].Coefficients().data(), scratch,
+  imageIUWT.Decompose(*_staticFor, imageIUWT[1].Coefficients().Data(), scratch,
                       false);
   for (size_t scale = 0; scale != endScale; ++scale) {
     psfResponse[scale].peakResponseToNextScale =
         centralPeak(imageIUWT[scale].Coefficients());
   }
 
-  imageIUWT.Decompose(*_threadPool, convolvedImage, scratch, false);
+  imageIUWT.Decompose(*_staticFor, convolvedImage, scratch, false);
 
   for (size_t scale = 0; scale != endScale; ++scale) {
     psfResponse[scale].convolvedPeakResponse =
@@ -101,7 +98,7 @@ void IUWTDeconvolutionAlgorithm::measureRMSPerScale(
   aocommon::UVector<bool> markedMask0(mask[0].size(), false);
   ImageAnalysis::Component2D c2D(_width / 2, _height / 2);
   float threshold = psfResponse[0].convolvedPeakResponse * _tolerance;
-  ImageAnalysis::FloodFill2D(imageIUWT[0].Coefficients().data(),
+  ImageAnalysis::FloodFill2D(imageIUWT[0].Coefficients().Data(),
                              markedMask0.data(), threshold, c2D, _width,
                              _height, psfResponse[0].convolvedArea);
 }
@@ -117,7 +114,7 @@ float IUWTDeconvolutionAlgorithm::mad(const float* dest) {
 float IUWTDeconvolutionAlgorithm::getMaxAbsWithoutMask(const Image& data,
                                                        size_t& x, size_t& y,
                                                        size_t width) {
-  size_t height = data.size() / width;
+  size_t height = data.Size() / width;
   size_t xBorder = _cleanBorder * width;
   size_t yBorder = _cleanBorder * height;
   size_t minX = xBorder, maxX = width - xBorder;
@@ -127,7 +124,7 @@ float IUWTDeconvolutionAlgorithm::getMaxAbsWithoutMask(const Image& data,
 
   float maxVal = boost::numeric::bounds<float>::lowest();
   for (size_t yi = minY; yi != maxY; ++yi) {
-    const float* dataPtr = data.data() + yi * width;
+    const float* dataPtr = data.Data() + yi * width;
     for (size_t xi = minX; xi != maxX; ++xi) {
       float val =
           _allowNegativeComponents ? std::fabs(dataPtr[xi]) : dataPtr[xi];
@@ -144,7 +141,7 @@ float IUWTDeconvolutionAlgorithm::getMaxAbsWithoutMask(const Image& data,
 float IUWTDeconvolutionAlgorithm::getMaxAbsWithMask(const Image& data,
                                                     size_t& x, size_t& y,
                                                     size_t width) {
-  size_t height = data.size() / width;
+  size_t height = data.Size() / width;
   size_t xBorder = _cleanBorder * width;
   size_t yBorder = _cleanBorder * height;
   size_t minX = xBorder, maxX = width - xBorder;
@@ -154,7 +151,7 @@ float IUWTDeconvolutionAlgorithm::getMaxAbsWithMask(const Image& data,
 
   float maxVal = boost::numeric::bounds<float>::lowest();
   for (size_t yi = minY; yi != maxY; ++yi) {
-    const float* dataPtr = data.data() + yi * width;
+    const float* dataPtr = data.Data() + yi * width;
     const bool* maskPtr = _mask + yi * width;
     for (size_t xi = minX; xi != maxX; ++xi) {
       if (maskPtr[xi]) {
@@ -174,7 +171,7 @@ float IUWTDeconvolutionAlgorithm::getMaxAbsWithMask(const Image& data,
 float IUWTDeconvolutionAlgorithm::dotProduct(const Image& lhs,
                                              const Image& rhs) {
   float sum = 0.0;
-  for (size_t i = 0; i != lhs.size(); ++i) sum += lhs[i] * rhs[i];
+  for (size_t i = 0; i != lhs.Size(); ++i) sum += lhs[i] * rhs[i];
   return sum;
 }
 
@@ -186,11 +183,11 @@ void IUWTDeconvolutionAlgorithm::factorAdd(float* dest, const float* rhs,
 
 void IUWTDeconvolutionAlgorithm::factorAdd(Image& dest, const Image& rhs,
                                            float factor) {
-  for (size_t i = 0; i != dest.size(); ++i) dest[i] += rhs[i] * factor;
+  for (size_t i = 0; i != dest.Size(); ++i) dest[i] += rhs[i] * factor;
 }
 
 void IUWTDeconvolutionAlgorithm::Subtract(float* dest, const Image& rhs) {
-  for (size_t i = 0; i != rhs.size(); ++i) dest[i] -= rhs[i];
+  for (size_t i = 0; i != rhs.Size(); ++i) dest[i] -= rhs[i];
 }
 
 void IUWTDeconvolutionAlgorithm::boundingBox(size_t& x1, size_t& y1, size_t& x2,
@@ -204,7 +201,7 @@ void IUWTDeconvolutionAlgorithm::boundingBox(size_t& x1, size_t& y1, size_t& x2,
   y1 = height;
   y2 = 0;
   for (size_t y = 0; y != height; ++y) {
-    const float* ptr = image.data() + y * width;
+    const float* ptr = image.Data() + y * width;
     for (size_t x = 0; x != x1; ++x) {
       if (std::fabs(ptr[x]) > m * 0.01) {
         x1 = x;
@@ -220,7 +217,7 @@ void IUWTDeconvolutionAlgorithm::boundingBox(size_t& x1, size_t& y1, size_t& x2,
   }
   x2++;
   for (size_t y = 0; y != height; ++y) {
-    const float* ptr = image.data() + y * width;
+    const float* ptr = image.Data() + y * width;
     for (size_t x = 0; x != width; ++x) {
       if (std::fabs(ptr[x]) > m * 0.01) {
         if (y1 > y) y1 = y;
@@ -272,24 +269,26 @@ void IUWTDeconvolutionAlgorithm::adjustBox(size_t& x1, size_t& y1, size_t& x2,
 }
 
 void IUWTDeconvolutionAlgorithm::trim(Image& dest, const float* source,
-                                      size_t oldWidth, size_t /*oldHeight*/,
-                                      size_t x1, size_t y1, size_t x2,
-                                      size_t y2) {
+                                      size_t oldWidth, size_t x1, size_t y1,
+                                      size_t x2, size_t y2) {
   // We do this so that dest and source can be the same image.
-  dest = Image((x2 - x1), (y2 - y1));
+  Image scratch((x2 - x1), (y2 - y1));
   for (size_t y = y1; y != y2; ++y) {
     const float* oldPtr = &source[y * oldWidth];
-    float* newPtr = &dest[(y - y1) * (x2 - x1)];
+    float* newPtr = &scratch[(y - y1) * (x2 - x1)];
     for (size_t x = x1; x != x2; ++x) {
       newPtr[x - x1] = oldPtr[x];
     }
   }
+  dest = std::move(scratch);
 }
 
 void IUWTDeconvolutionAlgorithm::untrim(Image& image, size_t width,
                                         size_t height, size_t x1, size_t y1,
                                         size_t x2, size_t y2) {
-  image = Image(width, height, 0.0);
+  Image scratch(width, height, 0.0);
+  std::copy_n(image.Data(), image.Width() * image.Height(), scratch.Data());
+  image = scratch;
   size_t y = y2;
   while (y != y1) {
     --y;
@@ -318,7 +317,7 @@ void IUWTDeconvolutionAlgorithm::untrim(Image& image, size_t width,
 
 float IUWTDeconvolutionAlgorithm::sum(const Image& img) const {
   float s = 0.0;
-  for (size_t i = 0; i != img.size(); ++i) s += img[i];
+  for (size_t i = 0; i != img.Size(); ++i) s += img[i];
   return s;
 }
 
@@ -326,9 +325,9 @@ float IUWTDeconvolutionAlgorithm::snr(const IUWTDecomposition& noisyImg,
                                       const IUWTDecomposition& model) const {
   float mSum = 0.0, nSum = 0.0;
   for (size_t scale = 0; scale != noisyImg.NScales(); ++scale) {
-    const Image &n = noisyImg[scale].Coefficients(),
-                &m = model[scale].Coefficients();
-    for (size_t i = 0; i != n.size(); ++i) {
+    const Image& n = noisyImg[scale].Coefficients();
+    const Image& m = model[scale].Coefficients();
+    for (size_t i = 0; i != n.Size(); ++i) {
       mSum += m[i] * m[i];
       float d = m[i] - n[i];
       nSum += d * d;
@@ -339,20 +338,20 @@ float IUWTDeconvolutionAlgorithm::snr(const IUWTDecomposition& noisyImg,
 
 float IUWTDeconvolutionAlgorithm::rmsDiff(const Image& a, const Image& b) {
   float sum = 0.0;
-  for (size_t i = 0; i != a.size(); ++i) {
+  for (size_t i = 0; i != a.Size(); ++i) {
     float d = a[i] - b[i];
     sum += d * d;
   }
-  return sqrt(sum / a.size());
+  return std::sqrt(sum / a.Size());
 }
 
 float IUWTDeconvolutionAlgorithm::rms(const Image& image) {
   float sum = 0.0;
-  for (size_t i = 0; i != image.size(); ++i) {
+  for (size_t i = 0; i != image.Size(); ++i) {
     float v = image[i];
     sum += v * v;
   }
-  return sqrt(sum / image.size());
+  return std::sqrt(sum / image.Size());
 }
 
 bool IUWTDeconvolutionAlgorithm::runConjugateGradient(
@@ -367,12 +366,11 @@ bool IUWTDeconvolutionAlgorithm::runConjugateGradient(
   for (size_t minorIter = 0; minorIter != 20; ++minorIter) {
     // scratch = gradient (x) psf
     scratch = gradient;
-    FFTConvolver::ConvolveSameSize(_fftwManager, scratch.data(),
-                                   psfKernel.data(), width, height,
-                                   _threadPool->size());
+    schaapcommon::fft::Convolve(scratch.Data(), psfKernel.Data(), width, height,
+                                _staticFor->NThreads());
 
     // calc: IUWT gradient (x) psf
-    iuwt.Decompose(*_threadPool, scratch.data(), scratch.data(), false);
+    iuwt.Decompose(*_staticFor, scratch.Data(), scratch.Data(), false);
 
     // calc: mask IUWT gradient (x) psf
     iuwt.ApplyMask(mask);
@@ -386,7 +384,7 @@ bool IUWTDeconvolutionAlgorithm::runConjugateGradient(
     float stepSize = dotProduct(maskedDirty, maskedDirty) / gradientDotScratch;
 
     // model_i+1 = model_i + stepsize * gradient
-    factorAdd(structureModel.data(), gradient.data(), stepSize, width, height);
+    factorAdd(structureModel.Data(), gradient.Data(), stepSize, width, height);
 
     // For Dabbech's approach (see below) :
     //  Image scratch2 = maskedDirty;
@@ -394,7 +392,7 @@ bool IUWTDeconvolutionAlgorithm::runConjugateGradient(
     float gradStepDen = dotProduct(maskedDirty, maskedDirty);
     if (gradStepDen == 0.0) return false;
     // residual_i+1 = residual_i - stepsize * scratch
-    factorAdd(maskedDirty.data(), scratch.data(), -stepSize, width, height);
+    factorAdd(maskedDirty.Data(), scratch.Data(), -stepSize, width, height);
 
     // PyMORESANE uses this:
     // gradstep = <residual_i+1, residual_i+1> / <residual_i, residual_i>
@@ -408,14 +406,13 @@ bool IUWTDeconvolutionAlgorithm::runConjugateGradient(
     // gradient_i+1 = residual_i+1 + gradstep * gradient_i
     scratch = gradient;
     gradient = maskedDirty;
-    factorAdd(gradient.data(), scratch.data(), gradStep, width, height);
+    factorAdd(gradient.Data(), scratch.Data(), gradStep, width, height);
 
     // scratch = mask IUWT PSF (x) model
     scratch = structureModel;
-    FFTConvolver::ConvolveSameSize(_fftwManager, scratch.data(),
-                                   psfKernel.data(), width, height,
-                                   _threadPool->size());
-    iuwt.Decompose(*_threadPool, scratch.data(), scratch.data(), false);
+    schaapcommon::fft::Convolve(scratch.Data(), psfKernel.Data(), width, height,
+                                _staticFor->NThreads());
+    iuwt.Decompose(*_staticFor, scratch.Data(), scratch.Data(), false);
     iuwt.ApplyMask(mask);
 
     float previousSNR = modelSNR;
@@ -444,39 +441,16 @@ struct PointSource {
   bool operator<(const PointSource& rhs) const { return flux < rhs.flux; }
 };
 
-void IUWTDeconvolutionAlgorithm::constrainedPSFConvolve(float* image,
-                                                        const float* psf,
-                                                        size_t width,
-                                                        size_t height) {
-  Image smallerPsf(width, height, 0.0), kernel(width, height);
-  size_t s = round(sqrt(_psfResponse[0].convolvedArea * 25.0));
-  size_t smallWidth = std::min(s, width);
-  size_t smallHeight = std::min(s, height);
-  std::cout << "Constrained PSF=" << smallWidth << " x " << smallHeight << '\n';
-  size_t xMin = width / 2 - smallWidth / 2, xMax = width / 2 + smallWidth / 2;
-  size_t yMin = height / 2 - smallHeight / 2,
-         yMax = height / 2 + smallHeight / 2;
-  for (size_t y = yMin; y != yMax; ++y) {
-    for (size_t x = xMin; x != xMax; ++x) {
-      smallerPsf[y * width + x] = psf[y * width + x];
-    }
-  }
-  FFTConvolver::PrepareKernel(kernel.data(), smallerPsf.data(), width, height,
-                              _threadPool->size());
-  FFTConvolver::ConvolveSameSize(_fftwManager, image, kernel.data(), width,
-                                 height, _threadPool->size());
-}
-
 bool IUWTDeconvolutionAlgorithm::findAndDeconvolveStructure(
     IUWTDecomposition& iuwt, Image& dirty, const Image& psf,
-    const Image& psfKernel, Image& scratch, ImageSet& structureModel,
-    size_t curEndScale, size_t curMinScale,
+    const Image& psfKernel, const std::vector<Image>& psfs, Image& scratch,
+    ImageSet& structureModel, size_t curEndScale, size_t curMinScale,
     std::vector<IUWTDeconvolutionAlgorithm::ValComponent>& maxComponents) {
-  iuwt.Decompose(*_threadPool, dirty.data(), scratch.data(), false);
+  iuwt.Decompose(*_staticFor, dirty.Data(), scratch.Data(), false);
   aocommon::UVector<float> thresholds(curEndScale);
   _rmses.resize(curEndScale);
   for (size_t scale = 0; scale != curEndScale; ++scale) {
-    float r = mad(iuwt[scale].Coefficients().data());
+    float r = mad(iuwt[scale].Coefficients().Data());
     _rmses[scale] = r;
     thresholds[scale] = r * (_thresholdSigmaLevel * 4.0 / 5.0);
   }
@@ -546,16 +520,16 @@ bool IUWTDeconvolutionAlgorithm::findAndDeconvolveStructure(
   }
 
   ImageAnalysis::Component maxComp(maxX, maxY, maxValScale);
-  return fillAndDeconvolveStructure(iuwt, dirty, structureModel, scratch, psf,
-                                    psfKernel, curEndScale, curMinScale, _width,
-                                    _height, thresholds, maxComp, true, _mask);
+  return fillAndDeconvolveStructure(
+      iuwt, dirty, structureModel, scratch, psf, psfKernel, psfs, curEndScale,
+      curMinScale, _width, _height, thresholds, maxComp, true, _mask);
 }
 
 bool IUWTDeconvolutionAlgorithm::fillAndDeconvolveStructure(
     IUWTDecomposition& iuwt, Image& dirty, ImageSet& structureModelFull,
     Image& scratch, const Image& psf, const Image& psfKernel,
-    size_t curEndScale, size_t curMinScale, size_t width, size_t height,
-    const aocommon::UVector<float>& thresholds,
+    const std::vector<Image>& psfs, size_t curEndScale, size_t curMinScale,
+    size_t width, size_t height, const aocommon::UVector<float>& thresholds,
     const ImageAnalysis::Component& maxComp, bool allowTrimming,
     const bool* priorMask) {
   IUWTMask mask(curEndScale, width, height);
@@ -581,14 +555,15 @@ bool IUWTDeconvolutionAlgorithm::fillAndDeconvolveStructure(
     std::cout << "Bounding box: (" << x1 << ',' << y1 << ")-(" << x2 << ','
               << y2 << ")\n";
     size_t newWidth = x2 - x1, newHeight = y2 - y1;
-    trim(dirty, dirty, width, height, x1, y1, x2, y2);
+    trim(dirty, dirty, x1, y1, x2, y2);
     Image smallPSF;
 
-    trimPsf(smallPSF, psf.data(), width, height, newWidth, newHeight);
+    trimPsf(smallPSF, psf, newWidth, newHeight);
 
     Image smallPSFKernel(smallPSF.Width(), smallPSF.Height());
-    FFTConvolver::PrepareKernel(smallPSFKernel.data(), smallPSF.data(),
-                                newWidth, newHeight, _threadPool->size());
+    schaapcommon::fft::PrepareConvolutionKernel(
+        smallPSFKernel.Data(), smallPSF.Data(), newWidth, newHeight,
+        _staticFor->NThreads());
 
     scratch = Image(dirty.Width(), dirty.Height());
 
@@ -609,8 +584,8 @@ bool IUWTDeconvolutionAlgorithm::fillAndDeconvolveStructure(
 
     aocommon::UVector<bool> trimmedPriorMask;
     bool* trimmedPriorMaskPtr;
-    if (priorMask == 0)
-      trimmedPriorMaskPtr = 0;
+    if (priorMask == nullptr)
+      trimmedPriorMaskPtr = nullptr;
     else {
       trimmedPriorMask.resize(newWidth * newHeight);
       trimmedPriorMaskPtr = trimmedPriorMask.data();
@@ -622,14 +597,13 @@ bool IUWTDeconvolutionAlgorithm::fillAndDeconvolveStructure(
                                         maxComp.scale);
     bool result = fillAndDeconvolveStructure(
         *trimmedIUWT, dirty, *trimmedStructureModel, scratch, smallPSF,
-        smallPSFKernel, curEndScale, curMinScale, x2 - x1, y2 - y1, thresholds,
-        newMaxComp, false, trimmedPriorMaskPtr);
+        smallPSFKernel, psfs, curEndScale, curMinScale, x2 - x1, y2 - y1,
+        thresholds, newMaxComp, false, trimmedPriorMaskPtr);
     for (size_t i = 0; i != structureModelFull.size(); ++i) {
-      memcpy(scratch.data(), (*trimmedStructureModel)[i],
-             (y2 - y1) * (x2 - x1) * sizeof(float));
+      std::copy_n((*trimmedStructureModel)[i].Data(), (y2 - y1) * (x2 - x1),
+                  scratch.Data());
       untrim(scratch, width, height, x1, y1, x2, y2);
-      memcpy(structureModelFull[i], scratch.data(),
-             width * height * sizeof(float));
+      std::copy_n(scratch.Data(), width * height, structureModelFull.Data(i));
     }
 
     dirty = Image(scratch.Width(), scratch.Height());
@@ -640,14 +614,15 @@ bool IUWTDeconvolutionAlgorithm::fillAndDeconvolveStructure(
     return result;
   } else {
     if (curEndScale <= 3) {
+      // TODO: remove?
       // bool pointSourcesWereFound = extractPointSources(iuwt, mask,
-      // dirty.data(), structureModel.data()); if(pointSourcesWereFound)
+      // dirty.Data(), structureModel.Data()); if(pointSourcesWereFound)
       // return
       // true;
     }
 
     // get undeconvolved dirty
-    iuwt.Decompose(*_threadPool, dirty.data(), scratch.data(), false);
+    iuwt.Decompose(*_staticFor, dirty.Data(), scratch.Data(), false);
 
     iuwt.ApplyMask(mask);
     iuwt.Recompose(scratch, false);
@@ -661,11 +636,10 @@ bool IUWTDeconvolutionAlgorithm::fillAndDeconvolveStructure(
 
     float rmsBefore = rms(dirty);
     scratch = structureModel;
-    FFTConvolver::ConvolveSameSize(_fftwManager, scratch.data(),
-                                   psfKernel.data(), width, height,
-                                   _threadPool->size());
+    schaapcommon::fft::Convolve(scratch.Data(), psfKernel.Data(), width, height,
+                                _staticFor->NThreads());
     maskedDirty = dirty;  // we use maskedDirty as temporary
-    factorAdd(maskedDirty.data(), scratch.data(), -_gain, width, height);
+    factorAdd(maskedDirty.Data(), scratch.Data(), -_gain, width, height);
     float rmsAfter = rms(maskedDirty);
     if (rmsAfter > rmsBefore) {
       std::cout << "RMS got worse: " << rmsBefore << " -> " << rmsAfter << '\n';
@@ -674,7 +648,7 @@ bool IUWTDeconvolutionAlgorithm::fillAndDeconvolveStructure(
 
     // TODO when only one image is available, this is not necessary
     performSubImageFitAll(iuwt, mask, structureModel, scratch, maskedDirty,
-                          maxComp, structureModelFull, psf.data(), dirty);
+                          maxComp, structureModelFull, psf, psfs, dirty);
 
     return true;
   }
@@ -683,42 +657,43 @@ bool IUWTDeconvolutionAlgorithm::fillAndDeconvolveStructure(
 void IUWTDeconvolutionAlgorithm::performSubImageFitAll(
     IUWTDecomposition& iuwt, const IUWTMask& mask, const Image& structureModel,
     Image& scratchA, Image& scratchB, const ImageAnalysis::Component& maxComp,
-    ImageSet& fittedModel, const float* psf, const Image& dirty) {
+    ImageSet& fittedModel, const Image& psf, const std::vector<Image>& psfs,
+    const Image& dirty) {
   size_t width = iuwt.Width(), height = iuwt.Height();
 
   if (_dirtySet->size() == 1) {
     // With only one image, we don't have to refit
     Image img(width, height);
-    std::copy_n(structureModel.data(), width * height, img.begin());
+    std::copy_n(structureModel.Data(), width * height, img.begin());
     fittedModel.SetImage(0, std::move(img));
   } else {
     std::cout << "Fitting structure in images: " << std::flush;
     aocommon::UVector<float> correctionFactors;
     scratchA = dirty;
     performSubImageFitSingle(iuwt, mask, structureModel, scratchB, maxComp, psf,
-                             scratchA, 0, correctionFactors);
+                             scratchA, nullptr, correctionFactors);
 
     fittedModel = 0.0;
 
     for (size_t imgIndex = 0; imgIndex != _dirtySet->size(); ++imgIndex) {
       std::cout << '.' << std::flush;
-      const float* subPsf = _psfs[_dirtySet->PSFIndex(imgIndex)];
+      const aocommon::Image& subPsf = psfs[_dirtySet->PSFIndex(imgIndex)];
 
-      trim(scratchA, (*_dirtySet)[imgIndex], _width, _height, _curBoxXStart,
-           _curBoxYStart, _curBoxXEnd, _curBoxYEnd);
+      trim(scratchA, (*_dirtySet)[imgIndex], _curBoxXStart, _curBoxYStart,
+           _curBoxXEnd, _curBoxYEnd);
 
       Image smallSubPsf;
-      const float* subPsfData;
+      const Image* subPsfImage;
       if (_width != width || _height != height) {
-        trimPsf(smallSubPsf, subPsf, _width, _height, width, height);
-        subPsfData = smallSubPsf.data();
+        trimPsf(smallSubPsf, subPsf, width, height);
+        subPsfImage = &smallSubPsf;
       } else {
-        subPsfData = subPsf;
+        subPsfImage = &subPsf;
       }
 
       performSubImageFitSingle(iuwt, mask, structureModel, scratchB, maxComp,
-                               subPsfData, scratchA, fittedModel[imgIndex],
-                               correctionFactors);
+                               *subPsfImage, scratchA,
+                               fittedModel.Data(imgIndex), correctionFactors);
     }
     std::cout << '\n';
   }
@@ -726,21 +701,21 @@ void IUWTDeconvolutionAlgorithm::performSubImageFitAll(
 
 void IUWTDeconvolutionAlgorithm::performSubImageFitSingle(
     IUWTDecomposition& iuwt, const IUWTMask& mask, const Image& structureModel,
-    Image& scratchB, const ImageAnalysis::Component& maxComp, const float* psf,
+    Image& scratchB, const ImageAnalysis::Component& maxComp, const Image& psf,
     Image& subDirty, float* fittedSubModel,
     aocommon::UVector<float>& correctionFactors) {
   size_t width = iuwt.Width(), height = iuwt.Height();
 
   Image psfKernel(width, height);
-  FFTConvolver::PrepareKernel(psfKernel.data(), psf, width, height,
-                              _threadPool->size());
+  schaapcommon::fft::PrepareConvolutionKernel(
+      psfKernel.Data(), psf.Data(), width, height, _staticFor->NThreads());
 
   Image& maskedDirty = scratchB;
 
-  iuwt.Decompose(*_threadPool, subDirty.data(), subDirty.data(), false);
+  iuwt.Decompose(*_staticFor, subDirty.Data(), subDirty.Data(), false);
   iuwt.ApplyMask(mask);
   iuwt.Recompose(maskedDirty, false);
-  aocommon::UVector<bool> mask2d(structureModel.size(), false);
+  aocommon::UVector<bool> mask2d(structureModel.Size(), false);
   float peakLevel = std::fabs(structureModel[maxComp.y * width + maxComp.x]);
   size_t componentIndex = 0;
   for (size_t y = 0; y != height; ++y) {
@@ -750,7 +725,7 @@ void IUWTDeconvolutionAlgorithm::performSubImageFitSingle(
       if (!maskRow[x] && std::fabs(modelRow[x]) > peakLevel * 1e-4) {
         std::vector<ImageAnalysis::Component2D> area;
         ImageAnalysis::Component2D comp(x, y);
-        ImageAnalysis::FloodFill2D(structureModel.data(), mask2d.data(),
+        ImageAnalysis::FloodFill2D(structureModel.Data(), mask2d.data(),
                                    peakLevel * 1e-4, comp, width, height, area);
         // Find bounding box and copy active pixels to subDirty
         subDirty = Image(width, height, 0.0);
@@ -774,7 +749,7 @@ void IUWTDeconvolutionAlgorithm::performSubImageFitSingle(
         // if no fittedSubModel was given, we just need to store the factors.
         // Otherwise, scale the deconvolved model and add it to the contaminated
         // model.
-        if (fittedSubModel != 0) {
+        if (fittedSubModel != nullptr) {
           const float integratedFactor = correctionFactors[componentIndex];
           if (std::isfinite(factor) && std::isfinite(integratedFactor) &&
               integratedFactor != 0.0) {
@@ -798,24 +773,26 @@ void IUWTDeconvolutionAlgorithm::performSubImageFitSingle(
 float IUWTDeconvolutionAlgorithm::performSubImageComponentFitBoxed(
     IUWTDecomposition& iuwt, const IUWTMask& mask,
     const std::vector<ImageAnalysis::Component2D>& area, Image& model,
-    Image& maskedDirty, const float* psf, const Image& psfKernel, size_t x1,
+    Image& maskedDirty, const Image& psf, const Image& psfKernel, size_t x1,
     size_t y1, size_t x2, size_t y2) {
-  const size_t width = iuwt.Width(), height = iuwt.Height();
+  const size_t width = iuwt.Width();
+  const size_t height = iuwt.Height();
   if (x1 > 0 || y1 > 0 || x2 < width || y2 < height) {
     size_t newWidth = x2 - x1, newHeight = y2 - y1;
     IUWTDecomposition smallIUWTW(iuwt.NScales(), newWidth, newHeight);
     std::unique_ptr<IUWTMask> smallMask(mask.CreateTrimmed(x1, y1, x2, y2));
     Image smallModel;
-    trim(smallModel, model, width, height, x1, y1, x2, y2);
+    trim(smallModel, model, x1, y1, x2, y2);
 
     Image smallPsf;
-    trimPsf(smallPsf, psf, width, height, newWidth, newHeight);
+    trimPsf(smallPsf, psf, newWidth, newHeight);
     Image smallPsfKernel(smallPsf.Width(), smallPsf.Height());
-    FFTConvolver::PrepareKernel(smallPsfKernel.data(), smallPsf.data(),
-                                newWidth, newHeight, _threadPool->size());
+    schaapcommon::fft::PrepareConvolutionKernel(
+        smallPsfKernel.Data(), smallPsf.Data(), newWidth, newHeight,
+        _staticFor->NThreads());
 
     Image smallMaskedDirty;
-    trim(smallMaskedDirty, maskedDirty, width, height, x1, y1, x2, y2);
+    trim(smallMaskedDirty, maskedDirty, x1, y1, x2, y2);
 
     float factor =
         performSubImageComponentFit(smallIUWTW, *smallMask, area, smallModel,
@@ -834,9 +811,9 @@ float IUWTDeconvolutionAlgorithm::performSubImageComponentFit(
     size_t yOffset) {
   const size_t width = iuwt.Width(), height = iuwt.Height();
   // Calculate IUWT^-1 mask IUWT model (x) PSF
-  FFTConvolver::ConvolveSameSize(_fftwManager, model.data(), psfKernel.data(),
-                                 width, height, _threadPool->size());
-  iuwt.Decompose(*_threadPool, model.data(), model.data(), false);
+  schaapcommon::fft::Convolve(model.Data(), psfKernel.Data(), width, height,
+                              _staticFor->NThreads());
+  iuwt.Decompose(*_staticFor, model.Data(), model.Data(), false);
   iuwt.ApplyMask(mask);
   iuwt.Recompose(model, false);
 
@@ -857,16 +834,15 @@ float IUWTDeconvolutionAlgorithm::performSubImageComponentFit(
 
 float IUWTDeconvolutionAlgorithm::PerformMajorIteration(
     size_t& iterCounter, size_t nIter, ImageSet& modelSet, ImageSet& dirtySet,
-    const aocommon::UVector<const float*>& psfs, bool& reachedMajorThreshold) {
-  std::unique_ptr<ThreadPool> threadPool(new ThreadPool());
-  _threadPool = &*threadPool;
+    const std::vector<aocommon::Image>& psfs, bool& reachedMajorThreshold) {
+  aocommon::StaticFor<size_t> static_for(aocommon::system::ProcessorCount());
+  _staticFor = &static_for;
 
   reachedMajorThreshold = false;
   if (iterCounter == nIter) return 0.0;
 
   _modelSet = &modelSet;
   _dirtySet = &dirtySet;
-  _psfs = psfs;
 
   _curBoxXStart = 0;
   _curBoxXEnd = _width;
@@ -883,26 +859,23 @@ float IUWTDeconvolutionAlgorithm::PerformMajorIteration(
 
   // Prepare the PSF for convolutions later on
   Image psfKernel(_width, _height);
-  FFTConvolver::PrepareKernel(psfKernel.data(), psf.data(), _width, _height,
-                              _threadPool->size());
+  schaapcommon::fft::PrepareConvolutionKernel(
+      psfKernel.Data(), psf.Data(), _width, _height, static_for.NThreads());
 
   std::cout << "Measuring PSF...\n";
   {
     Image convolvedPSF(psf);
     Image scratch(_width, _height);
 
-    FFTConvolver::ConvolveSameSize(_fftwManager, convolvedPSF.data(),
-                                   psfKernel.data(), _width, _height,
-                                   _threadPool->size());
-    measureRMSPerScale(psf.data(), convolvedPSF.data(), scratch.data(),
+    schaapcommon::fft::Convolve(convolvedPSF.Data(), psfKernel.Data(), _width,
+                                _height, static_for.NThreads());
+    measureRMSPerScale(psf.Data(), convolvedPSF.Data(), scratch.Data(),
                        maxScale, _psfResponse);
   }
 
-  ImageSet structureModel(&modelSet.Table(), modelSet.Settings(), _width,
-                          _height);
+  ImageSet structureModel(modelSet, _width, _height);
 
-  std::unique_ptr<IUWTDecomposition> iuwt(
-      new IUWTDecomposition(curEndScale, _width, _height));
+  auto iuwt = std::make_unique<IUWTDecomposition>(curEndScale, _width, _height);
 
   Image dirtyBeforeIteration;
 
@@ -914,13 +887,13 @@ float IUWTDeconvolutionAlgorithm::PerformMajorIteration(
   do {
     std::cout << "*** Deconvolution iteration " << iterCounter << " ***\n";
     dirtyBeforeIteration = dirty;
-    FFTConvolver::PrepareKernel(psfKernel.data(), psf.data(), _width, _height,
-                                _threadPool->size());
+    schaapcommon::fft::PrepareConvolutionKernel(
+        psfKernel.Data(), psf.Data(), _width, _height, static_for.NThreads());
     std::vector<ValComponent> maxComponents;
     Image scratch(_width, _height);
     bool succeeded = findAndDeconvolveStructure(
-        *iuwt, dirty, psf, psfKernel, scratch, structureModel, curEndScale,
-        curMinScale, maxComponents);
+        *iuwt, dirty, psf, psfKernel, psfs, scratch, structureModel,
+        curEndScale, curMinScale, maxComponents);
 
     if (succeeded) {
       structureModel *= _gain;
@@ -928,14 +901,14 @@ float IUWTDeconvolutionAlgorithm::PerformMajorIteration(
 
       // Calculate: dirty = dirty - structureModel (x) psf
       for (size_t i = 0; i != dirtySet.size(); ++i) {
-        scratch.Assign(structureModel[i], structureModel[i] + _width * _height);
+        scratch = structureModel[i];
         size_t psfIndex = dirtySet.PSFIndex(i);
-        FFTConvolver::PrepareKernel(psfKernel.data(), psfs[psfIndex], _width,
-                                    _height, _threadPool->size());
-        FFTConvolver::ConvolveSameSize(_fftwManager, scratch.data(),
-                                       psfKernel.data(), _width, _height,
-                                       _threadPool->size());
-        Subtract(dirtySet[i], scratch);
+        schaapcommon::fft::PrepareConvolutionKernel(
+            psfKernel.Data(), psfs[psfIndex].Data(), _width, _height,
+            static_for.NThreads());
+        schaapcommon::fft::Convolve(scratch.Data(), psfKernel.Data(), _width,
+                                    _height, static_for.NThreads());
+        Subtract(dirtySet.Data(i), scratch);
       }
       dirtySet.GetLinearIntegrated(dirty);
 
