@@ -3,25 +3,28 @@
 
 #include <wscversion.h>
 
-#include "../io/logger.h"
-
-#include "../units/angle.h"
-#include "../units/fluxdensity.h"
-
 #include "../structures/numberlist.h"
 
 #include <aocommon/fits/fitswriter.h>
+#include <aocommon/logger.h>
 #include <aocommon/radeccoord.h>
+#include <aocommon/units/angle.h>
+#include <aocommon/units/fluxdensity.h>
 
+#include <schaapcommon/fitters/spectralfitter.h>
 #include <schaapcommon/h5parm/jonesparameters.h>
 
 #include <boost/algorithm/string.hpp>
-#include <boost/optional/optional.hpp>
 
 #include <iostream>
+#include <optional>
 #include <string>
 #include <sstream>
 
+using aocommon::Logger;
+using aocommon::units::Angle;
+using aocommon::units::FluxDensity;
+
 void CommandLine::printHelp() {
   std::cout
       << "Syntax: wsclean [options] <input-ms> [<2nd-ms> [..]]\n"
@@ -116,18 +119,38 @@ void CommandLine::printHelp() {
          "-save-psf-pb\n"
          "   When applying beam correction, also save the primary-beam "
          "corrected PSF image.\n"
-         "-pb-undersampling <factor>\n"
-         "   Normally, the primary beam is calculated at a lower resolution "
-         "and interpolated, because calculating the beam is\n"
-         "   computationally expensive. The amount of undersampling can be "
-         "controlled by this parameter, or set to '1' for no\n"
-         "   undersampling. Default: 8.\n"
-         "-primary-beam-model\n"
+         "-pb-grid-size <npixel>\n"
+         "   Specify the grid size in number of pixels at which to evaluate "
+         "   the primary beam.\n"
+         "   Typically, the primary beam is calculated at a coarse resolution "
+         "grid \n"
+         "   and interpolated, to reduce the time spent in evaluating the "
+         "beam. \n"
+         "   This parameter controls the resolution of the grid at which to "
+         "evaluate \n"
+         "   the primary beam. For rectangular images, pb-grid-size \n"
+         "   indicates the number of pixels along the shortest dimension. \n"
+         "   The total number of pixels in the primary beam grid thus amounts "
+         "to: \n\n"
+         "   max(width, height) / min(width, height) * pb-grid-size**2. \n\n"
+         "   Default: 32.\n"
+         "-beam-model\n"
          "   Specify the beam model, only relevant for SKA and LOFAR. "
          "Available "
          "models are Hamaker, Lobes, OskarDipole, OskarSphericalWave.\n"
          "   Input is case insensitive. Default is Hamaker for LOFAR and\n"
          "   OskarSphericalWave for SKA.\n"
+         "-beam-mode\n"
+         "   [DEBUGGING ONLY] Manually specify the beam mode. Only relevant "
+         "for simulated SKA measurement sets. \n"
+         "   Available modes are array_factor, element and full.\n"
+         "   Input is case insensitive. Default is full.\n"
+         "-beam-normalisation-mode\n"
+         "    [DEBUGGING ONLY] Manually specify the normalisation of the beam. "
+         "Only "
+         "relevant for simulated SKA measurement sets. \n"
+         "    Available modes are none, preapplied, full, and amplitude. "
+         "Default is preapplied.\n"
          "-dry-run\n"
          "   Parses the command line and quits afterwards. No imaging is "
          "done.\n"
@@ -526,7 +549,7 @@ void CommandLine::printHelp() {
          "Useful to start at high\n"
          "   levels and go down with subsequent loops, e.g. 20,10,5\n"
          "-save-source-list\n"
-         "   Saves the found clean components as a BBS/NDPPP text sky model. "
+         "   Saves the found clean components as a BBS/DP3 text sky model. "
          "This parameter \n"
          "   enables Gaussian shapes during multi-scale cleaning "
          "(-multiscale-shape gaussian).\n"
@@ -580,8 +603,10 @@ void CommandLine::printHelp() {
          "the given size.\n"
          "-deconvolution-threads <n>\n"
          "   Number of threads to use during deconvolution. On machines with "
-         "   a large nr of cores, this may be used to decrease the memory "
+         "a large nr of cores, this may be used to decrease the memory "
          "usage.\n"
+         "   If not specified, the number of threads during deconvolution "
+         "is controlled with the -j option.\n"
          "\n"
          "  ** RESTORATION OPTIONS **\n"
          "-restore <input residual> <input model> <output image>\n"
@@ -662,6 +687,12 @@ size_t CommandLine::parse_size_t(const char* param, const char* name) {
   return v;
 }
 
+std::vector<std::string> CommandLine::parseStringList(const char* param) {
+  std::vector<std::string> list;
+  boost::split(list, param, [](char c) { return c == ','; });
+  return list;
+}
+
 double CommandLine::parse_double(const char* param, const char* name) {
   char* endptr;
   double v = std::strtod(param, &endptr);
@@ -702,8 +733,8 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
   Settings& settings = wsclean.GetSettings();
   int argi = 1;
   bool mfWeighting = false, noMFWeighting = false, dryRun = false;
-  auto atermKernelSize = boost::make_optional<double>(false, 0.0);
-  Logger::SetVerbosity(Logger::NormalVerbosity);
+  auto atermKernelSize = std::make_optional<double>();
+  Logger::SetVerbosity(Logger::kNormalVerbosity);
   while (argi < argc && argv[argi][0] == '-') {
     const std::string param =
         argv[argi][1] == '-' ? (&argv[argi][2]) : (&argv[argi][1]);
@@ -716,9 +747,7 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
 #ifdef HAVE_IDG
         Logger::Info << "IDG is available.\n";
 #endif
-#ifdef HAVE_WGRIDDER
         Logger::Info << "WGridder is available.\n";
-#endif
       }
       return false;
     } else if (param == "help") {
@@ -728,9 +757,9 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       }
       return false;
     } else if (param == "quiet") {
-      Logger::SetVerbosity(Logger::QuietVerbosity);
+      Logger::SetVerbosity(Logger::kQuietVerbosity);
     } else if (param == "v" || param == "verbose") {
-      Logger::SetVerbosity(Logger::VerboseVerbosity);
+      Logger::SetVerbosity(Logger::kVerboseVerbosity);
     } else if (param == "log-time") {
       Logger::SetLogTime(true);
     } else if (param == "temp-dir" || param == "tempdir") {
@@ -776,7 +805,7 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
     } else if (param == "scale") {
       ++argi;
       settings.pixelScaleX =
-          Angle::Parse(argv[argi], "scale parameter", Angle::Degrees);
+          Angle::Parse(argv[argi], "scale parameter", Angle::kDegrees);
       settings.pixelScaleY = settings.pixelScaleX;
     } else if (param == "nwlayers") {
       ++argi;
@@ -806,7 +835,7 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
     } else if (param == "threshold") {
       ++argi;
       settings.deconvolutionThreshold = FluxDensity::Parse(
-          argv[argi], "threshold parameter", FluxDensity::Jansky);
+          argv[argi], "threshold parameter", FluxDensity::kJansky);
     } else if (param == "auto-threshold") {
       ++argi;
       settings.autoDeconvolutionThreshold = true;
@@ -817,19 +846,19 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       settings.autoMask = true;
       settings.autoMaskSigma = parse_double(argv[argi], 0.0, "auto-mask");
     } else if (param == "local-rms" || param == "rms-background") {
-      settings.localRMS = true;
+      settings.localRMSMethod = LocalRmsMethod::kRmsWindow;
       if (param == "rms-background") deprecated(isSlave, param, "local-rms");
     } else if (param == "local-rms-window" ||
                param == "rms-background-window") {
       ++argi;
-      settings.localRMS = true;
+      settings.localRMSMethod = LocalRmsMethod::kRmsWindow;
       settings.localRMSWindow =
           parse_double(argv[argi], 0.0, "local-rms-window", false);
       if (param == "rms-background-window")
         deprecated(isSlave, param, "local-rms-window");
     } else if (param == "local-rms-image" || param == "rms-background-image") {
       ++argi;
-      settings.localRMS = true;
+      settings.localRMSMethod = LocalRmsMethod::kRmsWindow;
       settings.localRMSImage = argv[argi];
       if (param == "rms-background-image")
         deprecated(isSlave, param, "local-rms-image");
@@ -837,11 +866,10 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
                param == "rms-background-method") {
       ++argi;
       std::string method = argv[argi];
-      settings.localRMS = true;
       if (method == "rms")
-        settings.localRMSMethod = Settings::RMSWindow;
+        settings.localRMSMethod = LocalRmsMethod::kRmsWindow;
       else if (method == "rms-with-min")
-        settings.localRMSMethod = Settings::RMSAndMinimumWindow;
+        settings.localRMSMethod = LocalRmsMethod::kRmsAndMinimumWindow;
       else
         throw std::runtime_error("Unknown RMS background method specified");
       if (param == "rms-background-method")
@@ -865,12 +893,42 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
             "Invalid beam-model: should be either Hamaker, Lobes, OskarDipole "
             "or OskarSphericalWave (case insensitive)");
       }
+    } else if (param == "beam-mode") {
+      ++argi;
+      std::string beamMode = argv[argi];
+      boost::to_upper(beamMode);
+      if (beamMode == "ARRAY_FACTOR" || beamMode == "ELEMENT" ||
+          beamMode == "FULL") {
+        settings.beamMode = beamMode;
+      } else {
+        throw std::runtime_error(
+            "Invalid beam-mode: should be either array_factor, element or full "
+            "(case insensitive)");
+      }
+    } else if (param == "beam-normalisation-mode") {
+      ++argi;
+      std::string beamNormalisationMode = argv[argi];
+      boost::to_upper(beamNormalisationMode);
+      if (beamNormalisationMode == "NONE" ||
+          beamNormalisationMode == "PREAPPLIED" ||
+          beamNormalisationMode == "AMPLITUDE" ||
+          beamNormalisationMode == "FULL") {
+        settings.beamNormalisationMode = beamNormalisationMode;
+      } else {
+        throw std::runtime_error(
+            "Invalid beam-normalisation-mode: should be either none, "
+            "preapplied, amplitude or full "
+            "(case insensitive)");
+      }
     } else if (param == "apply-primary-beam") {
       settings.applyPrimaryBeam = true;
     } else if (param == "reuse-primary-beam") {
       settings.reusePrimaryBeam = true;
     } else if (param == "use-differential-lofar-beam") {
-      settings.useDifferentialLofarBeam = true;
+      // pre_applied_or_full is the beam normalisation mode
+      // that implements the behaviour of
+      // the old use_differential_beam option of EveryBeam
+      settings.beamNormalisationMode = "preapplied_or_full";
     } else if (param == "primary-beam-limit") {
       ++argi;
       settings.primaryBeamLimit =
@@ -882,10 +940,9 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       dryRun = true;
     } else if (param == "save-psf-pb") {
       settings.savePsfPb = true;
-    } else if (param == "pb-undersampling") {
+    } else if (param == "pb-grid-size") {
       ++argi;
-      settings.primaryBeamUndersampling =
-          parse_size_t(argv[argi], "pb-undersampling");
+      settings.primaryBeamGridSize = parse_size_t(argv[argi], "pb-grid-size");
     } else if (param == "negative") {
       settings.allowNegativeComponents = true;
     } else if (param == "no-negative" || param == "nonegative") {
@@ -1026,7 +1083,7 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
     } else if (param == "taper-gaussian") {
       ++argi;
       double taperBeamSize =
-          Angle::Parse(argv[argi], "Gaussian taper", Angle::Arcseconds);
+          Angle::Parse(argv[argi], "Gaussian taper", Angle::kArcseconds);
       settings.gaussianTaperBeamSize = taperBeamSize;
     } else if (param == "taper-edge") {
       ++argi;
@@ -1110,15 +1167,17 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       ++argi;
       settings.horizonMask = true;
       settings.horizonMaskDistance =
-          Angle::Parse(argv[argi], "horizon mask distance", Angle::Degrees);
+          Angle::Parse(argv[argi], "horizon mask distance", Angle::kDegrees);
     } else if (param == "fit-spectral-pol") {
       ++argi;
-      settings.spectralFittingMode = SpectralFittingMode::Polynomial;
+      settings.spectralFittingMode =
+          schaapcommon::fitters::SpectralFittingMode::Polynomial;
       settings.spectralFittingTerms =
           parse_size_t(argv[argi], "fit-spectral-pol");
     } else if (param == "fit-spectral-log-pol") {
       ++argi;
-      settings.spectralFittingMode = SpectralFittingMode::LogPolynomial;
+      settings.spectralFittingMode =
+          schaapcommon::fitters::SpectralFittingMode::LogPolynomial;
       settings.spectralFittingTerms =
           parse_size_t(argv[argi], "fit-spectral-log-pol");
     } else if (param == "force-spectrum") {
@@ -1179,18 +1238,18 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       argi += 3;
     } else if (param == "beam-size" || param == "beamsize") {
       ++argi;
-      double beam = Angle::Parse(argv[argi], "beam size", Angle::Arcseconds);
+      double beam = Angle::Parse(argv[argi], "beam size", Angle::kArcseconds);
       settings.manualBeamMajorSize = beam;
       settings.manualBeamMinorSize = beam;
       settings.manualBeamPA = 0.0;
       if (param == "beamsize") deprecated(isSlave, param, "beam-size");
     } else if (param == "beam-shape" || param == "beamshape") {
       double beamMaj = Angle::Parse(argv[argi + 1], "beam shape, major axis",
-                                    Angle::Arcseconds);
+                                    Angle::kArcseconds);
       double beamMin = Angle::Parse(argv[argi + 2], "beam shape, minor axis",
-                                    Angle::Arcseconds);
+                                    Angle::kArcseconds);
       double beamPA = Angle::Parse(argv[argi + 3], "beam shape, position angle",
-                                   Angle::Degrees);
+                                   Angle::kDegrees);
       argi += 3;
       settings.manualBeamMajorSize = beamMaj;
       settings.manualBeamMinorSize = beamMin;
@@ -1301,10 +1360,14 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       atermKernelSize = parse_double(argv[argi], 0.0, "aterm-kernel-size");
     } else if (param == "apply-facet-solutions") {
       ++argi;
-      settings.facetSolutionFile = argv[argi];
+      settings.facetSolutionFiles = parseStringList(argv[argi]);
       ++argi;
-      settings.facetSolutionTables =
-          schaapcommon::h5parm::JonesParameters::ParseList(argv[argi]);
+      settings.facetSolutionTables = parseStringList(argv[argi]);
+      if (settings.facetSolutionTables.size() > 2) {
+        throw std::runtime_error(
+            "List of solution tables (soltabs) should contain at most two "
+            "entries.");
+      }
     } else if (param == "apply-facet-beam") {
       settings.applyFacetBeam = true;
     } else if (param == "facet-beam-update") {
@@ -1335,9 +1398,7 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
     } else if (param == "direct-ft-precision") {
       ++argi;
       std::string precStr = argv[argi];
-      if (precStr == "half")
-        settings.directFTPrecision = DirectFTPrecision::Half;
-      else if (precStr == "float")
+      if (precStr == "float")
         settings.directFTPrecision = DirectFTPrecision::Float;
       else if (precStr == "double")
         settings.directFTPrecision = DirectFTPrecision::Double;
@@ -1394,8 +1455,8 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
   // and possibly set to quiet.
   if (!isSlave) printHeader();
 
-  size_t defaultAtermSize = settings.atermConfigFilename.empty() ? 5 : 16;
-  settings.atermKernelSize = atermKernelSize.get_value_or(defaultAtermSize);
+  const size_t defaultAtermSize = settings.atermConfigFilename.empty() ? 5 : 16;
+  settings.atermKernelSize = atermKernelSize.value_or(defaultAtermSize);
 
   settings.mfWeighting =
       (settings.joinedFrequencyDeconvolution && !noMFWeighting) || mfWeighting;
@@ -1422,7 +1483,7 @@ void CommandLine::Validate(WSClean& wsclean) {
 }
 
 void CommandLine::Run(class WSClean& wsclean) {
-  Settings& settings = wsclean.GetSettings();
+  const Settings& settings = wsclean.GetSettings();
   switch (settings.mode) {
     case Settings::RestoreMode:
       WSCFitsWriter::Restore(settings);
