@@ -1,9 +1,18 @@
 #include "settings.h"
 
-#include "../io/logger.h"
+#include "../io/facetreader.h"
+
+#include <aocommon/logger.h>
+
+#include <schaapcommon/h5parm/h5parm.h>
+
+#include <casacore/ms/MeasurementSets/MeasurementSet.h>
+#include <casacore/tables/Tables/TableRecord.h>
 
 #include <sstream>
 
+using aocommon::Logger;
+
 void Settings::Validate() const {
   if (mode == ImagingMode) {
     if (trimmedImageWidth == 0 && trimmedImageHeight == 0)
@@ -42,7 +51,7 @@ void Settings::Validate() const {
   }
 
   if (facetRegionFilename.empty()) {
-    if (!facetSolutionFile.empty())
+    if (!facetSolutionFiles.empty())
       throw std::runtime_error(
           "A facet solution file can only be specified in conjunction with a "
           "facet regions file. Either remove -apply-facet-solutions from the "
@@ -65,12 +74,36 @@ void Settings::Validate() const {
       }
       // This condition might become a bit more specific once xx,yy polarization
       // correction for h5 AND beam are implemented
-      if (applyFacetBeam && !facetSolutionFile.empty()) {
+      if (applyFacetBeam && !facetSolutionFiles.empty()) {
         throw std::runtime_error(
             "Applying h5parm solutions AND beam correction on multiple "
             "polarizations is not yet supported.");
       }
     }
+
+    if (!facetSolutionFiles.empty()) {
+      if (facetSolutionFiles.size() != 1 &&
+          facetSolutionFiles.size() != filenames.size()) {
+        throw std::runtime_error(
+            "Incorrect number of facet solution files provided. The number of "
+            "facet solution files should be either 1 or match the number of "
+            "input measurement sets.");
+      }
+
+      const std::size_t nfacets =
+          FacetReader::ReadFacets(facetRegionFilename).size();
+      for (const std::string& facetSolutionFile : facetSolutionFiles) {
+        schaapcommon::h5parm::H5Parm h5parm =
+            schaapcommon::h5parm::H5Parm(facetSolutionFile);
+        const size_t nsources = h5parm.GetNumSources();
+        if (nsources != nfacets) {
+          throw std::runtime_error(
+              "Number of source directions in one of the h5 facet solution "
+              "files does not match the number of facets in the facet "
+              "definition file.");
+        }
+      }
+    }
   }
 
   if (useIDG) {
@@ -107,10 +140,30 @@ void Settings::Validate() const {
           "IDG cannot apply facet based beam corrections. Remove facet related "
           "command line arguments and use -grid-with-beam "
           "instead.");
-    if (!facetSolutionFile.empty())
+    if (!facetSolutionFiles.empty())
       throw std::runtime_error(
           "IDG cannot apply facet based direction dependent corrections. "
           "Remove -apply-facet-solution from the command line instruction.");
+
+    if (baselineDependentAveragingInWavelengths != 0.0) {
+      throw std::runtime_error(
+          "IDG cannot be combined with (internally computed) "
+          "baseline-dependent averaging. Please remove baseline-averaging "
+          "option from your command.");
+    }
+
+    for (const auto& filename : filenames) {
+      casacore::MeasurementSet ms(filename);
+      const std::string& bda_factors = "BDA_FACTORS";
+      const bool has_bda = ms.keywordSet().isDefined(bda_factors) &&
+                           (ms.keywordSet().asTable(bda_factors).nrow() > 0);
+      if (has_bda) {
+        throw std::runtime_error(
+            "IDG cannot be combined with the baseline-dependently averaged "
+            "measurement set " +
+            filename);
+      }
+    }
   }
 
   if (gridWithBeam && !useIDG)
@@ -128,11 +181,6 @@ void Settings::Validate() const {
   if (!useIDG && !atermConfigFilename.empty())
     throw std::runtime_error(
         "Use of an aterm config file required IDG enabled: add -use-idg");
-  if (useDifferentialLofarBeam && !(gridWithBeam || applyPrimaryBeam))
-    throw std::runtime_error(
-        "Differential beam correction was requested, but no beam correction is "
-        "applied. Use either IDG and grid with the beam, or apply the average "
-        "beam.");
 
   if (baselineDependentAveragingInWavelengths != 0.0) {
     if (forceNoReorder)
@@ -167,7 +215,8 @@ void Settings::Validate() const {
 
   if (deconvolutionChannelCount != 0 &&
       deconvolutionChannelCount != channelsOut &&
-      spectralFittingMode == SpectralFittingMode::NoFitting)
+      spectralFittingMode ==
+          schaapcommon::fitters::SpectralFittingMode::NoFitting)
     throw std::runtime_error(
         "You have requested to deconvolve with a decreased number of channels "
         "(-deconvolution-channels), but you have not enabled spectral fitting. "
@@ -192,13 +241,22 @@ void Settings::Validate() const {
     throw std::runtime_error("A source list cannot be saved without cleaning.");
 
   if (!forcedSpectrumFilename.empty() &&
-      (spectralFittingMode != SpectralFittingMode::LogPolynomial ||
+      (spectralFittingMode !=
+           schaapcommon::fitters::SpectralFittingMode::LogPolynomial ||
        spectralFittingTerms != 2))
     throw std::runtime_error(
         "When using forced spectrum mode, currently it is required to fit "
         "logarithmic polynomials (i.e. spectral index + further terms). This "
         "implies you have to specify -fit-spectral-log-pol 2.");
 
+  if (parallelGridding != 1 &&
+      (applyFacetBeam || !facetSolutionFiles.empty()) &&
+      !schaapcommon::h5parm::H5Parm::IsThreadSafe()) {
+    throw std::runtime_error(
+        "Parallel gridding in combination with a facet beam or facet solutions,"
+        " requires an HDF5 library that supports multi-threading.");
+  }
+
   checkPolarizations();
 }
 
@@ -271,6 +329,10 @@ void Settings::Propagate(bool verbose) {
                  << trimmedImageHeight << '\n';
   }
 
+  if (parallelDeconvolutionMaxThreads == 0) {
+    parallelDeconvolutionMaxThreads = threadCount;
+  }
+
   // When using IDG with aterms, a PSF must be made, because the beam
   // image is created during the PSF imaging stage.
   if (useIDG && (!atermConfigFilename.empty() || gridWithBeam)) {
@@ -303,6 +365,78 @@ void Settings::RecalculatePaddedDimensions(bool verbose) {
   }
 }
 
+DeconvolutionSettings Settings::GetDeconvolutionSettings() const {
+  DeconvolutionSettings deconvolutionSettings;
+
+  deconvolutionSettings.trimmedImageWidth = trimmedImageWidth;
+  deconvolutionSettings.trimmedImageHeight = trimmedImageHeight;
+  deconvolutionSettings.channelsOut = channelsOut;
+  deconvolutionSettings.pixelScaleX = pixelScaleX;
+  deconvolutionSettings.pixelScaleY = pixelScaleY;
+  deconvolutionSettings.threadCount = threadCount;
+  deconvolutionSettings.prefixName = prefixName;
+
+  deconvolutionSettings.linkedPolarizations = linkedPolarizations;
+  deconvolutionSettings.parallelDeconvolutionMaxSize =
+      parallelDeconvolutionMaxSize;
+  deconvolutionSettings.parallelDeconvolutionMaxThreads =
+      parallelDeconvolutionMaxThreads;
+
+  deconvolutionSettings.deconvolutionThreshold = deconvolutionThreshold;
+  deconvolutionSettings.deconvolutionGain = deconvolutionGain;
+  deconvolutionSettings.deconvolutionMGain = deconvolutionMGain;
+  deconvolutionSettings.autoDeconvolutionThreshold = autoDeconvolutionThreshold;
+  deconvolutionSettings.autoMask = autoMask;
+  deconvolutionSettings.autoDeconvolutionThresholdSigma =
+      autoDeconvolutionThresholdSigma;
+  deconvolutionSettings.autoMaskSigma = autoMaskSigma;
+  deconvolutionSettings.localRMSMethod = localRMSMethod;
+  deconvolutionSettings.localRMSWindow = localRMSWindow;
+  deconvolutionSettings.localRMSImage = localRMSImage;
+  deconvolutionSettings.saveSourceList = saveSourceList;
+  deconvolutionSettings.deconvolutionIterationCount =
+      deconvolutionIterationCount;
+  deconvolutionSettings.majorIterationCount = majorIterationCount;
+  deconvolutionSettings.allowNegativeComponents = allowNegativeComponents;
+  deconvolutionSettings.stopOnNegativeComponents = stopOnNegativeComponents;
+  deconvolutionSettings.useMultiscale = useMultiscale;
+  deconvolutionSettings.useSubMinorOptimization = useSubMinorOptimization;
+  deconvolutionSettings.squaredJoins = squaredJoins;
+  deconvolutionSettings.spectralCorrectionFrequency =
+      spectralCorrectionFrequency;
+  deconvolutionSettings.spectralCorrection = spectralCorrection;
+  deconvolutionSettings.multiscaleFastSubMinorLoop = multiscaleFastSubMinorLoop;
+  deconvolutionSettings.multiscaleGain = multiscaleGain;
+  deconvolutionSettings.multiscaleDeconvolutionScaleBias =
+      multiscaleDeconvolutionScaleBias;
+  deconvolutionSettings.multiscaleMaxScales = multiscaleMaxScales;
+  deconvolutionSettings.multiscaleConvolutionPadding =
+      multiscaleConvolutionPadding;
+  deconvolutionSettings.multiscaleScaleList.assign(multiscaleScaleList.begin(),
+                                                   multiscaleScaleList.end());
+  deconvolutionSettings.multiscaleShapeFunction = multiscaleShapeFunction;
+  deconvolutionSettings.deconvolutionBorderRatio = deconvolutionBorderRatio;
+  deconvolutionSettings.fitsDeconvolutionMask = fitsDeconvolutionMask;
+  deconvolutionSettings.casaDeconvolutionMask = casaDeconvolutionMask;
+  deconvolutionSettings.horizonMask = horizonMask;
+  deconvolutionSettings.horizonMaskDistance = horizonMaskDistance;
+  deconvolutionSettings.pythonDeconvolutionFilename =
+      pythonDeconvolutionFilename;
+  deconvolutionSettings.useMoreSaneDeconvolution = useMoreSaneDeconvolution;
+  deconvolutionSettings.useIUWTDeconvolution = useIUWTDeconvolution;
+  deconvolutionSettings.iuwtSNRTest = iuwtSNRTest;
+  deconvolutionSettings.moreSaneLocation = moreSaneLocation;
+  deconvolutionSettings.moreSaneArgs = moreSaneArgs;
+  deconvolutionSettings.moreSaneSigmaLevels.assign(moreSaneSigmaLevels.begin(),
+                                                   moreSaneSigmaLevels.end());
+  deconvolutionSettings.spectralFittingMode = spectralFittingMode;
+  deconvolutionSettings.spectralFittingTerms = spectralFittingTerms;
+  deconvolutionSettings.forcedSpectrumFilename = forcedSpectrumFilename;
+  deconvolutionSettings.deconvolutionChannelCount = deconvolutionChannelCount;
+
+  return deconvolutionSettings;
+}
+
 bool Settings::determineReorder() const {
   return ((channelsOut != 1) || (polarizations.size() >= 4) ||
           (deconvolutionMGain != 1.0) ||
