@@ -3,6 +3,10 @@
 
 #include <map>
 
+#include <aocommon/uvector.h>
+
+#include <schaapcommon/fitters/nlplfitter.h>
+
 #include "measurement.h"
 #include "spectralenergydistribution.h"
 
@@ -374,7 +378,7 @@ class MeasuredSED : public SpectralEnergyDistribution {
       }
     }
     if (requireNonLinear) {
-      NonLinearPowerLawFitter fitter;
+      schaapcommon::fitters::NonLinearPowerLawFitter fitter;
       n = 0;
       for (FluxMap::const_iterator i = _measurements.begin();
            i != _measurements.end(); ++i) {
@@ -407,7 +411,7 @@ class MeasuredSED : public SpectralEnergyDistribution {
 
   void FitPowerlaw2ndOrder(long double& a, long double& b, long double& c,
                            aocommon::PolarizationEnum polarization) const {
-    NonLinearPowerLawFitter fitter;
+    schaapcommon::fitters::NonLinearPowerLawFitter fitter;
     size_t n = 0;
     for (FluxMap::const_iterator i = _measurements.begin();
          i != _measurements.end(); ++i) {
@@ -425,10 +429,10 @@ class MeasuredSED : public SpectralEnergyDistribution {
     c = cTemp;
   }
 
-  void FitLogPolynomial(aocommon::UVector<float>& terms, size_t nTerms,
+  void FitLogPolynomial(std::vector<float>& terms, size_t nTerms,
                         aocommon::PolarizationEnum polarization,
                         double referenceFrequencyHz = 1.0) const {
-    NonLinearPowerLawFitter fitter;
+    schaapcommon::fitters::NonLinearPowerLawFitter fitter;
     size_t n = 0;
     for (FluxMap::const_iterator i = _measurements.begin();
          i != _measurements.end(); ++i) {
@@ -439,7 +443,6 @@ class MeasuredSED : public SpectralEnergyDistribution {
         ++n;
       }
     }
-    terms.assign(nTerms, 0.0);
     fitter.Fit(terms, nTerms);
   }
 
