@@ -11,6 +11,13 @@ class DirectMSRowProvider : public MSRowProvider {
       : MSRowProvider(msPath, selection, selectedDataDescIds, dataColumnName,
                       requireModel) {}
 
+  explicit DirectMSRowProvider(
+      const casacore::MeasurementSet& ms, const MSSelection& selection,
+      const std::map<size_t, size_t>& selected_data_description_ids,
+      const std::string& data_column_name, bool require_model)
+      : MSRowProvider(ms, selection, selected_data_description_ids,
+                      data_column_name, require_model) {}
+
   virtual void ReadData(DataArray& data, FlagArray& flags, WeightArray& weights,
                         double& u, double& v, double& w, uint32_t& dataDescId,
                         uint32_t& antenna1, uint32_t& antenna2,
