@@ -1,14 +1,15 @@
 #include "msprovider.h"
 #include "msreaders/msreader.h"
 
-#include "../io/logger.h"
+#include <aocommon/logger.h>
 
 #include <casacore/ms/MeasurementSets/MeasurementSet.h>
 #include <casacore/tables/Tables/ScalarColumn.h>
 #include <casacore/tables/Tables/ArrColDesc.h>
 
 #include "../structures/msselection.h"
-#include "../structures/multibanddata.h"
+
+using aocommon::Logger;
 
 namespace {
 template <bool add>
@@ -23,9 +24,8 @@ void AddOrAssign<false>(std::complex<float>* dest, std::complex<float> source) {
 }  // namespace
 
 MSProvider::~MSProvider() {}
-MSProvider::MSProvider(const MSProvider&) {}
 
-void MSProvider::copyData(std::complex<float>* dest, size_t startChannel,
+void MSProvider::CopyData(std::complex<float>* dest, size_t startChannel,
                           size_t endChannel,
                           const std::vector<aocommon::PolarizationEnum>& polsIn,
                           const casacore::Array<std::complex<float>>& data,
@@ -35,23 +35,53 @@ void MSProvider::copyData(std::complex<float>* dest, size_t startChannel,
       data.cbegin() + startChannel * polCount;
   const size_t selectedChannelCount = endChannel - startChannel;
 
-  size_t polIndex;
   if (polOut == aocommon::Polarization::Instrumental) {
-    if (polsIn.size() != 4)
+    if (polsIn.size() != 4) {
       throw std::runtime_error(
           "This mode requires the four polarizations to be present in the "
           "measurement set");
+    }
     for (size_t ch = 0; ch != selectedChannelCount * polsIn.size(); ++ch) {
-      if (isCFinite(*inPtr))
+      if (IsCFinite(*inPtr))
         dest[ch] = *inPtr;
       else
         dest[ch] = 0;
-      inPtr++;
+      ++inPtr;
     }
-  } else if (aocommon::Polarization::TypeToIndex(polOut, polsIn, polIndex)) {
+  } else if (polOut == aocommon::Polarization::DiagonalInstrumental) {
+    if (polsIn.size() == 4) {
+      size_t ch = 0;
+      while (ch != selectedChannelCount * 2) {
+        if (IsCFinite(*inPtr))
+          dest[ch] = *inPtr;
+        else
+          dest[ch] = 0;
+        inPtr += 3;  // jump from xx to yy
+        ++ch;
+        if (IsCFinite(*inPtr))
+          dest[ch] = *inPtr;
+        else
+          dest[ch] = 0;
+        ++inPtr;
+        ++ch;
+      }
+    } else if (polsIn.size() == 2) {
+      for (size_t ch = 0; ch != selectedChannelCount * 2; ++ch) {
+        if (IsCFinite(*inPtr))
+          dest[ch] = *inPtr;
+        else
+          dest[ch] = 0;
+        ++inPtr;
+      }
+    } else
+      throw std::runtime_error(
+          "Diagonal instrument visibilities requested, but this requires 2 or "
+          "4 polarizations in the data");
+  } else if (size_t polIndex;
+             aocommon::Polarization::TypeToIndex(polOut, polsIn, polIndex)) {
     inPtr += polIndex;
     for (size_t ch = 0; ch != selectedChannelCount; ++ch) {
-      if (isCFinite(*inPtr))
+      if (IsCFinite(*inPtr))
         dest[ch] = *inPtr;
       else
         dest[ch] = 0;
@@ -85,7 +115,7 @@ void MSProvider::copyData(std::complex<float>* dest, size_t startChannel,
           // I = (XX + YY) / 2
           val = (*inPtr + val) * 0.5f;
 
-          if (isCFinite(val))
+          if (IsCFinite(val))
             dest[ch] = val;
           else
             dest[ch] = 0.0;
@@ -109,7 +139,7 @@ void MSProvider::copyData(std::complex<float>* dest, size_t startChannel,
             // Q = (XX - YY)/2
             val = (val - *inPtr) * 0.5f;
 
-            if (isCFinite(val))
+            if (IsCFinite(val))
               dest[ch] = val;
             else
               dest[ch] = 0.0;
@@ -134,7 +164,7 @@ void MSProvider::copyData(std::complex<float>* dest, size_t startChannel,
             // Q = (RL + LR)/2
             val = (*inPtr + val) * 0.5f;
 
-            if (isCFinite(val))
+            if (IsCFinite(val))
               dest[ch] = val;
             else
               dest[ch] = 0.0;
@@ -159,7 +189,7 @@ void MSProvider::copyData(std::complex<float>* dest, size_t startChannel,
             // U = (XY + YX)/2
             val = (val + *inPtr) * 0.5f;
 
-            if (isCFinite(val))
+            if (IsCFinite(val))
               dest[ch] = val;
             else
               dest[ch] = 0.0;
@@ -185,7 +215,7 @@ void MSProvider::copyData(std::complex<float>* dest, size_t startChannel,
             val = (val - *inPtr) * 0.5f;
             val = casacore::Complex(val.imag(), -val.real());
 
-            if (isCFinite(val))
+            if (IsCFinite(val))
               dest[ch] = val;
             else
               dest[ch] = 0.0;
@@ -211,7 +241,7 @@ void MSProvider::copyData(std::complex<float>* dest, size_t startChannel,
             val = (val - *inPtr) * 0.5f;
             val = casacore::Complex(val.imag(), -val.real());
 
-            if (isCFinite(val))
+            if (IsCFinite(val))
               dest[ch] = val;
             else
               dest[ch] = 0.0;
@@ -236,7 +266,7 @@ void MSProvider::copyData(std::complex<float>* dest, size_t startChannel,
             // V = (RR - LL)/2
             val = (val - *inPtr) * 0.5f;
 
-            if (isCFinite(val))
+            if (IsCFinite(val))
               dest[ch] = val;
             else
               dest[ch] = 0.0;
@@ -253,14 +283,14 @@ void MSProvider::copyData(std::complex<float>* dest, size_t startChannel,
 }
 
 template <typename NumType>
-void MSProvider::copyWeights(
+void MSProvider::CopyWeights(
     NumType* dest, size_t startChannel, size_t endChannel,
     const std::vector<aocommon::PolarizationEnum>& polsIn,
     const casacore::Array<std::complex<float>>& data,
     const casacore::Array<float>& weights, const casacore::Array<bool>& flags,
     aocommon::PolarizationEnum polOut) {
   const size_t polCount = polsIn.size();
-  casacore::Array<std::complex<float>>::const_contiter inPtr =
+  casacore::Array<std::complex<float>>::const_contiter dataPtr =
       data.cbegin() + startChannel * polCount;
   casacore::Array<float>::const_contiter weightPtr =
       weights.cbegin() + startChannel * polCount;
@@ -271,27 +301,60 @@ void MSProvider::copyWeights(
   size_t polIndex;
   if (polOut == aocommon::Polarization::Instrumental) {
     for (size_t ch = 0; ch != selectedChannelCount * polsIn.size(); ++ch) {
-      if (!*flagPtr && isCFinite(*inPtr))
+      if (!*flagPtr && IsCFinite(*dataPtr))
         // The factor of 4 is to be consistent with StokesI
         // It is for having conjugate visibilities and because IDG doesn't
         // separately count XX and YY visibilities
         dest[ch] = *weightPtr * 4.0f;
       else
         dest[ch] = 0.0f;
-      inPtr++;
+      dataPtr++;
       weightPtr++;
       flagPtr++;
     }
+  } else if (polOut == aocommon::Polarization::DiagonalInstrumental) {
+    if (polsIn.size() == 4) {
+      size_t ch = 0;
+      while (ch != selectedChannelCount * 2) {
+        if (!*flagPtr && IsCFinite(*dataPtr))
+          // See explanation above for factor of 4
+          dest[ch] = *weightPtr * 4.0f;
+        else
+          dest[ch] = 0.0f;
+        dataPtr += 3;  // jump from xx to yy
+        weightPtr += 3;
+        flagPtr += 3;
+        ++ch;
+        if (!*flagPtr && IsCFinite(*dataPtr))
+          dest[ch] = *weightPtr * 4.0f;
+        else
+          dest[ch] = 0.0f;
+        ++dataPtr;
+        ++weightPtr;
+        ++flagPtr;
+        ++ch;
+      }
+    } else if (polsIn.size() == 2) {
+      for (size_t ch = 0; ch != selectedChannelCount * 2; ++ch) {
+        if (!*flagPtr && IsCFinite(*dataPtr))
+          dest[ch] = *weightPtr * 4.0f;
+        else
+          dest[ch] = 0.0f;
+        ++dataPtr;
+        ++weightPtr;
+        ++flagPtr;
+      }
+    }
   } else if (aocommon::Polarization::TypeToIndex(polOut, polsIn, polIndex)) {
-    inPtr += polIndex;
+    dataPtr += polIndex;
     weightPtr += polIndex;
     flagPtr += polIndex;
     for (size_t ch = 0; ch != selectedChannelCount; ++ch) {
-      if (!*flagPtr && isCFinite(*inPtr))
+      if (!*flagPtr && IsCFinite(*dataPtr))
         dest[ch] = *weightPtr;
       else
         dest[ch] = 0.0f;
-      inPtr += polCount;
+      dataPtr += polCount;
       weightPtr += polCount;
       flagPtr += polCount;
     }
@@ -353,37 +416,37 @@ void MSProvider::copyWeights(
     }
 
     weightPtr += polIndexA;
-    inPtr += polIndexA;
+    dataPtr += polIndexA;
     flagPtr += polIndexA;
     for (size_t ch = 0; ch != selectedChannelCount; ++ch) {
       NumType w;
-      if (!*flagPtr && isCFinite(*inPtr))
+      if (!*flagPtr && IsCFinite(*dataPtr))
         w = *weightPtr * 4.0f;
       else
         w = 0.0f;
-      inPtr += polIndexB - polIndexA;
+      dataPtr += polIndexB - polIndexA;
       weightPtr += polIndexB - polIndexA;
       flagPtr += polIndexB - polIndexA;
-      if (!*flagPtr && isCFinite(*inPtr))
+      if (!*flagPtr && IsCFinite(*dataPtr))
         w = std::min<NumType>(w, *weightPtr * 4.0f);
       else
         w = 0.0f;
       dest[ch] = w;
       weightPtr += polCount - polIndexB + polIndexA;
-      inPtr += polCount - polIndexB + polIndexA;
+      dataPtr += polCount - polIndexB + polIndexA;
       flagPtr += polCount - polIndexB + polIndexA;
     }
   }
 }
 
-template void MSProvider::copyWeights<float>(
+template void MSProvider::CopyWeights<float>(
     float* dest, size_t startChannel, size_t endChannel,
     const std::vector<aocommon::PolarizationEnum>& polsIn,
     const casacore::Array<std::complex<float>>& data,
     const casacore::Array<float>& weights, const casacore::Array<bool>& flags,
     aocommon::PolarizationEnum polOut);
 
-template void MSProvider::copyWeights<std::complex<float>>(
+template void MSProvider::CopyWeights<std::complex<float>>(
     std::complex<float>* dest, size_t startChannel, size_t endChannel,
     const std::vector<aocommon::PolarizationEnum>& polsIn,
     const casacore::Array<std::complex<float>>& data,
@@ -391,7 +454,7 @@ template void MSProvider::copyWeights<std::complex<float>>(
     aocommon::PolarizationEnum polOut);
 
 template <bool add>
-void MSProvider::reverseCopyData(
+void MSProvider::ReverseCopyData(
     casacore::Array<std::complex<float>>& dest, size_t startChannel,
     size_t endChannel, const std::vector<aocommon::PolarizationEnum>& polsDest,
     const std::complex<float>* source, aocommon::PolarizationEnum polSource) {
@@ -408,6 +471,29 @@ void MSProvider::reverseCopyData(
       }
       dataIter++;
     }
+  } else if (polSource == aocommon::Polarization::DiagonalInstrumental) {
+    if (polsDest.size() == 2) {
+      for (size_t chp = 0; chp != selectedChannelCount * 2; ++chp) {
+        if (std::isfinite(source[chp].real())) {
+          AddOrAssign<add>(dataIter, source[chp]);
+        }
+        dataIter++;
+      }
+    } else {
+      size_t chp = 0;
+      while (chp != selectedChannelCount * 2) {
+        if (std::isfinite(source[chp].real())) {
+          AddOrAssign<add>(dataIter, source[chp]);
+        }
+        dataIter += 3;  // jump from xx to yy
+        ++chp;
+        if (std::isfinite(source[chp].real())) {
+          AddOrAssign<add>(dataIter, source[chp]);
+        }
+        ++dataIter;
+        ++chp;
+      }
+    }
   } else if (aocommon::Polarization::TypeToIndex(polSource, polsDest,
                                                  polIndex)) {
     for (size_t ch = 0; ch != selectedChannelCount; ++ch) {
@@ -570,17 +656,17 @@ void MSProvider::reverseCopyData(
 }
 
 // Explicit instantiation for true/false
-template void MSProvider::reverseCopyData<true>(
+template void MSProvider::ReverseCopyData<true>(
     casacore::Array<std::complex<float>>& dest, size_t startChannel,
     size_t endChannel, const std::vector<aocommon::PolarizationEnum>& polsDest,
     const std::complex<float>* source, aocommon::PolarizationEnum polSource);
 
-template void MSProvider::reverseCopyData<false>(
+template void MSProvider::ReverseCopyData<false>(
     casacore::Array<std::complex<float>>& dest, size_t startChannel,
     size_t endChannel, const std::vector<aocommon::PolarizationEnum>& polsDest,
     const std::complex<float>* source, aocommon::PolarizationEnum polSource);
 
-void MSProvider::reverseCopyWeights(
+void MSProvider::ReverseCopyWeights(
     casacore::Array<float>& dest, size_t startChannel, size_t endChannel,
     const std::vector<aocommon::PolarizationEnum>& polsDest,
     const float* source, aocommon::PolarizationEnum polSource) {
@@ -630,7 +716,7 @@ void MSProvider::reverseCopyWeights(
   }
 }
 
-void MSProvider::getRowRange(casacore::MeasurementSet& ms,
+void MSProvider::GetRowRange(casacore::MeasurementSet& ms,
                              const MSSelection& selection, size_t& startRow,
                              size_t& endRow) {
   startRow = 0;
@@ -657,7 +743,7 @@ void MSProvider::getRowRange(casacore::MeasurementSet& ms,
   }
 }
 
-void MSProvider::getRowRangeAndIDMap(casacore::MeasurementSet& ms,
+void MSProvider::GetRowRangeAndIDMap(casacore::MeasurementSet& ms,
                                      const MSSelection& selection,
                                      size_t& startRow, size_t& endRow,
                                      const std::set<size_t>& dataDescIds,
@@ -713,7 +799,7 @@ void MSProvider::getRowRangeAndIDMap(casacore::MeasurementSet& ms,
                << idToMSRow.size() << " rows)\n";
 }
 
-void MSProvider::initializeModelColumn(casacore::MeasurementSet& ms) {
+void MSProvider::InitializeModelColumn(casacore::MeasurementSet& ms) {
   casacore::ArrayColumn<casacore::Complex> dataColumn(
       ms, casacore::MS::columnName(casacore::MSMainEnums::DATA));
   if (ms.isColumn(casacore::MSMainEnums::MODEL_DATA)) {
@@ -759,7 +845,7 @@ void MSProvider::initializeModelColumn(casacore::MeasurementSet& ms) {
   }
 }
 
-casacore::ArrayColumn<float> MSProvider::initializeImagingWeightColumn(
+casacore::ArrayColumn<float> MSProvider::InitializeImagingWeightColumn(
     casacore::MeasurementSet& ms) {
   ms.reopenRW();
   casacore::ArrayColumn<casacore::Complex> dataColumn(
@@ -822,23 +908,17 @@ void MSProvider::ResetModelColumn() {
   }
 }
 
-bool MSProvider::openWeightSpectrumColumn(
-    casacore::MeasurementSet& ms,
-    std::unique_ptr<casacore::ROArrayColumn<float>>& weightColumn,
-    const casacore::IPosition& dataColumnShape) {
+bool MSProvider::OpenWeightSpectrumColumn(
+    const casacore::MeasurementSet& ms,
+    std::unique_ptr<casacore::ArrayColumn<float>>& weightColumn) {
   bool isWeightDefined;
   if (ms.isColumn(casacore::MSMainEnums::WEIGHT_SPECTRUM)) {
-    weightColumn.reset(new casacore::ROArrayColumn<float>(
+    weightColumn.reset(new casacore::ArrayColumn<float>(
         ms, casacore::MS::columnName(casacore::MSMainEnums::WEIGHT_SPECTRUM)));
     isWeightDefined = weightColumn->isDefined(0);
   } else {
     isWeightDefined = false;
   }
-  casacore::Array<float> weightArray(dataColumnShape);
-  if (isWeightDefined) {
-    casacore::IPosition weightShape = weightColumn->shape(0);
-    isWeightDefined = (weightShape == dataColumnShape);
-  }
   if (!isWeightDefined) {
     Logger::Warn
         << "WARNING: This measurement set has no or an invalid WEIGHT_SPECTRUM "
