@@ -3,12 +3,12 @@
 
 #include "synchronizedms.h"
 
-#include <casacore/casa/Arrays/Array.h>
-
-#include <casacore/tables/Tables/ArrayColumn.h>
-
+#include <aocommon/multibanddata.h>
 #include <aocommon/polarization.h>
 
+#include <casacore/casa/Arrays/Array.h>
+#include <casacore/tables/Tables/ArrayColumn.h>
+
 #include <complex>
 #include <set>
 #include <type_traits>
@@ -23,7 +23,7 @@ class MSReader;
 /**
  * The abstract MSProvider class is the base class for classes that read and
  * write the visibilities. Write functionality is directly provided by classes
- * that derive from MSProvider, whereas reading functionality is provided a
+ * that derive from MSProvider, whereas reading functionality is provided in a
  * separate class (MSReader), which can be instantiated with @ref
  * MSProvider::MakeReader. An MSProvider knows which rows are selected and
  * doesn't write to unselected rows. This information on selected rows is also
@@ -35,21 +35,27 @@ class MSReader;
  * The class maintains an index for the write position. The index for the
  * reading position is maintained by the closely connected @ref MSReader class.
  * Writing (and reading) goes sequentially through the data.
+ *
+ * An MS provider provides data for a single dataDescID, and therefore for
+ * a single spectral window. Measurement sets with multiple data desc IDs
+ * require creating multiple MS providers. Because of this, all rows of a
+ * single MS provider have the same number of channels.
  */
 class MSProvider {
  public:
-  friend class MSRowProvider;
-  friend class DirectMSRowProvider;
-  friend class MSReader;
-
   struct MetaData {
     double uInM, vInM, wInM;
-    size_t dataDescId, fieldId, antenna1, antenna2;
+    size_t fieldId, antenna1, antenna2;
     double time;
   };
 
+  MSProvider() = default;
+
   virtual ~MSProvider();
 
+  MSProvider(const MSProvider&) = delete;
+  MSProvider& operator=(const MSProvider&) = delete;
+
   virtual SynchronizedMS MS() = 0;
 
   /**
@@ -94,12 +100,19 @@ class MSProvider {
 
   /**
    * Polarization that this msprovider provides.
-   * Be aware that it may return 'Instrumental', which means it provides
-   * all polarizations that are provided by the underlying measurement set /
-   * data.
+   * Be aware that it may return 'DiagonalInstrumental' or 'Instrumental',
+   * which means it provides 2 or 4 polarizations that are provided by the
+   * underlying measurement set / data.
    */
   virtual aocommon::PolarizationEnum Polarization() = 0;
 
+  /**
+   * The dataDescID to which this MS provider is associated. An MSProvider
+   * is associated with only one dataDescId. When an MS has multiple
+   * dataDescIds, it will have multiple MSProviders.
+   */
+  virtual size_t DataDescId() = 0;
+
   /**
    * Number of channels provided by this provider. May be different from the
    * underlying measurement set if not all channels are selected.
@@ -120,6 +133,13 @@ class MSProvider {
    */
   virtual size_t NPolarizations() = 0;
 
+  /**
+   * Get the band information that this MSProvider covers. The BandData
+   * includes all channels in the original data, even when not all
+   * channels are selected (@sa NChannels()).
+   */
+  virtual const aocommon::BandData& Band() = 0;
+
   /**
    * Get a list of polarizations in the measurement set.
    * This will always list the individual polarizations, and not return
@@ -135,15 +155,14 @@ class MSProvider {
    */
   void ResetModelColumn();
 
- protected:
-  static void copyData(std::complex<float>* dest, size_t startChannel,
+  static void CopyData(std::complex<float>* dest, size_t startChannel,
                        size_t endChannel,
                        const std::vector<aocommon::PolarizationEnum>& polsIn,
                        const casacore::Array<std::complex<float>>& data,
                        aocommon::PolarizationEnum polOut);
 
   template <typename NumType>
-  static void copyWeights(NumType* dest, size_t startChannel, size_t endChannel,
+  static void CopyWeights(NumType* dest, size_t startChannel, size_t endChannel,
                           const std::vector<aocommon::PolarizationEnum>& polsIn,
                           const casacore::Array<std::complex<float>>& data,
                           const casacore::Array<float>& weights,
@@ -151,33 +170,33 @@ class MSProvider {
                           aocommon::PolarizationEnum polOut);
 
   template <typename NumType>
-  static bool isCFinite(const std::complex<NumType>& c) {
+  static bool IsCFinite(const std::complex<NumType>& c) {
     return std::isfinite(c.real()) && std::isfinite(c.imag());
   }
 
   template <bool add>
-  static void reverseCopyData(
+  static void ReverseCopyData(
       casacore::Array<std::complex<float>>& dest, size_t startChannel,
       size_t endChannel,
       const std::vector<aocommon::PolarizationEnum>& polsDest,
       const std::complex<float>* source, aocommon::PolarizationEnum polSource);
 
-  static void reverseCopyWeights(
+  static void ReverseCopyWeights(
       casacore::Array<float>& dest, size_t startChannel, size_t endChannel,
       const std::vector<aocommon::PolarizationEnum>& polsDest,
       const float* source, aocommon::PolarizationEnum polSource);
 
-  static void getRowRange(casacore::MeasurementSet& ms,
+  static void GetRowRange(casacore::MeasurementSet& ms,
                           const MSSelection& selection, size_t& startRow,
                           size_t& endRow);
 
-  static void getRowRangeAndIDMap(casacore::MeasurementSet& ms,
+  static void GetRowRangeAndIDMap(casacore::MeasurementSet& ms,
                                   const MSSelection& selection,
                                   size_t& startRow, size_t& endRow,
                                   const std::set<size_t>& dataDescIdMap,
                                   std::vector<size_t>& idToMSRow);
 
-  static void copyRealToComplex(std::complex<float>* dest, const float* source,
+  static void CopyRealToComplex(std::complex<float>* dest, const float* source,
                                 size_t n) {
     const float* end = source + n;
     while (source != end) {
@@ -187,24 +206,23 @@ class MSProvider {
     }
   }
 
-  static void initializeModelColumn(casacore::MeasurementSet& ms);
+  static void InitializeModelColumn(casacore::MeasurementSet& ms);
 
-  static casacore::ArrayColumn<float> initializeImagingWeightColumn(
+  static casacore::ArrayColumn<float> InitializeImagingWeightColumn(
       casacore::MeasurementSet& ms);
 
   /**
    * Make an arraycolumn object for the weight spectrum column if it exists and
    * is valid. The weight spectrum column is an optional column, the weight
    * column should be used if it doesn't exist. Moreover, some measurement sets
-   * have an empty or invalid sized weight spectrum column; this method only
+   * have an empty weight spectrum column; this method only
    * returns true if the column can be used.
    */
-  static bool openWeightSpectrumColumn(
-      casacore::MeasurementSet& ms,
-      std::unique_ptr<casacore::ROArrayColumn<float>>& weightColumn,
-      const casacore::IPosition& dataColumnShape);
+  static bool OpenWeightSpectrumColumn(
+      const casacore::MeasurementSet& ms,
+      std::unique_ptr<casacore::ArrayColumn<float>>& weightColumn);
 
-  static void expandScalarWeights(
+  static void ExpandScalarWeights(
       const casacore::Array<float>& weightScalarArray,
       casacore::Array<float>& weightSpectrumArray) {
     casacore::Array<float>::const_contiter src = weightScalarArray.cbegin();
@@ -215,12 +233,6 @@ class MSProvider {
       if (src == weightScalarArray.cend()) src = weightScalarArray.cbegin();
     }
   }
-
-  MSProvider() {}
-
- private:
-  MSProvider(const MSProvider&);
-  void operator=(const MSProvider&) {}
 };
 
 #endif
