@@ -14,26 +14,23 @@ class PartitionedMSReader final : public MSReader {
   PartitionedMSReader(PartitionedMS* partitionedMS);
   virtual ~PartitionedMSReader(){};
 
-  size_t RowId() const final override { return _currentInputRow; }
+  size_t RowId() const override { return _currentInputRow; }
 
-  bool CurrentRowAvailable() final override;
+  bool CurrentRowAvailable() override;
 
-  void NextInputRow() final override;
+  void NextInputRow() override;
 
-  void ReadMeta(double& u, double& v, double& w,
-                size_t& dataDescId) final override;
+  void ReadMeta(double& u, double& v, double& w) override;
 
-  void ReadMeta(MSProvider::MetaData& metaData) final override;
+  void ReadMeta(MSProvider::MetaData& metaData) override;
 
-  void ReadData(std::complex<float>* buffer) final override;
+  void ReadData(std::complex<float>* buffer) override;
 
-  void ReadModel(std::complex<float>* buffer) final override;
+  void ReadModel(std::complex<float>* buffer) override;
 
-  void ReadWeights(std::complex<float>* buffer) final override;
+  void ReadWeights(float* buffer) override;
 
-  void ReadWeights(float* buffer) final override;
-
-  void WriteImagingWeights(const float* buffer) final override;
+  void WriteImagingWeights(const float* buffer) override;
 
  private:
   size_t _currentInputRow;
@@ -45,4 +42,4 @@ class PartitionedMSReader final : public MSReader {
   std::unique_ptr<std::fstream> _imagingWeightsFile;
 };
 
-#endif
\ No newline at end of file
+#endif
