@@ -6,46 +6,55 @@
 #include "msrowprovider.h"
 #include "noisemsrowprovider.h"
 
-#include "../system/system.h"
-
-#include "../io/logger.h"
-
 #include "../main/progressbar.h"
 #include "../main/settings.h"
 
-#include <sys/mman.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-
-#include <errno.h>
-#include <fcntl.h>
-
 #include <cstdio>
 #include <fstream>
 #include <sstream>
 #include <memory>
 #include <vector>
 
+#include <aocommon/logger.h>
+
 #include <boost/filesystem/path.hpp>
 
 #include <casacore/measures/Measures/MEpoch.h>
 
 #include <casacore/measures/TableMeasures/ScalarMeasColumn.h>
 
+using aocommon::Logger;
+
+/**
+ * MAP_NORESERVE is unsuported AND not defined on hurd-i386, so
+ * assign it to zero in this case.
+ */
+#ifndef MAP_NORESERVE
+#define MAP_NORESERVE 0
+#endif
+
+namespace {
+struct PartitionFiles {
+  std::unique_ptr<std::ofstream> data;
+  std::unique_ptr<std::ofstream> weight;
+  std::unique_ptr<std::ofstream> model;
+};
+}  // namespace
+
 PartitionedMS::PartitionedMS(const Handle& handle, size_t partIndex,
                              aocommon::PolarizationEnum polarization,
                              size_t dataDescId)
     : _handle(handle),
       _partIndex(partIndex),
-      _modelFileMap(0),
+      _dataDescId(dataDescId),
       _currentOutputRow(0),
       _polarization(polarization),
       _polarizationCountInFile(
-          _polarization == aocommon::Polarization::Instrumental ? 4 : 1) {
+          aocommon::Polarization::GetVisibilityCount(_polarization)) {
   std::ifstream metaFile(getMetaFilename(
       handle._data->_msPath, handle._data->_temporaryDirectory, dataDescId));
 
-  metaFile.read(reinterpret_cast<char*>(&_metaHeader), sizeof(MetaHeader));
+  _metaHeader.Read(metaFile);
   std::vector<char> msPath(_metaHeader.filenameLength + 1, char(0));
   metaFile.read(msPath.data(), _metaHeader.filenameLength);
   Logger::Info << "Opening reordered part " << partIndex << " spw "
@@ -58,46 +67,22 @@ PartitionedMS::PartitionedMS(const Handle& handle, size_t partIndex,
   if (!dataFile.good())
     throw std::runtime_error("Error opening temporary data file '" +
                              partPrefix + ".tmp'");
-  dataFile.read(reinterpret_cast<char*>(&_partHeader), sizeof(PartHeader));
+  _partHeader.Read(dataFile);
   if (!dataFile.good())
     throw std::runtime_error("Error reading header from file '" + partPrefix +
                              ".tmp'");
 
   if (_partHeader.hasModel) {
-    _fd = open((partPrefix + "-m.tmp").c_str(), O_RDWR);
-    if (_fd == -1)
-      throw std::runtime_error("Error opening temporary model data file '" +
-                               partPrefix + "-m.tmp'");
-    size_t length = _partHeader.channelCount * _metaHeader.selectedRowCount *
-                    _polarizationCountInFile * sizeof(std::complex<float>);
-    if (length == 0)
-      _modelFileMap = nullptr;
-    else {
-      _modelFileMap =
-          reinterpret_cast<char*>(mmap(NULL, length, PROT_READ | PROT_WRITE,
-                                       MAP_SHARED | MAP_NORESERVE, _fd, 0));
-      if (_modelFileMap == MAP_FAILED) {
-        std::string msg = System::StrError(errno);
-        _modelFileMap = 0;
-        throw std::runtime_error(
-            std::string("Error creating memory map to temporary model file: "
-                        "mmap() returned MAP_FAILED with error message: ") +
-            msg);
-      }
-    }
+    const size_t length =
+        _partHeader.channelCount * _metaHeader.selectedRowCount *
+        _polarizationCountInFile * sizeof(std::complex<float>);
+    _modelFile = MappedFile(partPrefix + "-m.tmp", length);
   }
   metaFile.close();
   dataFile.close();
 }
 
-PartitionedMS::~PartitionedMS() {
-  if (_modelFileMap != 0) {
-    size_t length = _partHeader.channelCount * _metaHeader.selectedRowCount *
-                    _polarizationCountInFile * sizeof(std::complex<float>);
-    if (length != 0) munmap(_modelFileMap, length);
-  }
-  if (_partHeader.hasModel) close(_fd);
-}
+PartitionedMS::~PartitionedMS() {}
 
 std::unique_ptr<MSReader> PartitionedMS::MakeReader() {
   std::unique_ptr<MSReader> reader(new PartitionedMSReader(this));
@@ -115,7 +100,7 @@ void PartitionedMS::WriteModel(const std::complex<float>* buffer,
   size_t rowLength = _partHeader.channelCount * _polarizationCountInFile *
                      sizeof(std::complex<float>);
   std::complex<float>* modelWritePtr = reinterpret_cast<std::complex<float>*>(
-      _modelFileMap + rowLength * _currentOutputRow);
+      _modelFile.Data() + rowLength * _currentOutputRow);
 
   // In case the value was not sampled in this pass, it has been set to infinite
   // and should not overwrite the current value in the set.
@@ -179,11 +164,6 @@ string PartitionedMS::getMetaFilename(const string& msPathStr,
   return s.str();
 }
 
-// should be private but is not allowed on older compilers
-struct PartitionFiles {
-  std::unique_ptr<std::ofstream> data, weight, model;
-};
-
 /*
  * When partitioned:
  * One global file stores:
@@ -205,13 +185,20 @@ PartitionedMS::Handle PartitionedMS::Partition(
     bool initialModelRequired, const Settings& settings) {
   const bool modelUpdateRequired = settings.modelUpdateRequired;
   std::set<aocommon::PolarizationEnum> polsOut;
-  if (settings.useIDG)
-    polsOut.insert(aocommon::Polarization::Instrumental);
-  else
+  if (settings.useIDG) {
+    if (settings.polarizations.size() == 1) {
+      polsOut.insert(aocommon::Polarization::DiagonalInstrumental);
+    } else {
+      polsOut.insert(aocommon::Polarization::Instrumental);
+    }
+  } else {
     polsOut = settings.polarizations;
+  }
+  const size_t polarizationsPerFile =
+      aocommon::Polarization::GetVisibilityCount(*polsOut.begin());
   const std::string& temporaryDirectory = settings.temporaryDirectory;
 
-  size_t channelParts = channels.size();
+  const size_t channelParts = channels.size();
 
   if (channelParts != 1) {
     Logger::Debug << "Partitioning in " << channels.size() << " channels:";
@@ -241,13 +228,13 @@ PartitionedMS::Handle PartitionedMS::Partition(
       f.weight.reset(new std::ofstream(partPrefix + "-w.tmp"));
       if (initialModelRequired)
         f.model.reset(new std::ofstream(partPrefix + "-m.tmp"));
-      f.data->seekp(sizeof(PartHeader), std::ios::beg);
+      f.data->seekp(PartHeader::BINARY_SIZE, std::ios::beg);
 
       ++fileIndex;
     }
   }
 
-  std::unique_ptr<MSRowProvider> rowProvider;
+  std::unique_ptr<MsRowProviderBase> rowProvider;
   if (settings.baselineDependentAveragingInWavelengths == 0.0) {
     if (settings.simulateNoise) {
       std::unique_ptr<NoiseMSRowProvider> noiseRowProvider(
@@ -260,13 +247,12 @@ PartitionedMS::Handle PartitionedMS::Partition(
             settings.simulatedBaselineNoiseFilename);
       rowProvider = std::move(noiseRowProvider);
     } else
-      rowProvider.reset(
-          new DirectMSRowProvider(msPath, selection, selectedDataDescIds,
-                                  dataColumnName, initialModelRequired));
+      rowProvider = MakeMsRowProvider(msPath, selection, selectedDataDescIds,
+                                      dataColumnName, initialModelRequired);
   } else {
     if (initialModelRequired)
       throw std::runtime_error(
-          "Baseline-dependent averaging is enabled together with a model that "
+          "Baseline-dependent averaging is enabled together with a mode that "
           "requires the model data (e.g. -continue or -subtract-model). This "
           "is not possible.");
     rowProvider.reset(new AveragingMSRowProvider(
@@ -275,11 +261,10 @@ PartitionedMS::Handle PartitionedMS::Partition(
         initialModelRequired));
   }
 
-  std::vector<aocommon::PolarizationEnum> msPolarizations =
-      GetMSPolarizations(rowProvider->MS().polarization());
-  size_t nAntennas = rowProvider->MS().antenna().nrow();
-
-  const casacore::IPosition shape(rowProvider->DataShape());
+  const std::vector<aocommon::PolarizationEnum> msPolarizations =
+      GetMSPolarizations(rowProvider->Ms().polarization());
+  const size_t nAntennas = rowProvider->Ms().antenna().nrow();
+  const aocommon::MultiBandData bands(rowProvider->Ms());
 
   if (settings.parallelReordering == 1)
     Logger::Info << "Reordering " << msPath << " into " << channelParts << " x "
@@ -289,20 +274,17 @@ PartitionedMS::Handle PartitionedMS::Partition(
   // TODO rather than writing we can just skip and write later
   std::vector<std::unique_ptr<std::ofstream>> metaFiles(
       selectedDataDescIds.size());
-  for (std::map<size_t, size_t>::const_iterator i = selectedDataDescIds.begin();
-       i != selectedDataDescIds.end(); ++i) {
-    size_t dataDescId = i->first;
-    size_t spwIndex = i->second;
+  for (const std::pair<const size_t, size_t>& p : selectedDataDescIds) {
+    const size_t dataDescId = p.first;
+    const size_t spwIndex = p.second;
     std::string metaFilename =
         getMetaFilename(msPath, temporaryDirectory, dataDescId);
     metaFiles[spwIndex].reset(new std::ofstream(metaFilename));
     MetaHeader metaHeader;
-    memset(&metaHeader, 0, sizeof(MetaHeader));
     metaHeader.selectedRowCount = 0;  // not yet known
     metaHeader.filenameLength = msPath.size();
     metaHeader.startTime = rowProvider->StartTime();
-    metaFiles[spwIndex]->write(reinterpret_cast<char*>(&metaHeader),
-                               sizeof(metaHeader));
+    metaHeader.Write(*metaFiles[spwIndex]);
     metaFiles[spwIndex]->write(msPath.c_str(), msPath.size());
     if (!metaFiles[spwIndex]->good())
       throw std::runtime_error("Error writing to temporary file " +
@@ -310,14 +292,14 @@ PartitionedMS::Handle PartitionedMS::Partition(
   }
 
   // Write actual data
-  size_t polarizationsPerFile = settings.useIDG ? 4 : 1;
   std::vector<std::complex<float>> dataBuffer(polarizationsPerFile *
                                               maxChannels);
   std::vector<float> weightBuffer(polarizationsPerFile * maxChannels);
 
-  casacore::Array<std::complex<float>> dataArray(shape), modelArray(shape);
-  casacore::Array<float> weightSpectrumArray(shape);
-  casacore::Array<bool> flagArray(shape);
+  casacore::Array<std::complex<float>> dataArray;
+  casacore::Array<std::complex<float>> modelArray;
+  casacore::Array<float> weightSpectrumArray;
+  casacore::Array<bool> flagArray;
 
   std::unique_ptr<ProgressBar> progress1;
   if (settings.parallelReordering == 1)
@@ -332,23 +314,21 @@ PartitionedMS::Handle PartitionedMS::Partition(
                              rowProvider->TotalProgress());
 
     MetaRecord meta;
-    memset(&meta, 0, sizeof(MetaRecord));
 
     double time;
     uint32_t dataDescId, antenna1, antenna2, fieldId;
     rowProvider->ReadData(dataArray, flagArray, weightSpectrumArray, meta.u,
                           meta.v, meta.w, dataDescId, antenna1, antenna2,
                           fieldId, time);
-    meta.dataDescId = dataDescId;
     meta.antenna1 = antenna1;
     meta.antenna2 = antenna2;
     meta.fieldId = fieldId;
     meta.time = time;
-    size_t spwIndex = selectedDataDescIds[meta.dataDescId];
+    const size_t spwIndex = selectedDataDescIds[dataDescId];
     ++selectedRowCountPerSpwIndex[spwIndex];
     ++selectedRowsTotal;
     std::ofstream& metaFile = *metaFiles[spwIndex];
-    meta.write(metaFile);
+    meta.Write(metaFile);
     if (!metaFile.good())
       throw std::runtime_error("Error writing to temporary file");
 
@@ -356,13 +336,13 @@ PartitionedMS::Handle PartitionedMS::Partition(
 
     fileIndex = 0;
     for (size_t part = 0; part != channelParts; ++part) {
-      if (channels[part].dataDescId == int(meta.dataDescId)) {
-        size_t partStartCh = channels[part].start,
-               partEndCh = channels[part].end;
+      if (channels[part].dataDescId == int(dataDescId)) {
+        const size_t partStartCh = channels[part].start;
+        const size_t partEndCh = channels[part].end;
 
         for (aocommon::PolarizationEnum p : polsOut) {
           PartitionFiles& f = files[fileIndex];
-          copyData(dataBuffer.data(), partStartCh, partEndCh, msPolarizations,
+          CopyData(dataBuffer.data(), partStartCh, partEndCh, msPolarizations,
                    dataArray, p);
           f.data->write(reinterpret_cast<char*>(dataBuffer.data()),
                         (partEndCh - partStartCh) *
@@ -371,7 +351,7 @@ PartitionedMS::Handle PartitionedMS::Partition(
             throw std::runtime_error("Error writing to temporary data file");
 
           if (initialModelRequired) {
-            copyData(dataBuffer.data(), partStartCh, partEndCh, msPolarizations,
+            CopyData(dataBuffer.data(), partStartCh, partEndCh, msPolarizations,
                      modelArray, p);
             f.model->write(reinterpret_cast<char*>(dataBuffer.data()),
                            (partEndCh - partStartCh) *
@@ -382,7 +362,7 @@ PartitionedMS::Handle PartitionedMS::Partition(
                   "Error writing to temporary model data file");
           }
 
-          copyWeights(weightBuffer.data(), partStartCh, partEndCh,
+          CopyWeights(weightBuffer.data(), partStartCh, partEndCh,
                       msPolarizations, dataArray, weightSpectrumArray,
                       flagArray, p);
           f.weight->write(
@@ -404,23 +384,19 @@ PartitionedMS::Handle PartitionedMS::Partition(
   rowProvider->OutputStatistics();
 
   // Rewrite meta headers to include selected row count
-  for (std::map<size_t, size_t>::const_iterator i = selectedDataDescIds.begin();
-       i != selectedDataDescIds.end(); ++i) {
-    size_t spwIndex = i->second;
+  for (std::pair<const size_t, size_t>& p : selectedDataDescIds) {
+    size_t spwIndex = p.second;
     MetaHeader metaHeader;
-    memset(&metaHeader, 0, sizeof(MetaHeader));
     metaHeader.selectedRowCount = selectedRowCountPerSpwIndex[spwIndex];
     metaHeader.filenameLength = msPath.size();
     metaHeader.startTime = rowProvider->StartTime();
     metaFiles[spwIndex]->seekp(0);
-    metaFiles[spwIndex]->write(reinterpret_cast<char*>(&metaHeader),
-                               sizeof(metaHeader));
+    metaHeader.Write(*metaFiles[spwIndex]);
     metaFiles[spwIndex]->write(msPath.c_str(), msPath.size());
   }
 
   // Write header to parts and write empty model files (if requested)
   PartHeader header;
-  memset(&header, 0, sizeof(PartHeader));
   header.hasModel = includeModel;
   fileIndex = 0;
   dataBuffer.assign(maxChannels * polarizationsPerFile, 0.0);
@@ -436,7 +412,7 @@ PartitionedMS::Handle PartitionedMS::Partition(
          p != polsOut.end(); ++p) {
       PartitionFiles& f = files[fileIndex];
       f.data->seekp(0, std::ios::beg);
-      f.data->write(reinterpret_cast<char*>(&header), sizeof(PartHeader));
+      header.Write(*f.data);
       if (!f.data->good())
         throw std::runtime_error("Error writing to temporary data file");
 
@@ -467,7 +443,7 @@ PartitionedMS::Handle PartitionedMS::Partition(
 
   return Handle(msPath, dataColumnName, temporaryDirectory, channels,
                 initialModelRequired, modelUpdateRequired, polsOut, selection,
-                nAntennas);
+                bands, nAntennas);
 }
 
 void PartitionedMS::unpartition(
@@ -482,7 +458,7 @@ void PartitionedMS::unpartition(
     std::ifstream metaFile(getMetaFilename(
         handle._msPath, handle._temporaryDirectory, dataDescId.first));
     MetaHeader& metaHeader = metaHeaders[dataDescId.second];
-    metaFile.read(reinterpret_cast<char*>(&metaHeader), sizeof(MetaHeader));
+    metaHeader.Read(metaFile);
     std::vector<char> msPath(metaHeader.filenameLength + 1, char(0));
     metaFile.read(msPath.data(), metaHeader.filenameLength);
   }
@@ -496,8 +472,7 @@ void PartitionedMS::unpartition(
   if (!firstDataFile.good())
     throw std::runtime_error("Error opening temporary data file");
   PartHeader firstPartHeader;
-  firstDataFile.read(reinterpret_cast<char*>(&firstPartHeader),
-                     sizeof(PartHeader));
+  firstPartHeader.Read(firstDataFile);
   if (!firstDataFile.good())
     throw std::runtime_error("Error reading from temporary data file");
 
@@ -523,7 +498,7 @@ void PartitionedMS::unpartition(
     casacore::MeasurementSet ms(handle._msPath, casacore::Table::Update);
     const std::vector<aocommon::PolarizationEnum> msPolarizations =
         GetMSPolarizations(ms.polarization());
-    initializeModelColumn(ms);
+    InitializeModelColumn(ms);
     casacore::ScalarColumn<int> antenna1Column(
         ms, ms.columnName(casacore::MSMainEnums::ANTENNA1));
     casacore::ScalarColumn<int> antenna2Column(
@@ -544,8 +519,8 @@ void PartitionedMS::unpartition(
     const casacore::IPosition shape(dataColumn.shape(0));
     size_t channelCount = shape[1];
 
-    size_t polarizationsPerFile =
-        (*pols.begin()) == aocommon::Polarization::Instrumental ? 4 : 1;
+    const size_t polarizationsPerFile =
+        aocommon::Polarization::GetVisibilityCount(*pols.begin());
     std::vector<std::complex<float>> modelDataBuffer(channelCount *
                                                      polarizationsPerFile);
     std::vector<float> weightBuffer(channelCount * polarizationsPerFile);
@@ -554,7 +529,7 @@ void PartitionedMS::unpartition(
     ProgressBar progress(std::string("Writing changed model back to ") +
                          handle._msPath);
     size_t startRow, endRow;
-    getRowRange(ms, handle._selection, startRow, endRow);
+    GetRowRange(ms, handle._selection, startRow, endRow);
     size_t timestep =
         handle._selection.HasInterval() ? handle._selection.IntervalStart() : 0;
     double time = timeColumn(startRow);
@@ -591,7 +566,7 @@ void PartitionedMS::unpartition(
                 if (!modelFiles[fileIndex]->good())
                   throw std::runtime_error(
                       "Error reading from temporary model data file");
-                reverseCopyData<false>(modelDataArray, partStartCh, partEndCh,
+                ReverseCopyData<false>(modelDataArray, partStartCh, partEndCh,
                                        msPolarizations, modelDataBuffer.data(),
                                        *p);
 
@@ -649,7 +624,7 @@ void PartitionedMS::MakeIdToMSRowMapping(std::vector<size_t>& idToMSRow) {
     dataDescIdSet.insert(i->first);
   size_t startRow, endRow;
   SynchronizedMS ms = MS();
-  getRowRangeAndIDMap(*ms, selection, startRow, endRow, dataDescIdSet,
+  GetRowRangeAndIDMap(*ms, selection, startRow, endRow, dataDescIdSet,
                       idToMSRow);
 }
 
