@@ -1,6 +1,8 @@
 #include "multiscaletransforms.h"
 
-#include "../math/fftconvolver.h"
+#include <schaapcommon/fft/convolution.h>
+
+using aocommon::Image;
 
 void MultiScaleTransforms::Transform(std::vector<Image>& images, Image& scratch,
                                      float scale) {
@@ -9,11 +11,11 @@ void MultiScaleTransforms::Transform(std::vector<Image>& images, Image& scratch,
 
   scratch = 0.0;
 
-  FFTConvolver::PrepareSmallKernel(scratch.data(), _width, _height,
-                                   shape.data(), kernelSize, _threadCount);
+  schaapcommon::fft::PrepareSmallConvolutionKernel(
+      scratch.Data(), _width, _height, shape.Data(), kernelSize, _threadCount);
   for (Image& image : images)
-    FFTConvolver::ConvolveSameSize(_fftwManager, image.data(), scratch.data(),
-                                   _width, _height, _threadCount);
+    schaapcommon::fft::Convolve(image.Data(), scratch.Data(), _width, _height,
+                                _threadCount);
 }
 
 void MultiScaleTransforms::PrepareTransform(float* kernel, float scale) {
@@ -22,11 +24,10 @@ void MultiScaleTransforms::PrepareTransform(float* kernel, float scale) {
 
   std::fill_n(kernel, _width * _height, 0.0);
 
-  FFTConvolver::PrepareSmallKernel(kernel, _width, _height, shape.data(),
-                                   kernelSize, _threadCount);
+  schaapcommon::fft::PrepareSmallConvolutionKernel(
+      kernel, _width, _height, shape.Data(), kernelSize, _threadCount);
 }
 
 void MultiScaleTransforms::FinishTransform(float* image, const float* kernel) {
-  FFTConvolver::ConvolveSameSize(_fftwManager, image, kernel, _width, _height,
-                                 _threadCount);
+  schaapcommon::fft::Convolve(image, kernel, _width, _height, _threadCount);
 }
