@@ -1,11 +1,10 @@
 #ifndef MULTI_SCALE_TRANSFORMS_H
 #define MULTI_SCALE_TRANSFORMS_H
 
-#include "../structures/image.h"
-
 #include <cmath>
 #include <initializer_list>
 
+#include <aocommon/image.h>
 #include <aocommon/uvector.h>
 
 #include <vector>
@@ -14,24 +13,21 @@ class MultiScaleTransforms {
  public:
   enum Shape { TaperedQuadraticShape, GaussianShape };
 
-  MultiScaleTransforms(class FFTWManager& fftwManager, size_t width,
-                       size_t height, Shape shape)
-      : _fftwManager(fftwManager),
-        _width(width),
-        _height(height),
-        _shape(shape),
-        _threadCount(1) {}
+  MultiScaleTransforms(size_t width, size_t height, Shape shape)
+      : _width(width), _height(height), _shape(shape), _threadCount(1) {}
 
   void PrepareTransform(float* kernel, float scale);
   void FinishTransform(float* image, const float* kernel);
 
-  void Transform(Image& image, Image& scratch, float scale) {
-    std::vector<Image> images(1, std::move(image));
+  void Transform(aocommon::Image& image, aocommon::Image& scratch,
+                 float scale) {
+    std::vector<aocommon::Image> images(1, std::move(image));
     Transform(images, scratch, scale);
     image = std::move(images[0]);
   }
 
-  void Transform(std::vector<Image>& images, Image& scratch, float scale);
+  void Transform(std::vector<aocommon::Image>& images, aocommon::Image& scratch,
+                 float scale);
 
   size_t Width() const { return _width; }
   size_t Height() const { return _height; }
@@ -39,7 +35,7 @@ class MultiScaleTransforms {
   static float KernelIntegratedValue(float scaleInPixels, size_t maxN,
                                      Shape shape) {
     size_t n;
-    Image kernel = MakeShapeFunction(scaleInPixels, n, maxN, shape);
+    aocommon::Image kernel = MakeShapeFunction(scaleInPixels, n, maxN, shape);
 
     float value = 0.0;
     for (float& x : kernel) value += x;
@@ -49,7 +45,7 @@ class MultiScaleTransforms {
 
   static float KernelPeakValue(double scaleInPixels, size_t maxN, Shape shape) {
     size_t n;
-    Image kernel = MakeShapeFunction(scaleInPixels, n, maxN, shape);
+    aocommon::Image kernel = MakeShapeFunction(scaleInPixels, n, maxN, shape);
     return kernel[n / 2 + (n / 2) * n];
   }
 
@@ -57,7 +53,7 @@ class MultiScaleTransforms {
                                 float scaleSizeInPixels, size_t x, size_t y,
                                 float gain, Shape shape) {
     size_t n;
-    Image kernel =
+    aocommon::Image kernel =
         MakeShapeFunction(scaleSizeInPixels, n, std::min(width, height), shape);
     int left;
     if (x > n / 2)
@@ -74,7 +70,7 @@ class MultiScaleTransforms {
     for (size_t yi = top; yi != bottom; ++yi) {
       float* imagePtr = &image[yi * width];
       const float* kernelPtr =
-          &kernel.data()[(yi + n / 2 - y) * n + left + n / 2 - x];
+          &kernel.Data()[(yi + n / 2 - y) * n + left + n / 2 - x];
       for (size_t xi = left; xi != right; ++xi) {
         imagePtr[xi] += *kernelPtr * gain;
         ++kernelPtr;
@@ -82,8 +78,8 @@ class MultiScaleTransforms {
     }
   }
 
-  static Image MakeShapeFunction(float scaleSizeInPixels, size_t& n,
-                                 size_t maxN, Shape shape) {
+  static aocommon::Image MakeShapeFunction(float scaleSizeInPixels, size_t& n,
+                                           size_t maxN, Shape shape) {
     switch (shape) {
       default:
       case TaperedQuadraticShape:
@@ -93,7 +89,7 @@ class MultiScaleTransforms {
     }
   }
 
-  Image MakeShapeFunction(float scaleSizeInPixels, size_t& n) {
+  aocommon::Image MakeShapeFunction(float scaleSizeInPixels, size_t& n) {
     return MakeShapeFunction(scaleSizeInPixels, n, std::min(_width, _height),
                              _shape);
   }
@@ -105,7 +101,6 @@ class MultiScaleTransforms {
   void SetThreadCount(size_t threadCount) { _threadCount = threadCount; }
 
  private:
-  class FFTWManager& _fftwManager;
   size_t _width, _height;
   enum Shape _shape;
   size_t _threadCount;
@@ -114,16 +109,16 @@ class MultiScaleTransforms {
     return size_t(ceil(scaleInPixels * 0.5) * 2.0) + 1;
   }
 
-  static Image makeTaperedQuadraticShapeFunction(double scaleSizeInPixels,
-                                                 size_t& n) {
+  static aocommon::Image makeTaperedQuadraticShapeFunction(
+      double scaleSizeInPixels, size_t& n) {
     n = taperedQuadraticKernelSize(scaleSizeInPixels);
-    Image output(n, n);
+    aocommon::Image output(n, n);
     taperedQuadraticShapeFunction(n, output, scaleSizeInPixels);
     return output;
   }
 
-  static Image makeGaussianFunction(double scaleSizeInPixels, size_t& n,
-                                    size_t maxN) {
+  static aocommon::Image makeGaussianFunction(double scaleSizeInPixels,
+                                              size_t& n, size_t maxN) {
     float sigma = GaussianSigma(scaleSizeInPixels);
 
     n = int(ceil(sigma * 12.0 / 2.0)) * 2 + 1;  // bounding box of 12 sigma
@@ -136,11 +131,11 @@ class MultiScaleTransforms {
       sigma = 1.0;
       n = 1;
     }
-    Image output(n, n);
+    aocommon::Image output(n, n);
     const float mu = int(n / 2);
     const float twoSigmaSquared = 2.0 * sigma * sigma;
     float sum = 0.0;
-    float* outputPtr = output.data();
+    float* outputPtr = output.Data();
     aocommon::UVector<float> gaus(n);
     for (int i = 0; i != int(n); ++i) {
       float vI = float(i) - mu;
@@ -158,13 +153,13 @@ class MultiScaleTransforms {
     return output;
   }
 
-  static void taperedQuadraticShapeFunction(size_t n, Image& output2d,
+  static void taperedQuadraticShapeFunction(size_t n, aocommon::Image& output2d,
                                             double scaleSizeInPixels) {
     if (scaleSizeInPixels == 0.0)
       output2d[0] = 1.0;
     else {
       float sum = 0.0;
-      float* outputPtr = output2d.data();
+      float* outputPtr = output2d.Data();
       for (int y = 0; y != int(n); ++y) {
         float dy = y - 0.5 * (n - 1);
         float dydy = dy * dy;
