@@ -4,65 +4,62 @@
 #include "../deconvolution/peakfinder.h"
 #include "../deconvolution/simpleclean.h"
 
+#include <aocommon/image.h>
+
+#include <memory>
+
+using aocommon::Image;
+
 ThreadedDeconvolutionTools::ThreadedDeconvolutionTools(size_t threadCount)
     : _taskLanes(threadCount),
       _resultLanes(threadCount),
       _threadCount(threadCount) {
   for (size_t i = 0; i != _threadCount; ++i) {
-    _taskLanes[i] = new aocommon::Lane<ThreadTask*>(1);
-    _resultLanes[i] = new aocommon::Lane<ThreadResult*>(1);
+    _taskLanes[i].resize(1);
+    _resultLanes[i].resize(1);
     _threadGroup.emplace_back(&ThreadedDeconvolutionTools::threadFunc, this,
-                              _taskLanes[i], _resultLanes[i]);
+                              &_taskLanes[i], &_resultLanes[i]);
   }
 }
 
 ThreadedDeconvolutionTools::~ThreadedDeconvolutionTools() {
   for (size_t i = 0; i != _threadCount; ++i) {
-    _taskLanes[i]->write_end();
+    _taskLanes[i].write_end();
   }
 
   for (std::thread& t : _threadGroup) t.join();
-
-  for (size_t i = 0; i != _threadCount; ++i) {
-    delete _taskLanes[i];
-    delete _resultLanes[i];
-  }
 }
 
-void ThreadedDeconvolutionTools::SubtractImage(float* image, const float* psf,
-                                               size_t width, size_t height,
+void ThreadedDeconvolutionTools::SubtractImage(float* image,
+                                               const aocommon::Image& psf,
                                                size_t x, size_t y,
                                                float factor) {
   for (size_t thr = 0; thr != _threadCount; ++thr) {
-    SubtractionTask* task = new SubtractionTask();
+    auto task = std::make_unique<SubtractionTask>();
     task->image = image;
-    task->psf = psf;
-    task->width = width;
-    task->height = height;
+    task->psf = &psf;
     task->x = x;
     task->y = y;
     task->factor = factor;
-    task->startY = height * thr / _threadCount;
-    task->endY = height * (thr + 1) / _threadCount;
+    task->startY = psf.Height() * thr / _threadCount;
+    task->endY = psf.Height() * (thr + 1) / _threadCount;
     if (thr == _threadCount - 1) {
-      ThreadResult* result = (*task)();
-      delete result;
+      (*task)();
     } else {
-      _taskLanes[thr]->write(task);
+      _taskLanes[thr].write(std::move(task));
     }
   }
   for (size_t thr = 0; thr != _threadCount - 1; ++thr) {
-    ThreadResult* result = 0;
-    _resultLanes[thr]->read(result);
-    delete result;
+    std::unique_ptr<ThreadResult> result;
+    _resultLanes[thr].read(result);
   }
 }
 
-ThreadedDeconvolutionTools::ThreadResult*
+std::unique_ptr<ThreadedDeconvolutionTools::ThreadResult>
 ThreadedDeconvolutionTools::SubtractionTask::operator()() {
-  SimpleClean::PartialSubtractImage(image, psf, width, height, x, y, factor,
-                                    startY, endY);
-  return 0;
+  SimpleClean::PartialSubtractImage(image, psf->Data(), psf->Width(),
+                                    psf->Height(), x, y, factor, startY, endY);
+  return std::unique_ptr<ThreadedDeconvolutionTools::ThreadResult>();
 }
 
 void ThreadedDeconvolutionTools::FindMultiScalePeak(
@@ -79,19 +76,21 @@ void ThreadedDeconvolutionTools::FindMultiScalePeak(
   results.resize(scales.size());
 
   size_t size = std::min(scales.size(), _threadCount);
-  std::unique_ptr<Image::Ptr[]> imageData(new Image::Ptr[size]);
-  std::unique_ptr<Image::Ptr[]> scratchData(new Image::Ptr[size]);
+  std::vector<Image> imageData;
+  std::vector<Image> scratchData;
+  imageData.reserve(size);
+  scratchData.reserve(size);
   for (size_t i = 0; i != size; ++i) {
-    imageData[i] = Image::Make(msTransforms->Width(), msTransforms->Height());
-    scratchData[i] = Image::Make(msTransforms->Width(), msTransforms->Height());
+    imageData.emplace_back(msTransforms->Width(), msTransforms->Height());
+    scratchData.emplace_back(msTransforms->Width(), msTransforms->Height());
   }
 
   while (imageIndex < scales.size()) {
-    FindMultiScalePeakTask* task = new FindMultiScalePeakTask();
+    std::unique_ptr<FindMultiScalePeakTask> task(new FindMultiScalePeakTask());
     task->msTransforms = msTransforms;
-    (*imageData[nextThread]) = image;
-    task->image = imageData[nextThread].get();
-    task->scratch = scratchData[nextThread].get();
+    imageData[nextThread] = image;
+    task->image = &imageData[nextThread];
+    task->scratch = &scratchData[nextThread];
     task->scale = scales[imageIndex];
     task->allowNegativeComponents = allowNegativeComponents;
     if (scaleMasks.empty())
@@ -101,24 +100,23 @@ void ThreadedDeconvolutionTools::FindMultiScalePeak(
     task->borderRatio = borderRatio;
     task->calculateRMS = calculateRMS;
     task->rmsFactorImage = &rmsFactorImage;
-    _taskLanes[nextThread]->write(task);
+    _taskLanes[nextThread].write(std::move(task));
 
     ++nextThread;
     if (nextThread == _threadCount) {
       for (size_t thr = 0; thr != nextThread; ++thr) {
-        ThreadResult* result = nullptr;
-        _resultLanes[thr]->read(result);
+        std::unique_ptr<ThreadResult> result;
+        _resultLanes[thr].read(result);
         results[resultIndex].normalizedValue =
-            static_cast<FindMultiScalePeakResult*>(result)->normalizedValue;
+            static_cast<FindMultiScalePeakResult&>(*result).normalizedValue;
         results[resultIndex].unnormalizedValue =
-            static_cast<FindMultiScalePeakResult*>(result)->unnormalizedValue;
+            static_cast<FindMultiScalePeakResult&>(*result).unnormalizedValue;
         results[resultIndex].x =
-            static_cast<FindMultiScalePeakResult*>(result)->x;
+            static_cast<FindMultiScalePeakResult&>(*result).x;
         results[resultIndex].y =
-            static_cast<FindMultiScalePeakResult*>(result)->y;
+            static_cast<FindMultiScalePeakResult&>(*result).y;
         results[resultIndex].rms =
-            static_cast<FindMultiScalePeakResult*>(result)->rms;
-        delete result;
+            static_cast<FindMultiScalePeakResult&>(*result).rms;
         ++resultIndex;
       }
       nextThread = 0;
@@ -126,58 +124,59 @@ void ThreadedDeconvolutionTools::FindMultiScalePeak(
     ++imageIndex;
   }
   for (size_t thr = 0; thr != nextThread; ++thr) {
-    ThreadResult* result = nullptr;
-    _resultLanes[thr]->read(result);
+    std::unique_ptr<ThreadResult> result;
+    _resultLanes[thr].read(result);
     results[resultIndex].unnormalizedValue =
-        static_cast<FindMultiScalePeakResult*>(result)->unnormalizedValue;
+        static_cast<FindMultiScalePeakResult&>(*result).unnormalizedValue;
     results[resultIndex].normalizedValue =
-        static_cast<FindMultiScalePeakResult*>(result)->normalizedValue;
-    results[resultIndex].x = static_cast<FindMultiScalePeakResult*>(result)->x;
-    results[resultIndex].y = static_cast<FindMultiScalePeakResult*>(result)->y;
+        static_cast<FindMultiScalePeakResult&>(*result).normalizedValue;
+    results[resultIndex].x = static_cast<FindMultiScalePeakResult&>(*result).x;
+    results[resultIndex].y = static_cast<FindMultiScalePeakResult&>(*result).y;
     results[resultIndex].rms =
-        static_cast<FindMultiScalePeakResult*>(result)->rms;
-    delete result;
+        static_cast<FindMultiScalePeakResult&>(*result).rms;
     ++resultIndex;
   }
 }
 
-ThreadedDeconvolutionTools::ThreadResult*
+std::unique_ptr<ThreadedDeconvolutionTools::ThreadResult>
 ThreadedDeconvolutionTools::FindMultiScalePeakTask::operator()() {
   msTransforms->Transform(*image, *scratch, scale);
-  const size_t width = msTransforms->Width(), height = msTransforms->Height(),
-               scaleBorder = size_t(ceil(scale * 0.5)),
-               horBorderSize =
-                   std::max<size_t>(round(width * borderRatio), scaleBorder),
-               vertBorderSize =
-                   std::max<size_t>(round(height * borderRatio), scaleBorder);
-  FindMultiScalePeakResult* result = new FindMultiScalePeakResult();
+  const size_t width = msTransforms->Width();
+  const size_t height = msTransforms->Height();
+  const size_t scaleBorder = size_t(std::ceil(scale * 0.5));
+  const size_t horBorderSize =
+      std::max<size_t>(std::round(width * borderRatio), scaleBorder);
+  const size_t vertBorderSize =
+      std::max<size_t>(std::round(height * borderRatio), scaleBorder);
+  std::unique_ptr<FindMultiScalePeakResult> result(
+      new FindMultiScalePeakResult());
   if (calculateRMS)
     result->rms = RMS(*image, width * height);
   else
     result->rms = -1.0;
-  if (rmsFactorImage->empty()) {
-    if (mask == 0)
+  if (rmsFactorImage->Empty()) {
+    if (mask == nullptr)
       result->unnormalizedValue = PeakFinder::Find(
-          image->data(), width, height, result->x, result->y,
+          image->Data(), width, height, result->x, result->y,
           allowNegativeComponents, 0, height, horBorderSize, vertBorderSize);
     else
       result->unnormalizedValue =
-          PeakFinder::FindWithMask(image->data(), width, height, result->x,
+          PeakFinder::FindWithMask(image->Data(), width, height, result->x,
                                    result->y, allowNegativeComponents, 0,
                                    height, mask, horBorderSize, vertBorderSize);
 
     result->normalizedValue = result->unnormalizedValue;
   } else {
-    for (size_t i = 0; i != rmsFactorImage->size(); ++i)
+    for (size_t i = 0; i != rmsFactorImage->Size(); ++i)
       (*scratch)[i] = (*image)[i] * (*rmsFactorImage)[i];
 
-    if (mask == 0)
+    if (mask == nullptr)
       result->unnormalizedValue = PeakFinder::Find(
-          scratch->data(), width, height, result->x, result->y,
+          scratch->Data(), width, height, result->x, result->y,
           allowNegativeComponents, 0, height, horBorderSize, vertBorderSize);
     else
       result->unnormalizedValue =
-          PeakFinder::FindWithMask(scratch->data(), width, height, result->x,
+          PeakFinder::FindWithMask(scratch->Data(), width, height, result->x,
                                    result->y, allowNegativeComponents, 0,
                                    height, mask, horBorderSize, vertBorderSize);
 
@@ -186,7 +185,7 @@ ThreadedDeconvolutionTools::FindMultiScalePeakTask::operator()() {
           (*result->unnormalizedValue) /
           (*rmsFactorImage)[result->x + result->y * width];
     } else {
-      result->normalizedValue = boost::optional<float>();
+      result->normalizedValue.reset();
     }
   }
   return result;
