@@ -3,6 +3,24 @@
 #include <aocommon/io/serialostream.h>
 #include <aocommon/io/serialistream.h>
 
+#include "../idg/averagebeam.h"
+
+GriddingResult::GriddingResult()
+    : startTime(0.0),
+      beamSize(0.0),
+      imageWeight(0.0),
+      normalizationFactor(0.0),
+      actualWGridSize(0),
+      griddedVisibilityCount(0),
+      effectiveGriddedVisibilityCount(0),
+      visibilityWeightSum(0),
+      cache() {}
+
+GriddingResult::GriddingResult(GriddingResult&& source) noexcept = default;
+GriddingResult::~GriddingResult() = default;
+GriddingResult& GriddingResult::operator=(GriddingResult&& rhs) noexcept =
+    default;
+
 void GriddingResult::Serialize(aocommon::SerialOStream& stream) const {
   stream.ObjectVector(images)
       .Double(beamSize)
@@ -12,9 +30,8 @@ void GriddingResult::Serialize(aocommon::SerialOStream& stream) const {
       .UInt64(griddedVisibilityCount)
       .Double(effectiveGriddedVisibilityCount)
       .Double(visibilityWeightSum)
-      .UInt64(actualInversionWidth)
-      .UInt64(actualInversionHeight)
-      .Ptr(cache);
+      .Ptr(cache)
+      .Ptr(averageBeam);
 }
 
 void GriddingResult::Unserialize(aocommon::SerialIStream& stream) {
@@ -26,7 +43,6 @@ void GriddingResult::Unserialize(aocommon::SerialIStream& stream) {
       .UInt64(griddedVisibilityCount)
       .Double(effectiveGriddedVisibilityCount)
       .Double(visibilityWeightSum)
-      .UInt64(actualInversionWidth)
-      .UInt64(actualInversionHeight)
-      .Ptr(cache);
+      .Ptr(cache)
+      .Ptr(averageBeam);
 }
