@@ -2,9 +2,9 @@
 #define GRIDDING_TASK_H
 
 #include <aocommon/io/serialstreamfwd.h>
+#include <aocommon/image.h>
 #include <aocommon/polarization.h>
 
-#include "../structures/image.h"
 #include "../structures/imageweights.h"
 #include "../structures/observationinfo.h"
 
@@ -12,6 +12,8 @@
 
 #include "metadatacache.h"
 
+class AverageBeam;
+
 namespace schaapcommon {
 namespace facets {
 class Facet;
@@ -20,12 +22,20 @@ class Facet;
 
 class GriddingTask {
  public:
+  GriddingTask();
+  GriddingTask(const GriddingTask&) = delete;
+  GriddingTask(GriddingTask&& source) noexcept;
+  ~GriddingTask() noexcept;
+  GriddingTask& operator=(const GriddingTask& source) = delete;
+  GriddingTask& operator=(GriddingTask&& source) noexcept;
+
   enum Operation { Invert, Predict } operation;
   bool imagePSF;
   bool subtractModel;
   aocommon::PolarizationEnum polarization;
   bool verbose;
   std::unique_ptr<MetaDataCache> cache;
+  std::unique_ptr<AverageBeam> averageBeam;
   bool storeImagingWeights;
 
   std::shared_ptr<ImageWeights> imageWeights;
@@ -35,7 +45,7 @@ class GriddingTask {
    * Images for prediction. See the documentation of
    * @ref GriddingResult::images for an explanation of why this is a vector.
    */
-  std::vector<Image> modelImages;
+  std::vector<aocommon::Image> modelImages;
   ObservationInfo observationInfo;
 
   std::shared_ptr<schaapcommon::facets::Facet> facet;
