@@ -6,18 +6,22 @@
 #include <memory>
 #include <vector>
 
-#include "../idg/averagebeam.h"
-
 struct MetaDataCache {
   struct Entry {
     double minW, maxW, maxWWithFlags, maxBaselineUVW, maxBaselineInM,
         integrationTime;
   };
   std::vector<Entry> msDataVector;
-  std::unique_ptr<class AverageBeam> averageBeam;
-  float beamSum = 0.0;
-  float h5Sum = 0.0;
-
+  /** @{
+   *  These variables are incremented with a comparatively small value for each
+   * gridded visibility, hence a long double is used to accomodate sufficient
+   * precision.
+   */
+  long double h5Sum = 0.0;
+  long double correctionSum = 0.0;
+  /**
+   * @}
+   */
   void Serialize(aocommon::SerialOStream& stream) const;
   void Unserialize(aocommon::SerialIStream& stream);
 };
