@@ -3,14 +3,16 @@
 
 #include "../main/settings.h"
 
-#include "../io/logger.h"
-
 #include "../structures/imageweights.h"
 
 #include "../msproviders/msdatadescription.h"
 
 #include "../io/findmwacoefffile.h"
 
+#include <aocommon/image.h>
+#include <aocommon/logger.h>
+#include <aocommon/multibanddata.h>
+
 #include <schaapcommon/facets/facetimage.h>
 
 #include <boost/filesystem/operations.hpp>
@@ -28,6 +30,11 @@ using everybeam::ATermSettings;
 using everybeam::aterms::ATermConfig;
 #endif
 
+using aocommon::Image;
+using aocommon::Logger;
+using aocommon::Polarization;
+using aocommon::PolarizationEnum;
+
 namespace {
 void writeBeamImages(const ImageFilename& imageName,
                      const PrimaryBeamImageSet& beamImages,
@@ -45,7 +52,7 @@ void writeBeamImages(const ImageFilename& imageName,
   for (size_t i = 0; i != beamImages.NImages(); ++i) {
     writer.Write(
         imageName.GetBeamPrefix(settings) + "-" + std::to_string(i) + ".fits",
-        beamImages[i].data());
+        beamImages[i].Data());
   }
 }
 }  // namespace
@@ -55,16 +62,21 @@ PrimaryBeam::PrimaryBeam(const Settings& settings)
       _phaseCentreRA(0.0),
       _phaseCentreDec(0.0),
       _phaseCentreDL(0.0),
-      _phaseCentreDM(0.0) {
-  // Undersampling factor (default 8) and beam update time (default 1800s), see
-  // settings.h
-  _undersample = _settings.primaryBeamUndersampling;
-  _secondsBeforeBeamUpdate = _settings.primaryBeamUpdateTime;
+      _phaseCentreDM(0.0),
+      _undersample(computeUndersamplingFactor(settings)),
+      _secondsBeforeBeamUpdate(settings.primaryBeamUpdateTime)
+#ifdef HAVE_EVERYBEAM
+      ,
+      _beamMode(everybeam::ParseBeamMode(settings.beamMode)),
+      _beamNormalisationMode(
+          everybeam::ParseBeamNormalisationMode(settings.beamNormalisationMode))
+#endif
+{
 }
 
 PrimaryBeam::~PrimaryBeam() {}
 
-void PrimaryBeam::AddMS(std::unique_ptr<class MSDataDescription> description) {
+void PrimaryBeam::AddMS(std::unique_ptr<MSDataDescription> description) {
   _msList.emplace_back(std::move(description));
 }
 
@@ -80,8 +92,7 @@ void PrimaryBeam::CorrectImages(
         _settings.trimmedImageWidth, _settings.trimmedImageHeight, 1);
 
     if (_settings.polarizations ==
-        std::set<aocommon::PolarizationEnum>{aocommon::Polarization::XX,
-                                             aocommon::Polarization::YY}) {
+        std::set<PolarizationEnum>{Polarization::XX, Polarization::YY}) {
       // FIXME: to be implemented
       // This should multiply the 16 images (representing a Hermitian 4x4
       // matrix) with the diagonal 4x4 matrix with diagonal entries [1/sqrt(mx*
@@ -93,7 +104,7 @@ void PrimaryBeam::CorrectImages(
           "effects is not yet implemented.");
     } else {
       for (size_t i = 0; i != beamImages.NImages(); ++i) {
-        std::vector<float*> imagePtr{beamImages[i].data()};
+        std::vector<float*> imagePtr{beamImages[i].Data()};
         for (const ImagingTableEntry& entry : table) {
           const float m = metaCache.at(entry.index)->h5Sum / entry.imageWeight;
           const float factor = 1.0 / std::sqrt(m);
@@ -111,9 +122,16 @@ void PrimaryBeam::CorrectImages(
   }
 
   if (_settings.polarizations.size() == 1 || filenameKind == "psf") {
-    aocommon::PolarizationEnum pol = *_settings.polarizations.begin();
+    PolarizationEnum pol = *_settings.polarizations.begin();
 
-    if (pol == aocommon::Polarization::StokesI) {
+    const bool pseudo_correction =
+        _settings.polarizations.size() == 1 &&
+        (pol == Polarization::RR || pol == Polarization::LL);
+    if (pseudo_correction)
+      Logger::Warn
+          << "Warning: not all polarizations are available for full beam "
+             "correction, performing pseudo-Stokes I beam correction.\n";
+    if (pol == Polarization::StokesI || pseudo_correction) {
       ImageFilename stokesIName(imageName);
       stokesIName.SetPolarization(pol);
       std::string prefix;
@@ -123,10 +141,10 @@ void PrimaryBeam::CorrectImages(
         prefix = stokesIName.GetPrefix(_settings);
       aocommon::FitsReader reader(prefix + "-" + filenameKind + ".fits");
       Image image(reader.ImageWidth(), reader.ImageHeight());
-      reader.Read(image.data());
+      reader.Read(image.Data());
 
-      beamImages.ApplyStokesI(image.data(), _settings.primaryBeamLimit);
-      writer.Write(prefix + "-" + filenameKind + "-pb.fits", image.data());
+      beamImages.ApplyStokesI(image.Data(), _settings.primaryBeamLimit);
+      writer.Write(prefix + "-" + filenameKind + "-pb.fits", image.Data());
     } else {
       throw std::runtime_error(
           "Primary beam correction is requested, but this is not supported "
@@ -147,10 +165,10 @@ void PrimaryBeam::CorrectImages(
       reader.reset(new aocommon::FitsReader(name.GetPrefix(_settings) + "-" +
                                             filenameKind + ".fits"));
       images[polIndex] = Image(reader->ImageWidth(), reader->ImageHeight());
-      reader->Read(images[polIndex].data());
+      reader->Read(images[polIndex].Data());
     }
 
-    float* imagePtrs[2] = {images[0].data(), images[1].data()};
+    float* imagePtrs[2] = {images[0].Data(), images[1].Data()};
     beamImages.ApplyDiagonal(imagePtrs, _settings.primaryBeamLimit);
 
     for (size_t polIndex = 0; polIndex != 2; ++polIndex) {
@@ -161,7 +179,7 @@ void PrimaryBeam::CorrectImages(
       name.SetPolarization(pol);
       writer.SetPolarization(pol);
       writer.Write(name.GetPrefix(_settings) + "-" + filenameKind + "-pb.fits",
-                   images[polIndex].data());
+                   images[polIndex].Data());
     }
   } else if (aocommon::Polarization::HasFullStokesPolarization(
                  _settings.polarizations)) {
@@ -175,11 +193,11 @@ void PrimaryBeam::CorrectImages(
       reader.reset(new aocommon::FitsReader(name.GetPrefix(_settings) + "-" +
                                             filenameKind + ".fits"));
       images[polIndex] = Image(reader->ImageWidth(), reader->ImageHeight());
-      reader->Read(images[polIndex].data());
+      reader->Read(images[polIndex].Data());
     }
 
-    float* imagePtrs[4] = {images[0].data(), images[1].data(), images[2].data(),
-                           images[3].data()};
+    float* imagePtrs[4] = {images[0].Data(), images[1].Data(), images[2].Data(),
+                           images[3].Data()};
     beamImages.ApplyFullStokes(imagePtrs, _settings.primaryBeamLimit);
     for (size_t polIndex = 0; polIndex != 4; ++polIndex) {
       aocommon::PolarizationEnum pol =
@@ -188,7 +206,7 @@ void PrimaryBeam::CorrectImages(
       name.SetPolarization(pol);
       writer.SetPolarization(pol);
       writer.Write(name.GetPrefix(_settings) + "-" + filenameKind + "-pb.fits",
-                   images[polIndex].data());
+                   images[polIndex].Data());
     }
   } else {
     throw std::runtime_error(
@@ -211,7 +229,7 @@ PrimaryBeamImageSet PrimaryBeam::load(const ImageFilename& imageName,
     ImageFilename polName(imageName);
     polName.SetPolarization(aocommon::Polarization::StokesI);
     aocommon::FitsReader reader(polName.GetBeamPrefix(settings) + ".fits");
-    reader.Read(beamImages[0].data());
+    reader.Read(beamImages[0].Data());
     for (size_t i = 0;
          i != settings.trimmedImageWidth * settings.trimmedImageHeight; ++i)
       beamImages[0][i] = std::sqrt(beamImages[0][i]);
@@ -219,9 +237,9 @@ PrimaryBeamImageSet PrimaryBeam::load(const ImageFilename& imageName,
     // Copy zero entry to images of the diagonal
     std::array<size_t, 3> diagonal_entries = {3, 8, 15};
     for (size_t entry : diagonal_entries) {
-      std::copy_n(beamImages[0].data(),
+      std::copy_n(beamImages[0].Data(),
                   settings.trimmedImageWidth * settings.trimmedImageHeight,
-                  beamImages[entry].data());
+                  beamImages[entry].Data());
     }
     return beamImages;
   } else {
@@ -230,7 +248,7 @@ PrimaryBeamImageSet PrimaryBeam::load(const ImageFilename& imageName,
     for (size_t i = 0; i != beamImages.NImages(); ++i) {
       aocommon::FitsReader reader(imageName.GetBeamPrefix(settings) + "-" +
                                   std::to_string(i) + ".fits");
-      reader.Read(beamImages[i].data());
+      reader.Read(beamImages[i].Data());
     }
     return beamImages;
     // }
@@ -308,8 +326,8 @@ PrimaryBeamImageSet PrimaryBeam::MakeImage(
                                                        _phaseCentreDL,
                                                        _phaseCentreDM};
 
-  aocommon::UVector<double> buffer_total(width * height * beamImages.NImages(),
-                                         0);
+  aocommon::UVector<float> buffer_total(width * height * beamImages.NImages(),
+                                        0);
   double ms_weight_sum = 0;
   for (const MSProviderInfo& msProviderInfo : _msProviders) {
     // TODO: channelFrequency calculation might be telescope specific?
@@ -318,18 +336,19 @@ PrimaryBeamImageSet PrimaryBeam::MakeImage(
     const MSSelection& selection = *msProviderInfo.selection;
 
     SynchronizedMS ms = msProviderInfo.provider->MS();
-    MultiBandData band(ms->spectralWindow(), ms->dataDescription());
+    aocommon::MultiBandData band(ms->spectralWindow(), ms->dataDescription());
     ms.Reset();
     double centralFrequency = 0.0;
     for (size_t dataDescId = 0; dataDescId != band.DataDescCount();
          ++dataDescId) {
-      BandData subBand(band[dataDescId], selection.ChannelRangeStart(),
-                       selection.ChannelRangeEnd());
+      aocommon::BandData subBand(band[dataDescId],
+                                 selection.ChannelRangeStart(),
+                                 selection.ChannelRangeEnd());
       centralFrequency += subBand.CentreFrequency();
     }
     centralFrequency /= msInfo.bands.size();
 
-    aocommon::UVector<double> buffer(width * height * beamImages.NImages(), 0);
+    aocommon::UVector<float> buffer(width * height * beamImages.NImages(), 0);
     const double ms_weight =
         MakeBeamForMS(buffer, *msProviderInfo.provider, selection,
                       *imageWeights, coordinateSystem, centralFrequency);
@@ -353,7 +372,7 @@ PrimaryBeamImageSet PrimaryBeam::MakeImage(
 }
 
 double PrimaryBeam::MakeBeamForMS(
-    aocommon::UVector<double>& buffer, MSProvider& msProvider,
+    aocommon::UVector<float>& buffer, MSProvider& msProvider,
     const MSSelection& selection, const ImageWeights& imageWeights,
     const everybeam::coords::CoordinateSystem& coordinateSystem,
     double centralFrequency) {
@@ -383,8 +402,8 @@ double PrimaryBeam::MakeBeamForMS(
   aterm_settings.data_column_name = _settings.dataColumnName;
   everybeam::Options options = ATermConfig::ConvertToEBOptions(
       *ms, aterm_settings, frequencyInterpolation,
-      _settings.useDifferentialLofarBeam, useChannelFrequency,
-      elementResponseModel);
+      _settings.beamNormalisationMode, useChannelFrequency,
+      elementResponseModel, _settings.beamMode);
 
   // Make telescope
   std::unique_ptr<everybeam::telescope::Telescope> telescope =
@@ -436,8 +455,11 @@ double PrimaryBeam::MakeBeamForMS(
       break;
     case everybeam::TelescopeType::kVLATelescope:
     case everybeam::TelescopeType::kATCATelescope:
-      if (telescope_type == everybeam::TelescopeType::kATCATelescope) {
-        Logger::Warn << "ATCA primary beam correction not yet tested!\n";
+    case everybeam::TelescopeType::kGMRTTelescope:
+      if (telescope_type == everybeam::TelescopeType::kATCATelescope ||
+          telescope_type == everybeam::TelescopeType::kGMRTTelescope) {
+        Logger::Warn << "Warning: ATCA and GMRT primary beam corrections have "
+                        "not yet been tested!\n";
       }
       // Assign weight of 1 for these "time independent" telescopes
       ms_weight = 1.0;
@@ -458,9 +480,9 @@ double PrimaryBeam::MakeBeamForMS(
       telescope->GetGriddedResponse(coordinateSystem);
 
   // Note: field id is hard coded to 0
-  grid_response->CalculateIntegratedResponse(buffer.data(), time_array,
-                                             centralFrequency, 0, _undersample,
-                                             baseline_weights);
+  grid_response->IntegratedResponse(_beamMode, buffer.data(), time_array,
+                                    centralFrequency, 0, _undersample,
+                                    baseline_weights);
   return ms_weight;
 }
 
@@ -473,7 +495,8 @@ void PrimaryBeam::CalculateStationWeights(const ImageWeights& imageWeights,
   casacore::MSAntenna antTable(ms->antenna());
   aocommon::UVector<double> perAntennaWeights(antTable.nrow(), 0.0);
 
-  MultiBandData multiband(ms->spectralWindow(), ms->dataDescription());
+  aocommon::MultiBandData multiband(ms->spectralWindow(),
+                                    ms->dataDescription());
   size_t channelCount =
       selection.ChannelRangeEnd() - selection.ChannelRangeStart();
   size_t polarizationCount =
@@ -481,11 +504,11 @@ void PrimaryBeam::CalculateStationWeights(const ImageWeights& imageWeights,
                                                                           : 1;
   aocommon::UVector<float> weightArr(channelCount * polarizationCount);
   std::unique_ptr<MSReader> msReader = msProvider.MakeReader();
+  const aocommon::BandData band = multiband[msProvider.DataDescId()];
   while (msReader->CurrentRowAvailable()) {
     MSProvider::MetaData metaData;
     msReader->ReadMeta(metaData);
     if (metaData.time >= endTime) break;
-    const BandData& band(multiband[metaData.dataDescId]);
     msReader->ReadWeights(weightArr.data());
 
     for (size_t ch = 0; ch != channelCount; ++ch) {
@@ -537,3 +560,10 @@ std::tuple<double, double, size_t> PrimaryBeam::GetTimeInfo(
   return std::make_tuple(startTime, endTime, intervalCount);
 }
 #endif  // HAVE_EVERYBEAM
+
+size_t PrimaryBeam::computeUndersamplingFactor(const Settings& settings) {
+  return std::max(
+      std::min(settings.trimmedImageWidth / settings.primaryBeamGridSize,
+               settings.trimmedImageHeight / settings.primaryBeamGridSize),
+      (size_t)1);
+}
