@@ -1,7 +1,7 @@
 #ifndef APPLICATION_H
 #define APPLICATION_H
 
-#include "../io/logger.h"
+#include <aocommon/logger.h>
 
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -11,7 +11,8 @@
 class Application {
  public:
   static void Run(const std::string& commandLine) {
-    Logger::Info << "Running: " << commandLine << '\n';
+    aocommon::Logger::Info << "Running: " << commandLine << '\n';
+    const char* commandLineCStr = commandLine.c_str();
     int pid = vfork();
     switch (pid) {
       case -1:  // Error
@@ -19,7 +20,7 @@ class Application {
             "Could not vfork() new process for executing command line "
             "application");
       case 0:  // Child
-        execl("/bin/sh", "sh", "-c", commandLine.c_str(), NULL);
+        execl("/bin/sh", "sh", "-c", commandLineCStr, NULL);
         _exit(127);
     }
     // Wait for process to terminate
